@@ -1,40 +1,40 @@
 
 /*  A Bison parser, made from yacc.yy
- by  GNU Bison version 1.25
-  */
+    by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
 
-#define	DOCROOT	258
-#define	PORT	259
-#define	CGI	260
-#define	ALIAS	261
-#define	MIME	262
-#define	PIXDIR	263
-#define	NOT	264
-#define	EQ	265
-#define	NEQ	266
-#define	LEQ	267
-#define	GEQ	268
-#define	LE	269
-#define	GR	270
-#define	OR	271
-#define	AND	272
-#define	IN	273
-#define	EXIST	274
-#define	FIRST	275
-#define	RANDOM	276
-#define	MAX	277
-#define	MIN	278
-#define	WITH	279
-#define	BOOL	280
-#define	STRING	281
-#define	ID	282
-#define	NUM	283
-#define	FLOAT	284
+#define	DOCROOT	257
+#define	PORT	258
+#define	CGI	259
+#define	ALIAS	260
+#define	MIME	261
+#define	PIXDIR	262
+#define	NOT	263
+#define	EQ	264
+#define	NEQ	265
+#define	LEQ	266
+#define	GEQ	267
+#define	LE	268
+#define	GR	269
+#define	OR	270
+#define	AND	271
+#define	IN	272
+#define	EXIST	273
+#define	FIRST	274
+#define	RANDOM	275
+#define	MAX	276
+#define	MIN	277
+#define	WITH	278
+#define	BOOL	279
+#define	STRING	280
+#define	ID	281
+#define	NUM	282
+#define	FLOAT	283
 
 #line 1 "yacc.yy"
 
+
 #include <CORBA.h>
 
 extern "C" {
@@ -52,7 +52,7 @@ void initFlex( const char *s );
 int yylex();
 
 
-#line 19 "yacc.yy"
+#line 21 "yacc.yy"
 typedef union
 {
      char valb;
@@ -79,7 +79,7 @@ typedef union
 #define	YYFLAG		-32768
 #define	YYNTBASE	37
 
-#define YYTRANSLATE(x) ((unsigned)(x) <= 284 ? yytranslate[x] : 49)
+#define YYTRANSLATE(x) ((unsigned)(x) <= 283 ? yytranslate[x] : 49)
 
 static const char yytranslate[] = {     0,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
@@ -107,10 +107,10 @@ static const char yytranslate[] = {     0,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
-     2,     2,     2,     2,     2,     1,     2,     3,     4,     5,
-     6,     7,     8,     9,    10,    11,    12,    13,    14,    15,
-    16,    17,    18,    19,    20,    21,    22,    23,    24,    25,
-    26,    27,    28,    29
+     2,     2,     2,     2,     2,     1,     3,     4,     5,     6,
+     7,     8,     9,    10,    11,    12,    13,    14,    15,    16,
+    17,    18,    19,    20,    21,    22,    23,    24,    25,    26,
+    27,    28,    29
 };
 
 #if YYDEBUG != 0
@@ -140,10 +140,10 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-    73,    74,    75,    78,    79,    80,    81,    82,    85,    88,
-    89,    92,    93,    96,    97,    98,    99,   100,   101,   102,
-   105,   106,   109,   110,   113,   114,   115,   118,   119,   120,
-   123,   124,   127,   128,   129,   130,   131,   132,   133
+    75,    76,    77,    80,    81,    82,    83,    84,    87,    90,
+    91,    94,    95,    98,    99,   100,   101,   102,   103,   104,
+   107,   108,   111,   112,   115,   116,   117,   120,   121,   122,
+   125,   126,   129,   130,   131,   132,   133,   134,   135
 };
 #endif
 
@@ -227,7 +227,8 @@ static const short yycheck[] = {     9,
      0,    13,    32,    31,    40,     1
 };
 /* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
-#line 3 "/local/pkg/bison/1.25/share/bison.simple"
+#line 3 "/usr/lib/bison.simple"
+/* This file comes from bison-1.28.  */
 
 /* Skeleton output parser for bison,
    Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.
@@ -244,47 +245,67 @@ static const short yycheck[] = {     9,
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
 
 /* As a special exception, when this file is copied by Bison into a
    Bison output file, you may use that output file without restriction.
    This special exception was added by the Free Software Foundation
    in version 1.24 of Bison.  */
 
-#ifndef alloca
-#ifdef __GNUC__
-#define alloca __builtin_alloca
-#else /* not GNU C.  */
-#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi)
-#include <alloca.h>
-#else /* not sparc */
-#if defined (MSDOS) && !defined (__TURBOC__)
-#include <malloc.h>
-#else /* not MSDOS, or __TURBOC__ */
-#if defined(_AIX)
-#include <malloc.h>
- #pragma alloca
-#else /* not MSDOS, __TURBOC__, or _AIX */
-#ifdef __hpux
-#ifdef __cplusplus
-extern "C" {
-void *alloca (unsigned int);
-};
-#else /* not __cplusplus */
-void *alloca ();
-#endif /* not __cplusplus */
-#endif /* __hpux */
-#endif /* not _AIX */
-#endif /* not MSDOS, or __TURBOC__ */
-#endif /* not sparc.  */
-#endif /* not GNU C.  */
-#endif /* alloca not defined.  */
-
 /* This is the parser code that is written into each bison parser
   when the %semantic_parser declaration is not specified in the grammar.
   It was written by Richard Stallman by simplifying the hairy parser
   used when %semantic_parser is specified.  */
 
+#ifndef YYSTACK_USE_ALLOCA
+#ifdef alloca
+#define YYSTACK_USE_ALLOCA
+#else /* alloca not defined */
+#ifdef __GNUC__
+#define YYSTACK_USE_ALLOCA
+#define alloca __builtin_alloca
+#else /* not GNU C.  */
+#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi) || (defined (__sun) && defined (__i386))
+#define YYSTACK_USE_ALLOCA
+#include <alloca.h>
+#else /* not sparc */
+/* We think this test detects Watcom and Microsoft C.  */
+/* This used to test MSDOS, but that is a bad idea
+   since that symbol is in the user namespace.  */
+#if (defined (_MSDOS) || defined (_MSDOS_)) && !defined (__TURBOC__)
+#if 0 /* No need for malloc.h, which pollutes the namespace;
+	 instead, just don't use alloca.  */
+#include <malloc.h>
+#endif
+#else /* not MSDOS, or __TURBOC__ */
+#if defined(_AIX)
+/* I don't know what this was needed for, but it pollutes the namespace.
+   So I turned it off.   rms, 2 May 1997.  */
+/* #include <malloc.h>  */
+ #pragma alloca
+#define YYSTACK_USE_ALLOCA
+#else /* not MSDOS, or __TURBOC__, or _AIX */
+#if 0
+#ifdef __hpux /* haible@ilog.fr says this works for HPUX 9.05 and up,
+		 and on HPUX 10.  Eventually we can turn this on.  */
+#define YYSTACK_USE_ALLOCA
+#define alloca __builtin_alloca
+#endif /* __hpux */
+#endif
+#endif /* not _AIX */
+#endif /* not MSDOS, or __TURBOC__ */
+#endif /* not sparc */
+#endif /* not GNU C */
+#endif /* alloca not defined */
+#endif /* YYSTACK_USE_ALLOCA not defined */
+
+#ifdef YYSTACK_USE_ALLOCA
+#define YYSTACK_ALLOC alloca
+#else
+#define YYSTACK_ALLOC malloc
+#endif
+
 /* Note: there must be only one dollar sign in this file.
    It is replaced by the list of actions, each action
    as one case of the switch.  */
@@ -293,8 +314,8 @@ void *alloca ();
 #define yyclearin	(yychar = YYEMPTY)
 #define YYEMPTY		-2
 #define YYEOF		0
-#define YYACCEPT	return(0)
-#define YYABORT 	return(1)
+#define YYACCEPT	goto yyacceptlab
+#define YYABORT 	goto yyabortlab
 #define YYERROR		goto yyerrlab1
 /* Like YYERROR except do call yyerror.
    This remains here temporarily to ease the
@@ -375,12 +396,12 @@ int yydebug;			/*  nonzero means print parse trace	*/
 #ifndef YYMAXDEPTH
 #define YYMAXDEPTH 10000
 #endif
-
-/* Prevent warning if -Wstrict-prototypes.  */
-#ifdef __GNUC__
-int yyparse (void);
-#endif
 
+/* Define __yy_memcpy.  Note that the size argument
+   should be passed with type unsigned int, because that is what the non-GCC
+   definitions require.  With GCC, __builtin_memcpy takes an arg
+   of type size_t, but it can handle unsigned int.  */
+
 #if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
 #define __yy_memcpy(TO,FROM,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
 #else				/* not GNU C or C++ */
@@ -392,7 +413,7 @@ static void
 __yy_memcpy (to, from, count)
      char *to;
      char *from;
-     int count;
+     unsigned int count;
 {
   register char *f = from;
   register char *t = to;
@@ -407,10 +428,10 @@ __yy_memcpy (to, from, count)
 /* This is the most reliable way to avoid incompatibilities
    in available built-in functions on various systems.  */
 static void
-__yy_memcpy (char *to, char *from, int count)
+__yy_memcpy (char *to, char *from, unsigned int count)
 {
-  register char *f = from;
   register char *t = to;
+  register char *f = from;
   register int i = count;
 
   while (i-- > 0)
@@ -420,7 +441,7 @@ __yy_memcpy (char *to, char *from, int count)
 #endif
 #endif
 
-#line 196 "/local/pkg/bison/1.25/share/bison.simple"
+#line 217 "/usr/lib/bison.simple"
 
 /* The user can define YYPARSE_PARAM as the name of an argument to be passed
    into yyparse.  The argument should have type void *.
@@ -441,6 +462,15 @@ __yy_memcpy (char *to, char *from, int count)
 #define YYPARSE_PARAM_DECL
 #endif /* not YYPARSE_PARAM */
 
+/* Prevent warning if -Wstrict-prototypes.  */
+#ifdef __GNUC__
+#ifdef YYPARSE_PARAM
+int yyparse (void *);
+#else
+int yyparse (void);
+#endif
+#endif
+
 int
 yyparse(YYPARSE_PARAM_ARG)
      YYPARSE_PARAM_DECL
@@ -469,6 +499,7 @@ yyparse(YYPARSE_PARAM_ARG)
 #endif
 
   int yystacksize = YYINITDEPTH;
+  int yyfree_stacks = 0;
 
 #ifdef YYPURE
   int yychar;
@@ -553,18 +584,32 @@ yynewstate:
       if (yystacksize >= YYMAXDEPTH)
 	{
 	  yyerror("parser stack overflow");
+	  if (yyfree_stacks)
+	    {
+	      free (yyss);
+	      free (yyvs);
+#ifdef YYLSP_NEEDED
+	      free (yyls);
+#endif
+	    }
 	  return 2;
 	}
       yystacksize *= 2;
       if (yystacksize > YYMAXDEPTH)
 	yystacksize = YYMAXDEPTH;
-      yyss = (short *) alloca (yystacksize * sizeof (*yyssp));
-      __yy_memcpy ((char *)yyss, (char *)yyss1, size * sizeof (*yyssp));
-      yyvs = (YYSTYPE *) alloca (yystacksize * sizeof (*yyvsp));
-      __yy_memcpy ((char *)yyvs, (char *)yyvs1, size * sizeof (*yyvsp));
+#ifndef YYSTACK_USE_ALLOCA
+      yyfree_stacks = 1;
+#endif
+      yyss = (short *) YYSTACK_ALLOC (yystacksize * sizeof (*yyssp));
+      __yy_memcpy ((char *)yyss, (char *)yyss1,
+		   size * (unsigned int) sizeof (*yyssp));
+      yyvs = (YYSTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yyvsp));
+      __yy_memcpy ((char *)yyvs, (char *)yyvs1,
+		   size * (unsigned int) sizeof (*yyvsp));
 #ifdef YYLSP_NEEDED
-      yyls = (YYLTYPE *) alloca (yystacksize * sizeof (*yylsp));
-      __yy_memcpy ((char *)yyls, (char *)yyls1, size * sizeof (*yylsp));
+      yyls = (YYLTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yylsp));
+      __yy_memcpy ((char *)yyls, (char *)yyls1,
+		   size * (unsigned int) sizeof (*yylsp));
 #endif
 #endif /* no yyoverflow */
 
@@ -725,164 +770,164 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 73 "yacc.yy"
+#line 75 "yacc.yy"
 { setConstraintsParseTree( 0L ); ;
     break;}
 case 2:
-#line 74 "yacc.yy"
+#line 76 "yacc.yy"
 { setConstraintsParseTree( yyvsp[0].ptr ); ;
     break;}
 case 3:
-#line 75 "yacc.yy"
+#line 77 "yacc.yy"
 { ;
     break;}
 case 4:
-#line 78 "yacc.yy"
+#line 80 "yacc.yy"
 { setPreferencesParseTree( newWITH( yyvsp[0].ptr ) ); ;
     break;}
 case 5:
-#line 79 "yacc.yy"
+#line 81 "yacc.yy"
 { setPreferencesParseTree( newMAX( yyvsp[0].ptr ) ); ;
     break;}
 case 6:
-#line 80 "yacc.yy"
+#line 82 "yacc.yy"
 { setPreferencesParseTree( newMIN( yyvsp[0].ptr ) ); ;
     break;}
 case 7:
-#line 81 "yacc.yy"
+#line 83 "yacc.yy"
 { setPreferencesParseTree( newFIRST() ); ;
     break;}
 case 8:
-#line 82 "yacc.yy"
+#line 84 "yacc.yy"
 { setPreferencesParseTree( newRANDOM() ); ;
     break;}
 case 9:
-#line 85 "yacc.yy"
+#line 87 "yacc.yy"
 { yyval.ptr = yyvsp[0].ptr; ;
     break;}
 case 10:
-#line 88 "yacc.yy"
+#line 90 "yacc.yy"
 { yyval.ptr = newOR( yyvsp[-2].ptr, yyvsp[0].ptr ); ;
     break;}
 case 11:
-#line 89 "yacc.yy"
+#line 91 "yacc.yy"
 { yyval.ptr = yyvsp[0].ptr; ;
     break;}
 case 12:
-#line 92 "yacc.yy"
+#line 94 "yacc.yy"
 { yyval.ptr = newAND( yyvsp[-2].ptr, yyvsp[0].ptr ); ;
     break;}
 case 13:
-#line 93 "yacc.yy"
+#line 95 "yacc.yy"
 { yyval.ptr = yyvsp[0].ptr; ;
     break;}
 case 14:
-#line 96 "yacc.yy"
+#line 98 "yacc.yy"
 { yyval.ptr = newCMP( yyvsp[-2].ptr, yyvsp[0].ptr, 1 ); ;
     break;}
 case 15:
-#line 97 "yacc.yy"
+#line 99 "yacc.yy"
 { yyval.ptr = newCMP( yyvsp[-2].ptr, yyvsp[0].ptr, 2 ); ;
     break;}
 case 16:
-#line 98 "yacc.yy"
+#line 100 "yacc.yy"
 { yyval.ptr = newCMP( yyvsp[-2].ptr, yyvsp[0].ptr, 3 ); ;
     break;}
 case 17:
-#line 99 "yacc.yy"
+#line 101 "yacc.yy"
 { yyval.ptr = newCMP( yyvsp[-2].ptr, yyvsp[0].ptr, 4 ); ;
     break;}
 case 18:
-#line 100 "yacc.yy"
+#line 102 "yacc.yy"
 { yyval.ptr = newCMP( yyvsp[-2].ptr, yyvsp[0].ptr, 5 ); ;
     break;}
 case 19:
-#line 101 "yacc.yy"
+#line 103 "yacc.yy"
 { yyval.ptr = newCMP( yyvsp[-2].ptr, yyvsp[0].ptr, 6 ); ;
     break;}
 case 20:
-#line 102 "yacc.yy"
+#line 104 "yacc.yy"
 { yyval.ptr = yyvsp[0].ptr; ;
     break;}
 case 21:
-#line 105 "yacc.yy"
+#line 107 "yacc.yy"
 { yyval.ptr = newIN( yyvsp[-2].ptr, newID( yyvsp[0].name ) ); ;
     break;}
 case 22:
-#line 106 "yacc.yy"
+#line 108 "yacc.yy"
 { yyval.ptr = yyvsp[0].ptr; ;
     break;}
 case 23:
-#line 109 "yacc.yy"
+#line 111 "yacc.yy"
 { yyval.ptr = newMATCH( yyvsp[-2].ptr, yyvsp[0].ptr ); ;
     break;}
 case 24:
-#line 110 "yacc.yy"
+#line 112 "yacc.yy"
 { yyval.ptr = yyvsp[0].ptr; ;
     break;}
 case 25:
-#line 113 "yacc.yy"
+#line 115 "yacc.yy"
 { yyval.ptr = newCALC( yyvsp[-2].ptr, yyvsp[0].ptr, 1 ); ;
     break;}
 case 26:
-#line 114 "yacc.yy"
+#line 116 "yacc.yy"
 { yyval.ptr = newCALC( yyvsp[-2].ptr, yyvsp[0].ptr, 2 ); ;
     break;}
 case 27:
-#line 115 "yacc.yy"
+#line 117 "yacc.yy"
 { yyval.ptr = yyvsp[0].ptr; ;
     break;}
 case 28:
-#line 118 "yacc.yy"
+#line 120 "yacc.yy"
 { yyval.ptr = newCALC( yyvsp[-2].ptr, yyvsp[0].ptr, 3 ); ;
     break;}
 case 29:
-#line 119 "yacc.yy"
+#line 121 "yacc.yy"
 { yyval.ptr = newCALC( yyvsp[-2].ptr, yyvsp[0].ptr, 4 ); ;
     break;}
 case 30:
-#line 120 "yacc.yy"
+#line 122 "yacc.yy"
 { yyval.ptr = yyvsp[0].ptr; ;
     break;}
 case 31:
-#line 123 "yacc.yy"
+#line 125 "yacc.yy"
 { yyval.ptr = newNOT( yyvsp[0].ptr ); ;
     break;}
 case 32:
-#line 124 "yacc.yy"
+#line 126 "yacc.yy"
 { yyval.ptr = yyvsp[0].ptr; ;
     break;}
 case 33:
-#line 127 "yacc.yy"
+#line 129 "yacc.yy"
 { yyval.ptr = newBRACKETS( yyvsp[-1].ptr ); ;
     break;}
 case 34:
-#line 128 "yacc.yy"
+#line 130 "yacc.yy"
 { yyval.ptr = newEXIST( yyvsp[0].name ); ;
     break;}
 case 35:
-#line 129 "yacc.yy"
+#line 131 "yacc.yy"
 { yyval.ptr = newID( yyvsp[0].name ); ;
     break;}
 case 36:
-#line 130 "yacc.yy"
+#line 132 "yacc.yy"
 { yyval.ptr = newNUM( yyvsp[0].vali ); ;
     break;}
 case 37:
-#line 131 "yacc.yy"
+#line 133 "yacc.yy"
 { yyval.ptr = newFLOAT( yyvsp[0].vald ); ;
     break;}
 case 38:
-#line 132 "yacc.yy"
+#line 134 "yacc.yy"
 { yyval.ptr = newSTRING( yyvsp[0].name ); ;
     break;}
 case 39:
-#line 133 "yacc.yy"
+#line 135 "yacc.yy"
 { yyval.ptr = newBOOL( yyvsp[0].valb ); ;
     break;}
 }
    /* the action file gets copied in in place of this dollarsign */
-#line 498 "/local/pkg/bison/1.25/share/bison.simple"
+#line 543 "/usr/lib/bison.simple"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1077,8 +1122,32 @@ yyerrhandle:
 
   yystate = yyn;
   goto yynewstate;
+
+ yyacceptlab:
+  /* YYACCEPT comes here.  */
+  if (yyfree_stacks)
+    {
+      free (yyss);
+      free (yyvs);
+#ifdef YYLSP_NEEDED
+      free (yyls);
+#endif
+    }
+  return 0;
+
+ yyabortlab:
+  /* YYABORT comes here.  */
+  if (yyfree_stacks)
+    {
+      free (yyss);
+      free (yyvs);
+#ifdef YYLSP_NEEDED
+      free (yyls);
+#endif
+    }
+  return 1;
 }
-#line 138 "yacc.yy"
+#line 140 "yacc.yy"
 
 
 void yyerror ( char *s )  /* Called by yyparse on error */
