@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #include <sys/types.h>
@@ -29,6 +29,7 @@
 #include <mico/template_impl.h>
 #include "mediator.h"
 #include "poamediator.h"
+#include "imr_impl.h"
 
 CORBA::ORB_ptr orb;
 CORBA::BOA_ptr boa;
@@ -142,7 +143,7 @@ usage (const char *progname)
 int
 main (int argc, char *argv[])
 {
-    CORBA::Boolean forward_opt = FALSE;
+    CORBA::Boolean forward_opt = TRUE;
 
     orb = CORBA::ORB_init (argc, argv, "mico-local-orb");
     boa = orb->BOA_init (argc, argv, "mico-local-boa");
@@ -154,6 +155,7 @@ main (int argc, char *argv[])
     MICOGetOpt::OptMap opts;
     opts["--help"]    = "";
     opts["--forward"] = "";
+    opts["--dont-forward"] = "";
     opts["--db"]      = "arg-expected";
     opts["--ior"]     = "arg-expected";
 
@@ -172,6 +174,8 @@ main (int argc, char *argv[])
 	    thedb = val;
 	} else if (arg == "--forward") {
 	    forward_opt = TRUE;
+	} else if (arg == "--dont-forward") {
+	    forward_opt = FALSE;
 	} else if (arg == "--ior") {
 	    reffile = val;
 	} else if (arg == "--help") {
@@ -184,6 +188,12 @@ main (int argc, char *argv[])
 	usage (argv[0]);
     }
 
+    /*
+     * Provide Mediators and Implementation Repository
+     */
+
+    CORBA::Object_var imr = new ImplRepository_impl;
+    orb->set_initial_reference ("ImplementationRepository", imr);
     pmd = new POAMediatorImpl (orb, forward_opt);
     med = new MediatorImpl (forward_opt);
 
@@ -191,15 +201,20 @@ main (int argc, char *argv[])
 	restore_imr (thedb.c_str());
 
     if (reffile.length() > 0) {
-      CORBA::Object_var obj = orb->resolve_initial_references ("ImplementationRepository");
-      ofstream out (reffile.c_str());
-      if (!out) {
-	cerr << "error: cannot open output file " << reffile.c_str() << endl;
+      CORBA::String_var imrior = orb->object_to_string (imr);
+
+      if (reffile == "-") {
+	cout << imrior << endl;
       }
       else {
-	CORBA::String_var pmior = orb->object_to_string (obj);
-	out << pmior.in() << endl;
-	out.close ();
+	ofstream out (reffile.c_str());
+	if (!out) {
+	  cerr << "error: cannot open output file " << reffile << endl;
+	}
+	else {
+	  out << imrior << endl;
+	  out.close ();
+	}
       }
     }
 
