@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #include <string.h>
@@ -230,11 +230,12 @@ ImplRec::ImplRec (MediatorImpl *med, CORBA::ImplementationDef_ptr impl)
 
     // get implementation repository IOR
     obj = orb->resolve_initial_references ("InterfaceRepository");
-    assert (!CORBA::is_nil (obj));
 
-    s = orb->object_to_string (obj);
-    _std_args += " -ORBIfaceRepoIOR ";
-    _std_args += s;
+    if (!CORBA::is_nil (obj)) {
+      s = orb->object_to_string (obj);
+      _std_args += " -ORBIfaceRepoIOR ";
+      _std_args += s;
+    }
 
     // get mediator IOR
     s = orb->object_to_string (_med);
@@ -1045,7 +1046,7 @@ MediatorImpl::MediatorImpl (CORBA::Boolean forward)
     _theid = 1;
     _forward = forward;
 
-    _orbnc()->register_oa (this, OAPriorityRemote);
+    _orbnc()->register_oa (this);
 }
 
 MediatorImpl::~MediatorImpl ()
@@ -1140,7 +1141,7 @@ MediatorImpl::has_object (CORBA::Object_ptr obj)
 CORBA::Boolean
 MediatorImpl::is_local () const
 {
-    return FALSE;
+    return TRUE;
 }
 
 CORBA::Boolean
@@ -1158,7 +1159,7 @@ MediatorImpl::invoke (MsgId id, CORBA::Object_ptr obj,
         CORBA::OBJECT_NOT_EXIST ex;
 	req->set_out_args (&ex);
 	_orbnc()->answer_invoke (id, CORBA::InvokeSysEx,
-				 CORBA::Object::_nil(), req);
+				 CORBA::Object::_nil(), req, 0);
 	return TRUE;
     }
     ImplRec *irec = orec->impl();
@@ -1176,7 +1177,7 @@ MediatorImpl::invoke (MsgId id, CORBA::Object_ptr obj,
         CORBA::OBJ_ADAPTER ex;
 	req->set_out_args (&ex);
 	_orbnc()->answer_invoke (id, CORBA::InvokeSysEx,
-				 CORBA::Object::_nil(), req);
+				 CORBA::Object::_nil(), req, 0);
 	return TRUE;
     }
     case PolicyRetry:
@@ -1195,7 +1196,7 @@ MediatorImpl::invoke (MsgId id, CORBA::Object_ptr obj,
     if (response_exp && _forward && irec->can_forward()) {
 	// forward to server ...
 	_orbnc()->answer_invoke (id, CORBA::InvokeForward,
-				 orec->remote_obj(), req);
+				 orec->remote_obj(), req, 0);
 	return TRUE;
     }
 
@@ -1364,14 +1365,13 @@ MediatorImpl::locate (MsgId id, CORBA::Object_ptr obj)
     // XXX do a forward if impl is persistent ???
     // XXX pass on locate to impl BOA ???
 
-    _orbnc()->answer_locate (id, CORBA::LocateHere, CORBA::Object::_nil());
+    _orbnc()->answer_locate (id, CORBA::LocateHere, CORBA::Object::_nil(), 0);
     return TRUE;
 }
 
 CORBA::Object_ptr
 MediatorImpl::skeleton (CORBA::Object_ptr)
 {
-    assert (0);
     return CORBA::Object::_nil ();
 }
 
@@ -1440,7 +1440,7 @@ MediatorImpl::copy_code_sets (CORBA::IOR *ior2, CORBA::IOR *ior1)
 void
 MediatorImpl::create_obj (CORBA::Object_ptr remote_obj,
 			  const CORBA::OAMediator::RefData& id,
-			  CORBA::Object_ptr& local_obj,
+			  CORBA::Object_out local_obj,
 			  CORBA::OAMediator::ServerId svid)
 {
     ImplRec *irec = find_impl (svid);
@@ -1474,7 +1474,7 @@ MediatorImpl::create_obj (CORBA::Object_ptr remote_obj,
 void
 MediatorImpl::restore_obj (CORBA::Object_ptr remote_obj,
 			   CORBA::Object_ptr& local_obj,
-			   CORBA::OAMediator::RefData*& id,
+			   CORBA::OAMediator::RefData_out id,
 			   CORBA::OAMediator::ServerId svid)
 {
     ObjectRec *orec = find_obj (local_obj);
@@ -1647,12 +1647,14 @@ MediatorImpl::callback (CORBA::ORB_ptr, MsgId id,
 	// an invocation completed ...
 	CORBA::ORBRequest *req;
 	CORBA::Object_var obj;
-	CORBA::InvokeStatus stat = _orbnc()->get_invoke_reply (id, obj, req);
+	GIOP::AddressingDisposition ad;
+	CORBA::InvokeStatus stat =
+	  _orbnc()->get_invoke_reply (id, obj, req, ad);
 
 	MapIdId::iterator i = _requests.find (id);
 	assert (i != _requests.end());
 
-	_orbnc()->answer_invoke ((*i).second, stat, obj, req);
+	_orbnc()->answer_invoke ((*i).second, stat, obj, req, ad);
 
 	_requests.erase (i);
 	break;
