@@ -1,5 +1,5 @@
 /*
- *  MICO --- a free CORBA implementation
+ *  MICO --- an Open Source CORBA implementation
  *  Copyright (C) 1998 Frank Pilhofer
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -16,13 +16,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #include <CORBA.h>
 #include <mico/impl.h>
 #include <mico/util.h>
+#include <mico/os-misc.h>
 #include <mico/template_impl.h>
 #include "poamediator.h"
 
@@ -67,11 +68,7 @@ POAMediatorImpl::POAMediatorImpl (CORBA::ORB_ptr _orb, CORBA::Boolean _forward)
   CORBA::Object_var obj =
     orb->resolve_initial_references ("ImplementationRepository");
   imr = CORBA::ImplRepository::_narrow (obj);
-
-  // make sure that IIOPProxy is asked first. otherwise we would
-  // end up in an endless loop invoking ourselves because has_object()
-  // doesnt detect whether we are passed a local or remote object.
-  orb->register_oa (this, OAPriorityRemote-1);
+  orb->register_oa (this);
 }
 
 POAMediatorImpl::~POAMediatorImpl ()
@@ -94,6 +91,16 @@ POAMediatorImpl::~POAMediatorImpl ()
 char *
 POAMediatorImpl::create_impl (const char * svid, const char * ior)
 {
+  CORBA::ImplRepository::ImplDefSeq_var ids =
+    imr->find_by_name (svid);
+
+  if (ids->length() == 0) {
+    CORBA::ImplementationDef_var id =
+      imr->create (CORBA::ImplementationDef::ActivatePOA,
+		   CORBA::ImplementationDef::ObjectInfoList(0),
+		   svid, "");
+  }
+
   svmap[svid].ior = CORBA::IOR (ior);
   return CORBA::string_dup (myior.c_str());
 }
@@ -101,6 +108,11 @@ POAMediatorImpl::create_impl (const char * svid, const char * ior)
 void
 POAMediatorImpl::activate_impl (const char * svid)
 {
+  if (svmap[svid].pstate == Stopped ||
+      svmap[svid].pstate == Holding) {
+    return;
+  }
+
   svmap[svid].pstate = Active;
 
   /*
@@ -114,32 +126,175 @@ void
 POAMediatorImpl::deactivate_impl (const char * svid)
 {
   SvInf &inf = svmap[svid];
-  inf.pstate = Inactive;
-  assert (inf.proc);
-  inf.proc->detach ();
-  delete inf.proc;
-}
 
-void
-POAMediatorImpl::save_state (const char * svid,
-			     const CORBA::POAMediator::StateInf & state)
-{
-  svmap[svid].state = state;
-}
+  assert (inf.pstate == Stopped || inf.proc);
 
-CORBA::POAMediator::StateInf *
-POAMediatorImpl::restore_state (const char * svid)
-{
-  return new CORBA::POAMediator::StateInf (svmap[svid].state);
+  switch (inf.pstate) {
+  case Inactive:
+  case Started:
+  case Failed:
+    assert (0);
+
+  case Active:
+    inf.pstate = Inactive;
+    break;
+
+  case Holding:
+    inf.pstate = Stopped;
+    break;
+
+  case Stopped:
+    break;
+
+  default:
+    assert (0);
+  };
+
+  if (inf.proc) {
+    inf.proc->detach ();
+    delete inf.proc;
+    inf.proc = 0;
+  }
 }
 
 CORBA::Boolean
 POAMediatorImpl::force_activation (CORBA::ImplementationDef_ptr impl)
 {
   CORBA::String_var svid = impl->name ();
+  SvInf &inf = svmap[svid.in()];
+
+  switch (inf.pstate) {
+  case Inactive:
+    /* OK */
+    break;
+
+  case Started:
+  case Active:
+    return 1;
+
+  case Holding:
+    inf.pstate = Active;
+    return 1;
+
+  case Stopped:
+    if (inf.proc) {
+      cerr << "*** server cannot be activated: " << svid << endl
+	   << "    in stopped state, still wating for server to exit" << endl;
+      return 0;
+    }
+    inf.pstate = Inactive;
+    break;
+
+  case Failed:
+    inf.pstate = Inactive;
+    break;
+
+  default:
+    assert (0);
+  }
+
   return create_server (svid);
 }
 
+CORBA::Boolean
+POAMediatorImpl::hold (CORBA::ImplementationDef_ptr impl)
+{
+  CORBA::String_var svid = impl->name ();
+  SvInf &inf = svmap[svid.in()];
+
+  switch (inf.pstate) {
+  case Inactive:
+  case Failed:
+    inf.pstate = Holding;
+    return 1;
+
+  case Started:
+  case Active:
+  case Holding:
+    inf.pstate = Holding;
+    return 1;
+
+  case Stopped:
+    return 1;
+
+  default:
+    assert (0);
+  }
+
+  /* unreachable */
+  return 0;
+}
+
+CORBA::Boolean
+POAMediatorImpl::stop (CORBA::ImplementationDef_ptr impl)
+{
+  CORBA::String_var svid = impl->name ();
+  SvInf &inf = svmap[svid.in()];
+
+  switch (inf.pstate) {
+  case Inactive:
+  case Failed:
+  case Stopped:
+    inf.pstate = Stopped;
+    return 1;
+
+  case Started:
+  case Active:
+  case Holding:
+    assert (inf.proc);
+    break;
+
+  default:
+    assert (0);
+  }
+
+  /*
+   * Signal the server to exit and wait for confirmation
+   */
+
+  inf.pstate = Stopped;
+  inf.proc->terminate ();
+
+  CORBA::Timeout t (orb->dispatcher(), 5000);
+
+  while (!t.done() && svmap[svid.in()].proc) {
+    orb->perform_work ();
+  }
+
+  if (svmap[svid.in()].proc) {
+    cerr << "*** server cannot be stopped: " << svid << endl;
+    return 0;
+  }
+
+  return 1;
+}
+
+CORBA::Boolean
+POAMediatorImpl::_cxx_continue (CORBA::ImplementationDef_ptr impl)
+{
+  CORBA::String_var svid = impl->name ();
+  SvInf &inf = svmap[svid.in()];
+
+  switch (inf.pstate) {
+  case Inactive:
+  case Started:
+  case Active:
+    return 1;
+
+  case Failed:
+  case Stopped:
+  case Holding:
+    break;
+
+  default:
+    assert (0);
+  }
+
+  inf.pstate = Inactive;
+  invqueue.exec_later();
+  return 1;
+}
+
 /*
  * Try to start up a server
  */
@@ -153,8 +308,25 @@ POAMediatorImpl::create_server (const char * svid)
     return TRUE;
   }
 
+  /*
+   * If the server is in the Failed state, we employ a similar strategy
+   * as init does. We refuse to restart a server within 30 seconds after
+   * its failure. Note that the Failed state is only entered if the server
+   * crashes prior to its activation.
+   * Without this 30 sec pause, the request that started the server would
+   * be a ping of death, causing permanent restart.
+   * We assume that the server has been fixed after the pause.
+   * This can be overridden (i.e. restart a server before 30 secs have
+   * elapsed) by using `imr activate'
+   */
+
   if (inf.pstate == Failed) {
-    return FALSE;
+    OSMisc::TimeVal curtime = OSMisc::gettime();
+    if ((curtime.tv_sec - inf.failed) < 30) {
+      // less than 30 secs elapsed since first failure
+      return FALSE;
+    }
+    inf.pstate = Inactive;
   }
 
   assert (inf.pstate == Inactive);
@@ -181,6 +353,10 @@ POAMediatorImpl::create_server (const char * svid)
   CORBA::String_var s = idv->command();
   string command = (const char *) s;
 
+  if (command.length() == 0) {
+    return 0;
+  }
+
   command += " -POAImplName ";
   command += svid;
 
@@ -190,9 +366,12 @@ POAMediatorImpl::create_server (const char * svid)
 
   CORBA::Object_var obj =
     orb->resolve_initial_references ("InterfaceRepository");
-  s = orb->object_to_string (obj);
-  command += " -ORBIfaceRepoIOR ";
-  command += s;
+
+  if (!CORBA::is_nil (obj)) {
+    s = orb->object_to_string (obj);
+    command += " -ORBIfaceRepoIOR ";
+    command += s;
+  }
 
   s = orb->object_to_string (this);
   command += " -POARemoteIOR ";
@@ -258,12 +437,20 @@ POAMediatorImpl::has_object (CORBA::Object_ptr obj)
 
   /*
    * Look up ServerId in Map
+   *
+   * If it isn't in the map, but in the Implementation Repository, recreate
+   * an svmap entry so that the server will be restarted upon invoke().
    */
 
   MapSvInf::iterator it = svmap.find (svid);
 
   if (it == svmap.end()) {
-    return FALSE;
+    CORBA::ImplRepository::ImplDefSeq_var ims =
+      imr->find_by_name (svid.c_str());
+    if (ims->length() == 0) {
+      return FALSE;
+    }
+    svmap[svid].pstate = Inactive;
   }
 
   return TRUE;
@@ -272,7 +459,7 @@ POAMediatorImpl::has_object (CORBA::Object_ptr obj)
 CORBA::Boolean
 POAMediatorImpl::is_local () const
 {
-    return FALSE;
+    return TRUE;
 }
 
 CORBA::Boolean
@@ -326,7 +513,17 @@ POAMediatorImpl::invoke (MsgId id, CORBA::Object_ptr obj,
     CORBA::OBJECT_NOT_EXIST ex;
     req->set_out_args (&ex);
     orb->answer_invoke (id, CORBA::InvokeSysEx,
-			CORBA::Object::_nil(), req);
+			CORBA::Object::_nil(), req, 0);
+    return TRUE;
+  }
+
+  /*
+   * If the Server is stopped, queue the request
+   */
+
+  if ((*it).second.pstate == Stopped ||
+      (*it).second.pstate == Holding) {
+    invqueue.add (new MICO::ReqQueueRec (id, req, obj, pr, response_exp));
     return TRUE;
   }
 
@@ -345,7 +542,7 @@ POAMediatorImpl::invoke (MsgId id, CORBA::Object_ptr obj,
       CORBA::OBJECT_NOT_EXIST ex;
       req->set_out_args (&ex);
       orb->answer_invoke (id, CORBA::InvokeSysEx,
-			  CORBA::Object::_nil(), req);
+			  CORBA::Object::_nil(), req, 0);
       return TRUE;
     }
 
@@ -373,7 +570,7 @@ POAMediatorImpl::invoke (MsgId id, CORBA::Object_ptr obj,
   if (response_exp && forward) {
     // forward to server ...
     orb->answer_invoke (id, CORBA::InvokeForward,
-			remote_ref, req);
+			remote_ref, req, 0);
     return TRUE;
   }
   /*
@@ -424,7 +621,8 @@ POAMediatorImpl::bind (MsgId id, const char *repoid,
 
     SvInf &inf = svmap[ims[i]->name()];
 
-    if (inf.pstate != Active) {
+    if (inf.pstate != Active && inf.pstate != Stopped &&
+	inf.pstate != Holding) {
       if (!create_server (ims[i]->name())) {
 	orb->answer_bind (id, CORBA::LocateUnknown,
 			  CORBA::Object::_nil());
@@ -493,7 +691,7 @@ POAMediatorImpl::bind (MsgId id, const char *repoid,
 CORBA::Boolean
 POAMediatorImpl::locate (MsgId id, CORBA::Object_ptr)
 {
-  orb->answer_locate (id, CORBA::LocateHere, CORBA::Object::_nil());
+  orb->answer_locate (id, CORBA::LocateHere, CORBA::Object::_nil(), 0);
   return TRUE;
 }
 
@@ -525,12 +723,11 @@ void
 POAMediatorImpl::shutdown_server ()
 {
   /*
-   * Kill server and give them a chance to save their state
+   * Signal servers
    */
 
   for (MapSvInf::iterator it = svmap.begin(); it != svmap.end(); it++) {
-    if ((*it).second.pstate == Started ||
-	(*it).second.pstate == Active) {
+    if ((*it).second.proc != 0) {
       (*it).second.proc->terminate ();
     }
   }
@@ -540,13 +737,13 @@ POAMediatorImpl::shutdown_server ()
    */
 
   CORBA::Boolean waiting;
-  CORBA::Long t1 = OSMisc::gettime ();
+  CORBA::Timeout t (orb->dispatcher(), 10000);
 
   do {
     waiting = FALSE;
 
-    for (MapSvInf::iterator it = svmap.begin(); it != svmap.end(); it++) {
-      if ((*it).second.pstate == Active) {
+    for (MapSvInf::iterator it1 = svmap.begin(); it1 != svmap.end(); it1++) {
+      if ((*it1).second.proc != 0) {
 	waiting = TRUE;
       }
     }
@@ -555,7 +752,13 @@ POAMediatorImpl::shutdown_server ()
       orb->perform_work ();
     }
   }
-  while (waiting && (OSMisc::gettime() - t1 < 60000));
+  while (!t.done() && waiting);
+
+  for (MapSvInf::iterator it2 = svmap.begin(); it2 != svmap.end(); it2++) {
+    if ((*it2).second.proc != 0) {
+      cerr << "*** server cannot be stopped: " << (*it2).first << endl;
+    }
+  }
 }
 
 void
@@ -586,12 +789,13 @@ POAMediatorImpl::callback (CORBA::ORB_ptr orb, MsgId id,
       // an invocation completed ...
       CORBA::ORBRequest *req;
       CORBA::Object_var obj;
-      CORBA::InvokeStatus stat = orb->get_invoke_reply (id, obj, req);
+      GIOP::AddressingDisposition ad;
+      CORBA::InvokeStatus stat = orb->get_invoke_reply (id, obj, req, ad);
 
       MapIdId::iterator i = requests.find (id);
       assert (i != requests.end());
 
-      orb->answer_invoke ((*i).second, stat, obj, req);
+      orb->answer_invoke ((*i).second, stat, obj, req, ad);
       requests.erase (i);
     }
     break;
@@ -668,11 +872,14 @@ POAMediatorImpl::callback (MICO::Process * proc,
     if ((*it).second.pstate == Active) {
       (*it).second.pstate = Inactive;
       delete (*it).second.proc;
+      (*it).second.proc = 0;
     }
     else if ((*it).second.pstate == Started) {
+      OSMisc::TimeVal ct = OSMisc::gettime();
       cerr << "*** server exited abnormally or could not be run:" << endl
 	   << "    name: " << (*it).first << endl;
       (*it).second.pstate = Failed;
+      (*it).second.failed = ct.tv_sec;
 
       /*
        * Fail pending requests
@@ -680,62 +887,21 @@ POAMediatorImpl::callback (MICO::Process * proc,
 
       invqueue.exec_later();
     }
+    else if ((*it).second.pstate == Stopped) {
+      delete (*it).second.proc;
+      (*it).second.proc = 0;
+    }
+    else if ((*it).second.pstate == Holding) {
+      delete (*it).second.proc;
+      (*it).second.proc = 0;
+      (*it).second.pstate = Stopped;
+    }
+    else {
+      assert (0);
+    }
     break;
 
   default:
     assert (0);
   }
 }
-
-/*
- * Saving and restoring of persistency data
- */
-
-void
-POAMediatorImpl::save (ostream & o)
-{
-  MapSvInf::iterator it = svmap.begin ();
-
-  o << svmap.size() << endl;
-
-  while (it != svmap.end()) {
-    o << (*it).first << endl;
-    o << (*it).second.state.length() << endl;
-    for (CORBA::ULong i=0; i<(*it).second.state.length(); i++) {
-      o << (*it).second.state[i].name.in() << endl;
-      o << (*it).second.state[i].info.in() << endl;
-    }
-    it++;
-  }
-}
-
-void
-POAMediatorImpl::restore (istream & in)
-{
-  char line1[1000], line2[1000], line3[1000], line4[1000];
-
-  if (in.getline (line1, sizeof (line1)).eof()) {
-    return;
-  }
-
-  CORBA::ULong entries = atoi (line1);
-
-  for (CORBA::ULong i=0; i<entries; i++) {
-    if (in.getline (line1, sizeof (line1)).eof() ||
-	in.getline (line2, sizeof (line2)).eof()) {
-      break;
-    }
-
-    CORBA::ULong len = atoi (line2);
-    svmap[line1].state.length (len);
-
-    for (CORBA::ULong j=0; j<len; j++) {
-      if (in.getline (line3, sizeof (line3)).eof() ||
-	  in.getline (line4, sizeof (line4)).eof()) {
-	break;
-      }
-      svmap[line1].state[j].name = CORBA::string_dup (line3);
-      svmap[line1].state[j].info = CORBA::string_dup (line4);
-    }
-  }
-}
