@@ -1,6 +1,6 @@
 // -*- c++ -*-
 /*
- *  MICO --- a free CORBA implementation
+ *  MICO --- an Open Source CORBA implementation
  *  Copyright (C) 1998 Frank Pilhofer
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 
@@ -40,6 +40,8 @@ public:
     Inactive,
     Started,
     Active,
+    Holding,
+    Stopped,
     Failed
   };
 
@@ -77,6 +79,9 @@ public:
   void deactivate_impl (const char *);
 
   CORBA::Boolean force_activation (CORBA::ImplementationDef_ptr);
+  CORBA::Boolean hold (CORBA::ImplementationDef_ptr);
+  CORBA::Boolean stop (CORBA::ImplementationDef_ptr);
+  CORBA::Boolean _cxx_continue (CORBA::ImplementationDef_ptr);
 
   /*
    * Helper
