@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 
@@ -30,9 +30,10 @@
 #include <mico/util.h>
 
 
-CodeGenCPPSkel::CodeGenCPPSkel( DB &db, IDLParam &params )
-  : CodeGen( db.get_idl_container() ),
-    CodeGenCPPUtil( db, params )
+CodeGenCPPSkel::CodeGenCPPSkel( DB &db, IDLParam &params,
+				CORBA::Container_ptr con )
+  : CodeGen( con ),
+    CodeGenCPPUtil( db, params, con )
 {
 }
 
@@ -48,16 +49,10 @@ void CodeGenCPPSkel::get_dispatcher_list( CORBA::InterfaceDef_ptr in,
 					  CORBA::OpDescriptionSeq &ops,
 					  CORBA::AttrDescriptionSeq &attr )
 {
-  if( !_params.gen_full_dispatcher) {
-    CORBA::InterfaceDef::FullInterfaceDescription_var desc;
-    desc = in->describe_interface();
-    ops = desc->operations;
-    attr = desc->attributes;
-  } else {
     CORBA::ContainedSeq_var c;
 
     // Get all operations
-    c = in->contents( CORBA::dk_Operation, FALSE );
+    c = in->contents( CORBA::dk_Operation, !_params.gen_full_dispatcher );
     for( CORBA::ULong i0 = 0; i0 < c->length(); i0++ ) {
       CORBA::ULong j = ops.length();
       ops.length( j + 1 );
@@ -68,7 +63,7 @@ void CodeGenCPPSkel::get_dispatcher_list( CORBA::InterfaceDef_ptr in,
     }
   
     // Get all attributes
-    c = in->contents( CORBA::dk_Attribute, FALSE );
+    c = in->contents( CORBA::dk_Attribute, !_params.gen_full_dispatcher );
     for( CORBA::ULong i1 = 0; i1 < c->length(); i1++ ) {
       CORBA::ULong j = attr.length();
       attr.length( j + 1 );
@@ -77,7 +72,6 @@ void CodeGenCPPSkel::get_dispatcher_list( CORBA::InterfaceDef_ptr in,
       check = d->value >>= attr[j];
       assert(check);
     }
-  }
 }
 
 /*
@@ -125,7 +119,7 @@ struct OpKey {
     : hash(h), kind(k), index(i)
   {
   }
-  int operator< (const OpKey &k) const
+  bool operator< (const OpKey &k) const
   {
     if (hash != k.hash)
       return hash < k.hash;
@@ -192,9 +186,7 @@ void CodeGenCPPSkel::emit_dispatcher (CORBA::OpDescriptionSeq &ops,
 			    OpKey::Op, j1));
     }
 
-    o << "extern CORBA::ULong mico_string_hash (const char *, CORBA::ULong);"
-      << endl;
-    o << "switch (mico_string_hash (_req->op_name(), " << p << ")) "
+    o << "switch (mico_string_hash (__req->op_name(), " << p << ")) "
       << BL_OPEN;
 
     set<OpKey, less<OpKey> >::iterator j2;
@@ -257,10 +249,13 @@ void CodeGenCPPSkel::emit_skel()
     if( check_for_included_defn( in ) )
       continue;
     
+    if (_idl_objs[i0]->obj->def_kind() == CORBA::dk_LocalInterface)
+      continue;
+    
     if( !CORBA::is_nil( val ) ) {
       CORBA::InterfaceDefSeq_var ids = val->supported_interfaces ();
       for (CORBA::ULong j=0; j<ids->length(); j++) {
-	if (!ids[j]->is_abstract()) {
+	if (ids[j]->def_kind() == CORBA::dk_Interface) {
 	  in = CORBA::InterfaceDef::_duplicate (ids[j]);
 	  break;
 	}
@@ -270,6 +265,11 @@ void CodeGenCPPSkel::emit_skel()
       }
     }
 
+    CORBA::String_var absname = con->absolute_name ();
+    if (_db->is_in_forward_dcl (absname.in())) {
+      continue;
+    }
+
     set_name_prefix( c );
     tmp = con->name();
     string className;
@@ -284,7 +284,8 @@ void CodeGenCPPSkel::emit_skel()
       get_dispatcher_list( in, operations, attributes );
     }
 
-    if (_params.boa && !CORBA::is_nil (in) && !in->is_abstract()) {
+    if (_params.boa && !CORBA::is_nil (in) &&
+	in->def_kind() == CORBA::dk_Interface) {
       o << endl;
       o << absClassName << _SKEL << "::" << className << _SKEL;
       o << "( const CORBA::BOA::ReferenceData &_id )" << endl;
@@ -292,7 +293,6 @@ void CodeGenCPPSkel::emit_skel()
       o << "CORBA::ImplementationDef_var _impl =" << indent << endl;
       o << "_find_impl( \"" << in_id << "\", \"";
       o << className << "\" );" << exdent << endl;
-      o << "assert( !CORBA::is_nil( _impl ) );" << endl;
       o << "_create_ref( _id," << indent << endl;
       o << "0," << endl;
       o << "_impl," << endl;
@@ -331,9 +331,9 @@ void CodeGenCPPSkel::emit_skel()
       o << "CORBA::StaticServerRequest_ptr ";
 
       if (attributes.length() + operations.length() == 0) {
-	o << "/*_req*/";
+	o << "/*__req*/";
       } else {
-	o << "_req";
+	o << "__req";
       }
       o << ", CORBA::Environment & /*_env*/ )" << endl << BL_OPEN;
 
@@ -350,8 +350,8 @@ void CodeGenCPPSkel::emit_skel()
 	  attributes.length() + operations.length() > 0 ) {
 	o << exdent << "#ifdef HAVE_EXCEPTIONS" << endl;
 	o << "} catch( CORBA::SystemException_catch &_ex ) " << BL_OPEN;
-	o << "_req->set_exception( _ex->_clone() );" << endl;
-	o << "_req->write_results();" << endl;
+	o << "__req->set_exception( _ex->_clone() );" << endl;
+	o << "__req->write_results();" << endl;
 	o << "return true;" << exdent << endl;
 	o << "} catch( ... ) " << BL_OPEN;
 	o << "assert( 0 );" << endl;
@@ -463,7 +463,7 @@ void CodeGenCPPSkel::emit_skel()
 	if (!CORBA::is_nil (base_value)) {
 	  base_supported = base_value->supported_interfaces ();
 	  for (CORBA::ULong i=0; i<base_supported->length(); i++) {
-	    if (!base_supported[i]->is_abstract()) {
+	    if (base_supported[i]->def_kind() == CORBA::dk_Interface) {
 	      tmp = base_value->absolute_name();
 	      base_name = ID(tmp);
 	      base_name = base_name.substr (2);
@@ -534,7 +534,7 @@ void CodeGenCPPSkel::emit_skel()
       o << "bool" << endl;
       o << "POA_" << absClassName << "::dispatch (";
       o << "CORBA::StaticServerRequest_ptr";
-      o << " _req)" << endl;
+      o << " __req)" << endl;
       o << BL_OPEN;
 
       /*
@@ -555,8 +555,8 @@ void CodeGenCPPSkel::emit_skel()
 	    attributes.length() + operations.length() > 0 ) {
 	  o << exdent << "#ifdef HAVE_EXCEPTIONS" << endl;
 	  o << "} catch( CORBA::SystemException_catch &_ex ) " << BL_OPEN;
-	  o << "_req->set_exception( _ex->_clone() );" << endl;
-	  o << "_req->write_results();" << endl;
+	  o << "__req->set_exception( _ex->_clone() );" << endl;
+	  o << "__req->write_results();" << endl;
 	  o << "return true;" << exdent << endl;
 	  o << "} catch( ... ) " << BL_OPEN;
 	  o << "assert( 0 );" << endl;
@@ -579,7 +579,7 @@ void CodeGenCPPSkel::emit_skel()
 #ifdef _VC_NAMESPACE_BUG
           base_name = vc_emit_workaround ("POA_", base_name);
 #endif
-	  o << "if (POA_" << base_name << "::dispatch (_req)) " << BL_OPEN;
+	  o << "if (POA_" << base_name << "::dispatch (__req)) " << BL_OPEN;
 	  o << "return true;" << endl;
 	  o << BL_CLOSE << endl;
 	}
@@ -592,7 +592,7 @@ void CodeGenCPPSkel::emit_skel()
 #ifdef _VC_NAMESPACE_BUG
 	base_name = vc_emit_workaround ("POA_", base_name);
 #endif
-	o << "if (POA_" << base_name << "::dispatch (_req)) ";
+	o << "if (POA_" << base_name << "::dispatch (__req)) ";
 
 	o << BL_OPEN << "return true;" << endl;
 	o << BL_CLOSE << endl;
@@ -603,14 +603,14 @@ void CodeGenCPPSkel::emit_skel()
 	if (!CORBA::is_nil (base_value)) {
 	  base_supported = base_value->supported_interfaces ();
 	  for (CORBA::ULong i=0; i<base_supported->length(); i++) {
-	    if (!base_supported[i]->is_abstract()) {
+	    if (base_supported[i]->def_kind() == CORBA::dk_Interface) {
 	      tmp = base_supported[i]->absolute_name();
 	      base_name = ID(tmp);
 	      base_name = base_name.substr (2);
 #ifdef _VC_NAMESPACE_BUG
 	      base_name = vc_emit_workaround ("POA_", base_name);    
 #endif
-	      o << "if (POA_" << base_name << "::dispatch (_req)) " << BL_OPEN;
+	      o << "if (POA_" << base_name << "::dispatch (__req)) " << BL_OPEN;
 	      o << "return TRUE;" << endl;
 	      o << BL_CLOSE;
 	    }
@@ -624,16 +624,16 @@ void CodeGenCPPSkel::emit_skel()
       o << "void" << endl;
       o << "POA_" << absClassName << "::invoke (";
       o << "CORBA::StaticServerRequest_ptr";
-      o << " _req)" << endl;
+      o << " __req)" << endl;
       o << BL_OPEN;
-      o << "if (dispatch (_req)) " << BL_OPEN;
+      o << "if (dispatch (__req)) " << BL_OPEN;
       o << "  return;" << endl;
       o << BL_CLOSE;
       o << endl;
       o << "CORBA::Exception * ex = " << endl;
       o << "  new CORBA::BAD_OPERATION (0, CORBA::COMPLETED_NO);" << endl;
-      o << "_req->set_exception (ex);" << endl;
-      o << "_req->write_results();" << endl;
+      o << "__req->set_exception (ex);" << endl;
+      o << "__req->write_results();" << endl;
       o << BL_CLOSE << endl;
       use_rel_names (oldurn);
     }
@@ -698,7 +698,7 @@ CodeGenCPPSkel::emit_sii_method_dispatcher( string &dispatch_name,
 					    CORBA::OperationDescription &op,
 					    CORBA::IDLType_ptr result )
 {
-  o << "if( strcmp( _req->op_name(), \"" << 
+  o << "if( strcmp( __req->op_name(), \"" << 
     dispatch_name << "\" ) == 0 ) " << BL_OPEN;
   CORBA::ParDescriptionSeq &p = op.parameters;
   CORBA::ULong k;
@@ -709,27 +709,41 @@ CodeGenCPPSkel::emit_sii_method_dispatcher( string &dispatch_name,
     // Declaration for parameter
     if( p[ k ].mode != CORBA::PARAM_OUT ) {
       emit_type_for_variable( p[ k ].type_def, automatic );
-      o << " " << ID(p[ k ].name);
-      emit_array_suffix( p[ k ].type_def );
-      if (p[k].mode == CORBA::PARAM_INOUT && is_var_inout (p[k].type_def))
-	o << " ((CORBA::Boolean)FALSE)";
-      o << ";" << endl;
-    } else {
-      if( p[k].type->is_array() && p[k].type->is_variable() ) {
+      if( !p[k].type->is_array() ) {
+	o << " " << ID(p[ k ].name);
+	emit_array_suffix( p[ k ].type_def );
+#if 0
+	if (p[k].mode == CORBA::PARAM_INOUT && is_var_inout (p[k].type_def))
+	  o << " ((CORBA::Boolean)FALSE)";
+#endif
+	o << ";" << endl;
+      } else {
+	o << "_slice* " << ID(p[ k ].name) << " = ";
 	emit_type_for_variable( p[ k ].type_def, automatic );
-	o << "_slice*";
-      } else
+	o << "_alloc();" << endl;
+      }
+    } else {
+      if( p[k].type->is_array() ) {
+	emit_type_for_variable( p[ k ].type_def, automatic );
+	o << "_slice* " << ID(p[ k ].name) << " = ";
+	emit_type_for_variable( p[ k ].type_def, automatic );
+	o << "_alloc();" << endl;
+      } else {
 	emit_type_for_variable( p[ k ].type_def, manual );
-      o << " " << ID(p[ k ].name) << ";" << endl;
+	o << " " << ID(p[ k ].name) << ";" << endl;
+      }
     }
 
     // Emit StaticAny for argument
     o << "CORBA::StaticAny _" << ID(p[k].name) << "( ";
     emit_marshaller_ref( p[k].type_def );
     if( p[k].mode != CORBA::PARAM_OUT || !is_variable_for_sii (p[k].type) ) {
-      o << ", &" << ID(p[k].name);
+      o << ", ";
+      if( !p[k].type->is_array() )
+	o << "&";
+      o << ID(p[k].name);
       if (p[k].mode != CORBA::PARAM_OUT)
-	emit_marshaller_suffix( p[k].type_def );
+	emit_marshaller_suffix( p[k].type_def, FALSE );
     }
     o << " );" << endl;
   }
@@ -755,7 +769,7 @@ CodeGenCPPSkel::emit_sii_method_dispatcher( string &dispatch_name,
   }
 
   for( k = 0; k != p.length(); k++ ) {
-    o << "_req->add_";
+    o << "__req->add_";
     switch( p[ k ].mode ) {
     case CORBA::PARAM_IN:
       o << "in";
@@ -773,11 +787,11 @@ CodeGenCPPSkel::emit_sii_method_dispatcher( string &dispatch_name,
   }
 
   if( tc_result->kind() != CORBA::tk_void ) {
-    o << "_req->set_result( &__res );" << endl;
+    o << "__req->set_result( &__res );" << endl;
   }
   o << endl;
 
-  o << "if( !_req->read_args() )" << endl;
+  o << "if( !__req->read_args() )" << endl;
   o << indent << "return true;" << exdent << endl << endl;
   
   if( !_params.no_exceptions && op.mode == CORBA::OP_NORMAL &&
@@ -798,17 +812,14 @@ CodeGenCPPSkel::emit_sii_method_dispatcher( string &dispatch_name,
     if( k != 0 )
       o << ",";
     o << " " << ID(p[ k ].name);
-#ifdef USE_CORBA_2_1
-    // for 2.1 compliance String_var does not have a operator char*&
-    if( p[k].type->is_string() && p[k].mode == CORBA::PARAM_INOUT )
-      o << ".inout()";
-#endif
+    if (p[k].mode != CORBA::PARAM_OUT)
+	emit_marshaller_suffix( p[k].type_def, FALSE );
   }
   CORBA::ContextIdSeq& ctx = op.contexts;
   if( ctx.length() > 0 ) {
     if( p.length() > 0 )
       o << ", ";
-    o << "_req->ctx()";
+    o << "__req->ctx()";
   }
   if( k != 0 || ctx.length() > 0 )
     o << " ";
@@ -839,8 +850,8 @@ CodeGenCPPSkel::emit_sii_method_dispatcher( string &dispatch_name,
       CORBA::String_var tmp = con->absolute_name();
       CORBA::String_var ex_abs_name = (const char *) ID(tmp);
       o << "} catch( " << ex_abs_name << "_catch &_ex ) " << BL_OPEN;
-      o << "_req->set_exception( _ex->_clone() );" << endl;
-      o << "_req->write_results();" << endl;
+      o << "__req->set_exception( _ex->_clone() );" << endl;
+      o << "__req->write_results();" << endl;
       o << "return true;" << endl;
       o << exdent;
     }
@@ -849,15 +860,14 @@ CodeGenCPPSkel::emit_sii_method_dispatcher( string &dispatch_name,
   }
   
   // Write back OUT and INOUT parameters
-  o << "_req->write_results();" << endl;
+  o << "__req->write_results();" << endl;
 
 
   // Release memory of variable length out and inout parameters
   for( k = 0; k < p.length(); k++ ) {
-    if (p[k].mode == CORBA::PARAM_OUT &&
-	!(p[k].type->is_array() && !p[k].type->is_variable())) {
+    if( ( p[ k ].mode == CORBA::PARAM_OUT ) || p[ k ].type->is_array() ) {
       string name = ID(p[k].name);
-      emit_release (p[k].type_def, name.c_str());
+      emit_release( p[k].type_def, name.c_str() );
     }
   }
   
