@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,21 +16,18 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #include <CORBA.h>
 #include <mico/util.h>
-#include <mico/ir_impl.h>
 #include <mico/template_impl.h>
 #include "db.h"
 
 
 DB::DB()
 {
-  _parse_tree = NULL;
-
   vector<string> empty_scope;
   _current_scope.push_back (empty_scope);
 
@@ -40,30 +37,6 @@ DB::DB()
 
 DB::~DB()
 {
-  if( _parse_tree != NULL )
-    delete _parse_tree;
-}
-
-void DB::set_parse_tree( ParseNode* node )
-{
-  if( _parse_tree != NULL )
-    delete _parse_tree;
-  _parse_tree = node;
-}
-
-ParseNode* DB::get_parse_tree()
-{
-  return _parse_tree;
-}
-
-void DB::set_idl_container( CORBA::Container_ptr con )
-{
-  _con = CORBA::Container::_duplicate( con );
-}
-
-CORBA::Container_ptr DB::get_idl_container()
-{
-  return CORBA::Container::_duplicate( _con );
 }
 
 /*
@@ -133,19 +106,27 @@ string DB::gen_pseudo_repoid( CORBA::IDLType_ptr t )
 
 void DB::register_repoid( ParseNode* node, string &id )
 {
-  if( !(node->is_toplevel()) )
-    _repoid_fname_map[ id ] = node->getFname();
+  if (repoid_defined (id)) {
+    return;
+  }
+  _repoid_fname_map[ id ] = node->getFname();
+}
+
+void DB::register_repoid( string &id, string &fname )
+{
+  _repoid_fname_map[ id ] = fname;
 }
 
 void DB::register_anonym_type( ParseNode* node, CORBA::IDLType_ptr t )
 {
-  if( node->is_toplevel() )
-    return;
-
   string pseudo_id;
   
   pseudo_id = gen_pseudo_repoid( t );
 
+  if (repoid_defined (pseudo_id)) {
+    return;
+  }
+
   _repoid_fname_map[ pseudo_id ] = node->getFname();
 }
 
@@ -159,6 +140,18 @@ string DB::get_fname_for_repoid( string &id )
   return _repoid_fname_map[ id ];
 }
 
+string
+DB::get_toplevel_fname ()
+{
+  return _toplevel_fname;
+}
+
+void
+DB::set_toplevel_fname (const char * fname)
+{
+  _toplevel_fname = fname;
+}
+
 void DB::enter_scope( string& name )
 {
   _current_scope.back().push_back( name );
@@ -237,17 +230,105 @@ void DB::set_repo_id( string& name, string& id )
   _id.push_back( id );
 }
 
+void
+DB::set_prefix( string& name, string& prefix )
+{
+  _prefix_name.push_back( name );
+  _prefix.push_back( prefix );
+}
+
+/*
+ * Look for a typePrefix to use for some type and its descendants
+ * Prefixes can be recursively re-defined, so we must continue to look
+ * for the most specific match.
+ */
+
+string
+DB::get_prefix (const char * absolute_name)
+{
+  CORBA::ULong longest_match=0;
+  CORBA::ULong length;
+  string prefix = "";
+
+  for (vector<string>::size_type i=0; i<_prefix_name.size(); i++) {
+    length = _prefix_name[i].length();
+    if (strncmp (_prefix_name[i].data(), absolute_name, length) == 0 &&
+	(absolute_name[length] == '\0' ||
+	 absolute_name[length] == ':'  ||
+	 (length == 2 && absolute_name[0]==':' && absolute_name[1]==':')) &&
+	length > longest_match) {
+      longest_match = length;
+      prefix = _prefix[i];
+    }
+  }
+
+  if (longest_match == 0) {
+    prefix = _current_prefix.back ();
+  }
+
+  for (VecVecString::size_type j=0; j < _current_scope.back().size(); j++) {
+    if (prefix.length() > 0)
+      prefix += "/";
+    prefix += (_current_scope.back())[j];
+  }
+
+  if (prefix.length() > 0)
+    prefix += "/";
+
+  return prefix;
+}
+
 void DB::set_repoids( CORBA::Repository_ptr repo )
 {
+  /*
+   * Also update the id when setting the version. But generation of a
+   * new id is problematic. At this point, we don't know what prefix
+   * was in use when this definition was read. So we do a hack and
+   * just replace the last three characters of the current id.
+   */
   for( vector<string>::size_type i = 0; i < _version.size(); i++ ) {
     CORBA::Contained_var c =
-      repo->lookup( (char *) _version_name[ i ].c_str() );
+      repo->lookup( _version_name[ i ].c_str() );
     assert( !CORBA::is_nil( c ) );
-    c->version( (char *) _version[ i ].c_str() );
+    c->version( _version[ i ].c_str() );
+
+    CORBA::String_var cid = c->id ();
+    string oldid = cid.in();
+    string fname = get_fname_for_repoid (oldid);
+    CORBA::ULong ilen = oldid.length ();
+    assert (ilen > 3 && _version[i].length() == 3 &&
+	    cid[ilen-4] == ':' && cid[ilen-2] == '.');
+    string newid = oldid;
+    newid.replace (ilen - 3, 3, _version[i]);
+    register_repoid (newid, fname);
+    c->id (newid.c_str());
   }
   for( vector<string>::size_type i0 = 0; i0 < _id.size(); i0++ ) {
-    CORBA::Contained_var c = repo->lookup( (char *) _id_name[ i0 ].c_str() );
+    CORBA::Contained_var c = repo->lookup( _id_name[ i0 ].c_str() );
     assert( !CORBA::is_nil( c ) );
-    c->id( (char *) _id[ i0 ].c_str() );
+    CORBA::String_var oldid = c->id ();
+    string soldid = oldid.in();
+    string fname = get_fname_for_repoid (soldid);
+    register_repoid (_id[i0], fname);
+    c->id( _id[ i0 ].c_str() );
   }
 }
+
+void
+DB::add_forward_dcl( const char* scoped_name )
+{
+  _forward_dcl.insert( scoped_name );
+}
+
+void
+DB::remove_forward_dcl( const char* scoped_name )
+{
+  _forward_dcl.erase( scoped_name );
+}
+
+bool
+DB::is_in_forward_dcl( const char* scoped_name )
+{
+  return _forward_dcl.find( scoped_name ) != _forward_dcl.end();
+}
+
