@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #include "dep.h"
@@ -75,6 +75,7 @@ IDLDep::build_dependencies()
   set_current_iface( CORBA::Container::_nil() );
   traverse_container( _container );
   compute_dependencies();
+  // print (_top);
   return _ordered_objs;
 };
 
@@ -173,20 +174,66 @@ IDLDep::create_dep( Dep* from, Dep* to )
   // if there is a circular dependency (which would occur for
   // recursive data structures), we don't need to register a dependency
   if( is_circular_dep( from, to ) ) {
-    
+      
     // Actually, if "to" denotes an InterfaceDef, then we had a
     // forward declaration before "from" was defined. Therefore
     // we need to add this dependency to the forward declaration
     // at this time.
     if( !CORBA::is_nil( to->obj ) &&
 	(to->obj->def_kind() == CORBA::dk_Interface ||
-	 to->obj->def_kind() == CORBA::dk_Value)) {
+	 to->obj->def_kind() == CORBA::dk_AbstractInterface ||
+	 to->obj->def_kind() == CORBA::dk_LocalInterface ||
+	 to->obj->def_kind() == CORBA::dk_Struct ||
+	 to->obj->def_kind() == CORBA::dk_Union ||
+	 to->obj->def_kind() == CORBA::dk_Value ||
+	 to->obj->def_kind() == CORBA::dk_Component)) {
       Dep *d = locate_dep_for_forward_iface( to->obj );
       from->deps.push_back( d );
     }
     
     return;
   }
+
+  /*
+   * If to is part of an interface, and from is not inside that interface,
+   * too, then make a dependency on the interface instead
+   */
+
+  CORBA::Contained_var co = CORBA::Contained::_narrow (to->obj);
+
+  if (!CORBA::is_nil (co) &&
+      co->def_kind() != CORBA::dk_Interface &&
+      co->def_kind() != CORBA::dk_AbstractInterface &&
+      co->def_kind() != CORBA::dk_LocalInterface &&
+      co->def_kind() != CORBA::dk_Value &&
+      co->def_kind() != CORBA::dk_Component &&
+      !from->iface_as_forward &&
+      !to->iface_as_forward) {
+
+    CORBA::Container_var con = co->defined_in ();
+    co = CORBA::Contained::_narrow (con);
+
+    while (!CORBA::is_nil (co) &&
+	   co->def_kind() != CORBA::dk_Interface &&
+	   co->def_kind() != CORBA::dk_AbstractInterface &&
+	   co->def_kind() != CORBA::dk_LocalInterface &&
+	   co->def_kind() != CORBA::dk_Value &&
+	   co->def_kind() != CORBA::dk_Component) {
+      con = co->defined_in ();
+      co = CORBA::Contained::_narrow (con);
+    }
+
+    if (!CORBA::is_nil (co)) {
+      CORBA::Contained_var co2 = CORBA::Contained::_narrow (from->obj);
+      if (!CORBA::is_nil (co2)) {
+	CORBA::String_var s1 = co->absolute_name ();
+	CORBA::String_var s2 = co2->absolute_name ();
+	if (strncmp (s1.in(), s2.in(), strlen (s1.in())) != 0) {
+	  to = locate_dep (co, true, true);
+	}
+      }
+    }
+  }
   
   from->deps.push_back( to );
 }
@@ -198,7 +245,7 @@ IDLDep::create_iface_dep( Dep* to )
   // we need to create a dependency from the interface to
   // the contained object
   if( !CORBA::is_nil( _current_iface ) ) {
-    Dep* from = locate_dep( _current_iface, true );
+    Dep* from = locate_dep( _current_iface, true, true );
     create_dep( from, to );
   }
 }
@@ -224,65 +271,74 @@ IDLDep::locate_dep_for_forward_iface( CORBA::IRObject_ptr obj )
 }
 
 Dep*
-IDLDep::locate_dep( CORBA::IRObject_ptr obj, bool create_iface )
+IDLDep::locate_dep( CORBA::IRObject_ptr obj,
+		    bool create_type,
+		    bool create_iface )
 {
-  MapObjDep::iterator i = _all_deps.find( obj );
-
-  if( i == _all_deps.end() ) {
-    // This object is not yet registered
-
-    if( obj->def_kind() == CORBA::dk_Primitive ) {
-      // For primitive data types, we return _bottom
-      return _bottom;
-    }
-
-    if( (obj->def_kind() == CORBA::dk_Interface ||
-	 obj->def_kind() == CORBA::dk_Value) && !create_iface ) {
+  if (((obj->def_kind() == CORBA::dk_Interface ||
+	obj->def_kind() == CORBA::dk_AbstractInterface ||
+	obj->def_kind() == CORBA::dk_LocalInterface ||
+	obj->def_kind() == CORBA::dk_Value ||
+	obj->def_kind() == CORBA::dk_Component) && !create_iface) ||
+      ((obj->def_kind() == CORBA::dk_Struct ||
+	obj->def_kind() == CORBA::dk_Union) && !create_type)) {
       // We're trying to find an interface that has not yet been
       // registered and no dependecy node should be generated.
       // This means that the interface is used in a forward
       // declaration. We need to generate a special forward
       // node.
       return locate_dep_for_forward_iface( obj );
+  }
+
+  MapObjDep::iterator i = _all_deps.find( obj );
+
+  if (i != _all_deps.end()) {
+    return (*i).second;
+  }
+
+  // This object is not yet registered
+
+  if( obj->def_kind() == CORBA::dk_Primitive ) {
+    // For primitive data types, we return _bottom
+    return _bottom;
+  }
+
+  // Create a new node in the dependency graph
+  Dep* d = new Dep;
+  d->obj = CORBA::IRObject::_duplicate( obj );
+  _all_deps[ obj ] = d;
+
+  // Link the new obj with _top
+  create_dep( _top, d );
+
+  // if obj is a sequence or an array we also need to register
+  // recursively all nested obj
+  switch( obj->def_kind() ) {
+  case CORBA::dk_Sequence:
+    {
+      CORBA::SequenceDef_var seq = CORBA::SequenceDef::_narrow( obj );
+      CORBA::IDLType_var elem = seq->element_type_def();
+      Dep* elem_dep = locate_dep( elem );
+      create_dep( d, elem_dep );
+      break;
     }
-    
-    // Create a new node in the dependency graph
-    Dep* d = new Dep;
-    d->obj = CORBA::IRObject::_duplicate( obj );
-    _all_deps[ obj ] = d;
-
-    // Link the new obj with _top
-    create_dep( _top, d );
-
-    // if obj is a sequence or an array we also need to register
-    // recursively all nested obj
-    switch( obj->def_kind() ) {
-    case CORBA::dk_Sequence:
-      {
-	CORBA::SequenceDef_var seq = CORBA::SequenceDef::_narrow( obj );
-	CORBA::IDLType_var elem = seq->element_type_def();
-	Dep* elem_dep = locate_dep( elem );
-	create_dep( d, elem_dep );
-	break;
-      }
-    case CORBA::dk_Array:
-      {
-	CORBA::ArrayDef_var arr = CORBA::ArrayDef::_narrow( obj );
-	CORBA::IDLType_var elem = arr->element_type_def();
-	Dep* elem_dep = locate_dep( elem );
-	create_dep( d, elem_dep );
-	break;
-      }
+  case CORBA::dk_Array:
+    {
+      CORBA::ArrayDef_var arr = CORBA::ArrayDef::_narrow( obj );
+      CORBA::IDLType_var elem = arr->element_type_def();
+      Dep* elem_dep = locate_dep( elem );
+      create_dep( d, elem_dep );
+      break;
     }
   }
   
-  return _all_deps[ obj ];
+  return d;
 }
 
 void
 IDLDep::dep_Struct( CORBA::StructDef_ptr s )
 {
-  Dep* from = locate_dep( s );
+  Dep* from = locate_dep( s, true , true );
   create_iface_dep( from );
 
   CORBA::Container_var old_current = get_current_iface ();
@@ -295,7 +351,7 @@ IDLDep::dep_Struct( CORBA::StructDef_ptr s )
   for( CORBA::ULong i = 0; i < members->length(); i++ ) {
     CORBA::TypeCode_ptr tc = members[ i ].type;
     CORBA::IDLType_ptr m = members[ i ].type_def;
-    Dep* to = locate_dep( m );
+    Dep* to = locate_dep( m, true );
     create_dep( from, to );
   }
 }
@@ -303,7 +359,7 @@ IDLDep::dep_Struct( CORBA::StructDef_ptr s )
 void
 IDLDep::dep_Union( CORBA::UnionDef_ptr u )
 {
-  Dep* from = locate_dep( u );
+  Dep* from = locate_dep( u, true, true );
   create_iface_dep( from );
 
   CORBA::Container_var old_current = get_current_iface ();
@@ -316,12 +372,12 @@ IDLDep::dep_Union( CORBA::UnionDef_ptr u )
   for( CORBA::ULong i = 0; i < members->length(); i++ ) {
     CORBA::TypeCode_ptr tc = members[ i ].type;
     CORBA::IDLType_ptr m = members[ i ].type_def;
-    Dep* to = locate_dep( m );
+    Dep* to = locate_dep( m, true );
     create_dep( from, to );
   }
 
   CORBA::IDLType_var disc = u->discriminator_type_def();
-  Dep* to = locate_dep( disc );
+  Dep* to = locate_dep( disc, true );
   create_dep( from, to );
 }
 
@@ -332,7 +388,7 @@ IDLDep::dep_Constant( CORBA::ConstantDef_ptr c )
   create_iface_dep( from );
 
   CORBA::IDLType_var orig = c->type_def();
-  Dep* to = locate_dep( orig );
+  Dep* to = locate_dep( orig, true );
 
   create_dep( from, to );
 }
@@ -353,7 +409,7 @@ IDLDep::dep_Exception( CORBA::ExceptionDef_ptr e )
   CORBA::ULong i;
   for( i = 0; i < members->length(); i++ ) {
     CORBA::IDLType_ptr m = members[ i ].type_def;
-    Dep* to = locate_dep( m );
+    Dep* to = locate_dep( m, true );
     create_dep( from, to );
   }
 }
@@ -382,12 +438,12 @@ IDLDep::dep_Interface( CORBA::InterfaceDef_ptr in )
 {
   set_current_iface( in );
 
-  Dep* from = locate_dep( in, true );
+  Dep* from = locate_dep( in, true, true );
 
   // set dependencies for base interfaces
   CORBA::InterfaceDefSeq_var bases = in->base_interfaces();
   for( CORBA::ULong i0 = 0; i0 < bases->length(); i0++ ) {
-    Dep* to = locate_dep( bases[ i0 ], true );
+    Dep* to = locate_dep( bases[ i0 ], true, true );
     create_dep( from, to );
   }
 
@@ -400,21 +456,24 @@ IDLDep::dep_Interface( CORBA::InterfaceDef_ptr in )
 void
 IDLDep::dep_Operation( CORBA::OperationDef_ptr o )
 {
+  Dep* from = locate_dep (o);
+  create_iface_dep (from);
+
   CORBA::IDLType_var res = o->result_def();
   
-  Dep* d = locate_dep( res );
-  create_iface_dep( d );
+  Dep* d = locate_dep( res, true );
+  create_dep( from, d );
 
   CORBA::ParDescriptionSeq_var params = o->params();
   for( CORBA::ULong i = 0; i < params->length(); i++ ) {
-    d = locate_dep( params[ i ].type_def );
-    create_iface_dep( d );
+    d = locate_dep( params[ i ].type_def, true );
+    create_dep( from, d );
   }
   
   CORBA::ExceptionDefSeq_var exs = o->exceptions();
   for( CORBA::ULong i0 = 0; i0 < exs->length(); i0++ ) {
-    d = locate_dep( exs[ i0 ] );
-    create_iface_dep( d );
+    d = locate_dep( exs[ i0 ], true );
+    create_dep( from, d );
   }
   
 }
@@ -422,9 +481,12 @@ IDLDep::dep_Operation( CORBA::OperationDef_ptr o )
 void
 IDLDep::dep_Attribute( CORBA::AttributeDef_ptr a )
 {
+  Dep* from = locate_dep (a);
+  create_iface_dep (from);
+
   CORBA::IDLType_var t = a->type_def();
-  Dep* d = locate_dep( t );
-  create_iface_dep( d );
+  Dep* d = locate_dep( t, true );
+  create_dep( from, d );
 }
 
 void
@@ -434,14 +496,14 @@ IDLDep::dep_ValueBox( CORBA::ValueBoxDef_ptr vb )
   create_iface_dep (from);
 
   CORBA::IDLType_var orig = vb->original_type_def();
-  Dep* to = locate_dep (orig);
+  Dep* to = locate_dep (orig, true);
   create_dep (from, to);
 }
 
 void
 IDLDep::dep_Value( CORBA::ValueDef_ptr vb )
 {
-  Dep* from = locate_dep( vb, true );
+  Dep* from = locate_dep( vb, true, true );
   create_iface_dep( from );
 
   CORBA::Container_var old_current = get_current_iface ();
@@ -451,19 +513,19 @@ IDLDep::dep_Value( CORBA::ValueDef_ptr vb )
   // set dependencies for base values and supported interface
   CORBA::InterfaceDefSeq_var supported_interfaces = vb->supported_interfaces();
   for (CORBA::ULong i2=0; i2<supported_interfaces->length(); i2++) {
-    Dep * to = locate_dep (supported_interfaces[i2], true);
+    Dep * to = locate_dep (supported_interfaces[i2], true, true);
     create_dep (from, to);
   }
 
   CORBA::ValueDef_var base_value = vb->base_value ();
   if (!CORBA::is_nil (base_value)) {
-    Dep * to = locate_dep (base_value, true);
+    Dep * to = locate_dep (base_value, true, true);
     create_dep (from, to);
   }
 
   CORBA::ValueDefSeq_var abv = vb->abstract_base_values ();
   for (CORBA::ULong i0=0; i0<abv->length(); i0++) {
-    Dep * to = locate_dep (abv[i0], true);
+    Dep * to = locate_dep (abv[i0], true, true);
     create_dep (from, to);
   }
 
@@ -472,7 +534,7 @@ IDLDep::dep_Value( CORBA::ValueDef_ptr vb )
   for (CORBA::ULong i1=0; i1<isv->length(); i1++) {
     for (CORBA::ULong i2=0; i2<isv[i1].members.length(); i2++) {
       CORBA::IDLType_ptr m = isv[i1].members[i2].type_def;
-      Dep * to = locate_dep (m, true);
+      Dep * to = locate_dep (m, true, true);
       create_dep (from, to);
     }
   }
@@ -483,9 +545,12 @@ IDLDep::dep_Value( CORBA::ValueDef_ptr vb )
 void
 IDLDep::dep_ValueMember( CORBA::ValueMemberDef_ptr vmd )
 {
+  Dep* from = locate_dep( vmd );
+  create_iface_dep( from );
+
   CORBA::IDLType_var t = vmd->type_def();
-  Dep* d = locate_dep( t );
-  create_iface_dep( d );
+  Dep* d = locate_dep( t, true );
+  create_dep( from, d );
 }
 
 void
@@ -495,6 +560,99 @@ IDLDep::dep_Native( CORBA::NativeDef_ptr nd )
   create_iface_dep (from);
 }
 
+#ifdef USE_CCM
+
+void
+IDLDep::dep_Component (CORBA::ComponentIR::ComponentDef_ptr me)
+{
+  Dep* from = locate_dep( me, true, true );
+  create_iface_dep( from );
+
+  CORBA::Container_var old_current = get_current_iface ();
+  set_current_iface (me);
+  traverse_container (me);
+
+  CORBA::ComponentIR::ComponentDef_var base = me->base_component ();
+  if (!CORBA::is_nil (base)) {
+    Dep * to = locate_dep (base, true, true);
+    create_dep (from, to);
+  }
+
+  CORBA::InterfaceDefSeq_var supported = me->supported_interfaces();
+  for (CORBA::ULong si=0; si<supported->length(); si++) {
+    Dep * to = locate_dep (supported[si], true, true);
+    create_dep (from, to);
+  }
+
+  set_current_iface (old_current.in());
+}
+
+void
+IDLDep::dep_Provides (CORBA::ComponentIR::ProvidesDef_ptr me)
+{
+  Dep* from = locate_dep( me );
+  create_iface_dep( from );
+
+  CORBA::InterfaceDef_var it = me->interface_type ();
+  Dep * to = locate_dep (it);
+  create_dep (from, to);
+}
+
+void
+IDLDep::dep_Uses (CORBA::ComponentIR::UsesDef_ptr me)
+{
+  Dep* from = locate_dep( me );
+  create_iface_dep( from );
+
+  CORBA::InterfaceDef_var it = me->interface_type ();
+  Dep * to = locate_dep (it);
+  create_dep (from, to);
+}
+
+void
+IDLDep::dep_Event (CORBA::ComponentIR::EventDef_ptr me)
+{
+  Dep* from = locate_dep( me );
+  create_iface_dep( from );
+
+  CORBA::ValueDef_var it = me->event ();
+  Dep * to = locate_dep (it);
+  create_dep (from, to);
+}
+
+void
+IDLDep::dep_Home (CORBA::ComponentIR::HomeDef_ptr me)
+{
+  Dep* from = locate_dep( me, true, true );
+  create_iface_dep( from );
+
+  CORBA::Container_var old_current = get_current_iface ();
+  set_current_iface (me);
+  traverse_container (me);
+
+  CORBA::ComponentIR::HomeDef_var base = me->base_home ();
+  if (!CORBA::is_nil (base)) {
+    Dep * to = locate_dep (base, true, true);
+    create_dep (from, to);
+  }
+
+  CORBA::ComponentIR::ComponentDef_var managed = me->managed_component ();
+  if (!CORBA::is_nil (managed)) {
+    Dep * to = locate_dep (managed, true, true);
+    create_dep (from, to);
+  }
+
+  CORBA::ValueDef_var pk = me->primary_key ();
+  if (!CORBA::is_nil (pk)) {
+    Dep * to = locate_dep (pk);
+    create_dep (from, to);
+  }
+
+  set_current_iface (old_current.in());
+}
+
+#endif
+
 void
 IDLDep::traverse_container( CORBA::Container_ptr con )
 {
@@ -510,6 +668,8 @@ IDLDep::traverse_container( CORBA::Container_ptr con )
 	break;
       }
     case CORBA::dk_Interface:
+    case CORBA::dk_AbstractInterface:
+    case CORBA::dk_LocalInterface:
       {
 	CORBA::InterfaceDef_var in = CORBA::InterfaceDef::_narrow( c[ i ] );
 	dep_Interface( in );
@@ -588,6 +748,53 @@ IDLDep::traverse_container( CORBA::Container_ptr con )
 	dep_Native( nd );
 	break;
       }
+
+#ifdef USE_CCM
+    case CORBA::dk_Component:
+      {
+	CORBA::ComponentIR::ComponentDef_var comp =
+	  CORBA::ComponentIR::ComponentDef::_narrow (c[i]);
+	dep_Component (comp);
+	break;
+      }
+    case CORBA::dk_Provides:
+      {
+	CORBA::ComponentIR::ProvidesDef_var comp =
+	  CORBA::ComponentIR::ProvidesDef::_narrow (c[i]);
+	dep_Provides (comp);
+	break;
+      }
+    case CORBA::dk_Uses:
+      {
+	CORBA::ComponentIR::UsesDef_var comp =
+	  CORBA::ComponentIR::UsesDef::_narrow (c[i]);
+	dep_Uses (comp);
+	break;
+      }
+    case CORBA::dk_Emits:
+    case CORBA::dk_Publishes:
+    case CORBA::dk_Consumes:
+      {
+	CORBA::ComponentIR::EventDef_var comp =
+	  CORBA::ComponentIR::EventDef::_narrow (c[i]);
+	dep_Event (comp);
+	break;
+      }
+    case CORBA::dk_Home:
+      {
+	CORBA::ComponentIR::HomeDef_var comp =
+	  CORBA::ComponentIR::HomeDef::_narrow (c[i]);
+	dep_Home (comp);
+	break;
+      }
+    case CORBA::dk_Factory:
+    case CORBA::dk_Finder:
+      {
+	CORBA::OperationDef_var op = CORBA::OperationDef::_narrow (c[1]);
+	dep_Operation (op);
+	break;
+      }
+#endif
     default:
       assert( 0 );
     }
@@ -621,7 +828,7 @@ IDLDep::traverse_dep( Dep* dep )
     traverse_dep( dep->deps[ i ] );
 
   // We need to skip this node if the current object is
-  // an anonymous type or it is defined inside an InterfaceDef
+  // an anonymous type
   bool skip_dep = false;
 
   if( CORBA::is_nil( dep->obj ) ) {
@@ -631,39 +838,53 @@ IDLDep::traverse_dep( Dep* dep )
     skip_dep = k == CORBA::dk_Sequence || k == CORBA::dk_String ||
                k == CORBA::dk_Wstring || k == CORBA::dk_Array ||
                k == CORBA::dk_Fixed;
-
-    /*
-     * We can skip all objects that are declared inside an InterfaceDef.
-     * During the construction of the depencendy graph we have to
-     * include all declarations inside an InterfaceDef in order to
-     * place the interface itself at the right spot. But during
-     * code generation all nested declaration are taken directly
-     * from the interface itself rather than from _ordered_objs.
-     * The traversal of an InterfaceDef during the construction of
-     * the dependency graph also assures that all anonymous types
-     * are added to _ordered_objs.
-     *
-     * This is true for objects in the other container types
-     * StructDef, UnionDef and ExceptionDef, too.
-     */
-    CORBA::Contained_var c = CORBA::Contained::_narrow( dep->obj );
-    if( !CORBA::is_nil( c ) ) {
-      CORBA::Container_var con = c->defined_in();
-      if( con->def_kind() == CORBA::dk_Interface ||
-	  con->def_kind() == CORBA::dk_Value     ||
-	  con->def_kind() == CORBA::dk_Struct    ||
-	  con->def_kind() == CORBA::dk_Union     ||
-	  con->def_kind() == CORBA::dk_Exception )
-	skip_dep = true;
-    }
   }
   
+  /*
+   * If this is a forward decl, and we've already seen the decl itself,
+   * then ignore
+   */
+
+  if (!skip_dep && !dep->visited && dep->iface_as_forward) {
+    for (CORBA::ULong j=0; j<_ordered_objs.length(); j++) {
+      if (_ordered_objs[j]->obj->_is_equivalent (dep->obj)) {
+	dep->visited = 1;
+	break;
+      }
+    }
+  }
+
   if( !skip_dep && !dep->visited ) {
+    CORBA::String_var scope = CORBA::string_dup ("");
+    CORBA::Contained_var cv = CORBA::Contained::_narrow (dep->obj);
+    if (!CORBA::is_nil (cv)) {
+      CORBA::Container_var co = cv->defined_in ();
+      if (co->def_kind() != CORBA::dk_Module &&
+	  co->def_kind() != CORBA::dk_Repository) {
+	cv = CORBA::Contained::_narrow (co);
+	scope = cv->absolute_name ();
+      }
+    }
+    
     CORBA::ULong i = _ordered_objs.length();
-    _ordered_objs.length( i + 1 );
-    _ordered_objs[ i ] = new IRObj;
-    _ordered_objs[ i ]->obj = dep->obj;
-    _ordered_objs[ i ]->iface_as_forward = dep->iface_as_forward;
+
+    /*
+     * If the previous element is the forward declaration for the same
+     * object, just toggle its forward decl switch
+     *
+     */
+
+     if (i > 0 && _ordered_objs[i-1]->obj->_is_equivalent (dep->obj) &&
+	 _ordered_objs[i-1]->iface_as_forward && !dep->iface_as_forward) {
+       _ordered_objs[i-1]->iface_as_forward = 0;
+     }
+     else {
+      _ordered_objs.length( i + 1 );
+      _ordered_objs[ i ] = new IRObj;
+      _ordered_objs[ i ]->obj = dep->obj;
+      _ordered_objs[ i ]->scope = scope.in();
+      _ordered_objs[ i ]->iface_as_forward = dep->iface_as_forward;
+    }
     dep->visited = true;
   }
 }
@@ -696,6 +917,7 @@ IDLDep::add_anonymous_types( Dep* dep )
     _ordered_objs.length( i + 1 );
     _ordered_objs[ i ] = new IRObj;
     _ordered_objs[ i ]->obj = dep->obj;
+    _ordered_objs[ i ]->scope = "";
     _ordered_objs[ i ]->iface_as_forward = dep->iface_as_forward;
     dep->visited = true;
   }
