@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,29 +16,27 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #ifndef __IDLPARSER_H__
 #define __IDLPARSER_H__
 
-#include "parsenode.h"
 #include <CORBA.h>
+#include "parsenode.h"
 #include "db.h"
 #include "const.h"
 
 
 class IDLParser
 {
-private:
 
-  ParseNode*                 _root;
+private:
   ParseNode*                 _current_parse_node;
   bool                       _feed_included_defs;
   CORBA::Repository_ptr      _repo;
   char*                      _version;
-  set<string, less<string> > _forward_dcl;
   set<string, less<string> > _seen_iface;
   DB*                        _db;
 
@@ -48,9 +46,6 @@ private:
   char* gen_repo_id( ParseNode *node );
   char* gen_scoped_name( ParseNode *node );
   
-  void add_forward_dcl( const char* scoped_name );
-  void remove_forward_dcl( const char* scoped_name );
-  bool is_in_forward_dcl( const char* scoped_name );
   bool is_iface_defined( const char* scoped_name );
   
   void scan( CORBA::Container_ptr con, ParseNode *node );
@@ -102,8 +97,7 @@ private:
 			    CORBA::IDLType_ptr switch_type,
 			    CORBA::UnionMemberSeq& members );
  
-  void collectForwardDcl( CORBA::Container_ptr con, ParseNode *node,
-			  CORBA::Boolean is_abstract );
+  void collectForwardDcl( CORBA::Container_ptr con, ParseNode *node );
   void collectInterface( CORBA::Container_ptr con, ParseNode *node );
   void collectRaisesExpr( CORBA::Container_ptr con, ParseNode *node,
 			  CORBA::ExceptionDefSeq &exs );
@@ -122,17 +116,26 @@ private:
 
   CORBA::EnumDef_ptr collectEnum( CORBA::Container_ptr con, ParseNode *node );
   void collectEnumMembers( CORBA::Container_ptr con, 
-			   CORBA::EnumMemberSeq& m, ParseNode *n );
+			   CORBA::EnumDef_ptr e, ParseNode *n );
+
+#ifdef USE_CCM
+  void collectComponent (CORBA::Container_ptr con, ParseNode *node);
+  void collectProvides  (CORBA::Container_ptr con, ParseNode *node);
+  void collectUses      (CORBA::Container_ptr con, ParseNode *node);
+  void collectEvent     (CORBA::Container_ptr con, ParseNode *node);
+  void collectHome      (CORBA::Container_ptr con, ParseNode *node);
+  void collectFactory   (CORBA::Container_ptr con, ParseNode *node);
+#endif
   
   CORBA::IDLType_ptr collectType( CORBA::Container_ptr con, ParseNode *p );
   CORBA::ScopedName collectScopedName( ParseNode *n );
   bool is_TypeCode( ParseNode *n );
   bool is_Principal( ParseNode *n );
   Const collectConst( CORBA::Container_ptr con, ParseNode *p );
+
 public:
   IDLParser( DB &db, bool feed_included_defs = true );
-  ~IDLParser();
-  void collect();
+  void collect(CORBA::Container_ptr, ParseNode *);
 };
 
 
