@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #include <CORBA.h>
@@ -73,7 +73,9 @@ static ParseNodeDoc parseNodeDoc[] = {
   { "t_tilde", 0 },
   { "t_integer_literal", 0 },
   { "t_string_literal", 0 },
+  { "t_wstring_literal", 0 },
   { "t_character_literal", 0 },
+  { "t_wcharacter_literal", 0 },
   { "t_floating_pt_literal", 0 },
   { "t_boolean_literal", 0 },
   { "struct_type", 1 },
@@ -135,7 +137,29 @@ static ParseNodeDoc parseNodeDoc[] = {
   { "init_param_decl", 0 },
   { "t_native", 0 },
   { "t_valuebase", 0 },
-  { "t_abstractbase", 0 }
+  { "t_abstractbase", 0 },
+  { "t_import", 1 },
+  { "imports", 0 },
+  { "local_forward_dcl", 1 },
+  { "local_forward_header", 1 },
+  { "t_typeid", 1 },
+  { "t_typeprefix", 1 },
+  { "t_attr_raises", 0 },
+  { "struct_forward_decl", 1 },
+  { "union_forward_decl", 1 },
+  { "component_dcl", 0 },
+  { "component_forward_dcl", 1 },
+  { "component_header", 1 },
+  { "provides_dcl", 1 },
+  { "uses_dcl", 1 },
+  { "uses_multiple_dcl", 1 },
+  { "emits_dcl", 1 },
+  { "publishes_dcl", 1 },
+  { "consumes_dcl", 1 },
+  { "home_dcl", 0 },
+  { "home_header", 1 },
+  { "factory_dcl", 1 },
+  { "finder_dcl", 1 }
 };
 
 
@@ -195,12 +219,14 @@ bool ParseNode::is_toplevel()
   return _is_toplevel;
 }
 
-void ParseNode::setIntegerLiteral( CORBA::LongLong l )
+void
+ParseNode::setIntegerLiteral( CORBA::ULongLong l )
 {
   _u._int = l;
 }
 
-CORBA::LongLong ParseNode::getIntegerLiteral()
+CORBA::ULongLong
+ParseNode::getIntegerLiteral()
 {
   return _u._int;
 }
@@ -215,6 +241,18 @@ char* ParseNode::getStringLiteral()
   return _u._str;
 }
 
+void
+ParseNode::setWStringLiteral( CORBA::WChar * s )
+{
+  _u._wstr = s;
+}
+
+CORBA::WChar*
+ParseNode::getWStringLiteral()
+{
+  return _u._wstr;
+}
+
 void ParseNode::setCharacterLiteral( CORBA::Char c )
 {
   _u._char = c;
@@ -225,6 +263,18 @@ CORBA::Char ParseNode::getCharacterLiteral()
   return _u._char;
 }
 
+void
+ParseNode::setWCharacterLiteral( CORBA::WChar c )
+{
+  _u._wchar = c;
+}
+
+CORBA::WChar
+ParseNode::getWCharacterLiteral()
+{
+  return _u._wchar;
+}
+
 void ParseNode::setFloatingPtLiteral( CORBA::LongDouble f )
 {
   _u._float = f;
