@@ -34,17 +34,14 @@
 class RandomGenerator_impl : virtual public POA_RandomGenerator
 {
   list<CORBA::Long> random_numbers;
-  Mode mode;
+  RandomGenerator::Mode mode;
   long size;
  public:
-  RandomGenerator_impl (Mode _mode);
+  RandomGenerator_impl (RandomGenerator::Mode _mode);
 
-  CORBA::Long rand ()
-    throw (::CORBA::SystemException,
-    	   ::RandomGenerator::NoSuchNumber);
+  CORBA::Long rand ();
   
-  void add_number (CORBA::Long number)
-    throw (::CORBA::SystemException);
+  void add_number (CORBA::Long number);
 
   void cache_size (long _size);
 };
