@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #define MICO_CONF_IMR
@@ -29,22 +29,17 @@
 #include <iostream.h>
 #include <mico/impl.h>
 #include <mico/util.h>
+#include <mico/throw.h>
 #include <mico/os-math.h>
 #include <mico/template_impl.h>
 
-/*
- * whether or not to allow structural subtyping ala ODP-RM during
- * extraction from any...
- */
-#define ALLOW_SUBTYPING
-
-
 CORBA::Any::Any ()
 {
     thetc = _tc_void;
     checker = new TypeCodeChecker ();
-    ec = new MICO::SimpleEncoder ();
-    dc = new MICO::SimpleDecoder (ec->buffer(), FALSE);
+    ec = new MICO::CDREncoder ();
+    dc = new MICO::CDRDecoder (ec->buffer(), FALSE, ec->byteorder(),
+			       ec->converter(), FALSE);
     ec->valuestate (&value_estate, FALSE);
     dc->valuestate (&value_dstate, FALSE);
     extracted_value = 0;
@@ -54,8 +49,8 @@ CORBA::Any::Any (TypeCode_ptr tc, Buffer *b, Boolean release, int /*dummy*/)
 {
     thetc = CORBA::TypeCode::_duplicate (tc);
     checker = new TypeCodeChecker ();
-    ec = new MICO::SimpleEncoder (b, FALSE);
-    dc = new MICO::SimpleDecoder (b, release);
+    ec = new MICO::CDREncoder (b, FALSE, DefaultEndian, 0, FALSE, 0, TRUE);
+    dc = new MICO::CDRDecoder (b, release, DefaultEndian, 0, FALSE, 0, TRUE);
     ec->valuestate (&value_estate, FALSE);
     dc->valuestate (&value_dstate, FALSE);
     extracted_value = 0;
@@ -80,8 +75,9 @@ CORBA::Any::Any (TypeCode_ptr tc, void *value, Boolean release)
 {
     thetc = _tc_void;
     checker = new TypeCodeChecker ();
-    ec = new MICO::SimpleEncoder ();
-    dc = new MICO::SimpleDecoder (ec->buffer(), FALSE);
+    ec = new MICO::CDREncoder ();
+    dc = new MICO::CDRDecoder (ec->buffer(), FALSE, ec->byteorder(),
+			       ec->converter(), FALSE);
     ec->valuestate (&value_estate, FALSE);
     dc->valuestate (&value_dstate, FALSE);
     extracted_value = 0;
@@ -292,13 +288,13 @@ CORBA::Any::copy_any (Any &a, MapLL &vidmap, Boolean recurse)
 	return (a >>= to_octet (o)) && insert (from_octet (o));
     }
     case tk_any: {
-	Any any;
-	return ((a >>= any) && insert (any));
+	const Any* any;
+	return ((a >>= any) && insert (*any));
     }
     case tk_objref: {
-	Object_ptr obj;
+	Object_var obj;
 	String_var name;
-	return ((a >>= to_object (obj, &name.inout())) &&
+	return ((a >>= to_object (obj, name.out())) &&
 		insert (from_object (obj, name)));
     }
     case tk_enum: {
@@ -307,12 +303,12 @@ CORBA::Any::copy_any (Any &a, MapLL &vidmap, Boolean recurse)
 	break;
     }
     case tk_string: {
-	char *thestring;
+	const char *thestring;
 	return ((a >>= to_string (thestring, atc->length())) &&
 		insert (from_string (thestring, atc->length())));
     }
     case tk_wstring: {
-	wchar_t *thestring;
+	const wchar_t *thestring;
 	return ((a >>= to_wstring (thestring, atc->length())) &&
 		insert (from_wstring (thestring, atc->length())));
     }
@@ -343,7 +339,7 @@ CORBA::Any::copy_any (Any &a, MapLL &vidmap, Boolean recurse)
     switch (atc->kind()) {
     case tk_except: {
 	String_var repoid;
-	if (!a.except_get_begin (repoid.out()) || !except_put_begin (repoid))
+	if (!a.except_get_begin (repoid) || !except_put_begin (repoid))
 	    return FALSE;
 	CORBA::ULong n = atc->member_count();
 	for (CORBA::ULong i = 0; i < n; ++i) {
@@ -384,9 +380,42 @@ CORBA::Any::copy_any (Any &a, MapLL &vidmap, Boolean recurse)
 	ULong seqlen;
 	if (!a.seq_get_begin (seqlen) || !seq_put_begin (seqlen))
 	    return FALSE;
+
+	/*
+	 * __FP__ begin
+	 */
+
+	CORBA::TypeCode_var ctc = atc->content_type ();
+	if (ctc->unalias()->kind() == CORBA::tk_octet) {
+	  CORBA::Buffer * buf = a.dc->buffer ();
+	  if (buf->length() < seqlen) {
+	    return FALSE;
+	  }
+	  /*
+	   * Copy all octets at once
+	   */
+	  ec->put_octets (buf->data(), seqlen);
+	  buf->rseek_rel (seqlen);
+	  /*
+	   * We've just confused the TypeCode Checker
+	   */
+	  Boolean r;
+	  r  = checker->seq_end (TRUE);
+	  r &= a.checker->seq_end (TRUE);
+	  r &= a.dc->seq_end ();
+	  ec->seq_end ();
+	  reset_extracted_value ();
+	  assert (r);
+	  return r;
+	}
+
+	/*
+	 * __FP__ end
+	 */
+
 	for (CORBA::ULong i = 0; i < seqlen; ++i) {
-	    if (!copy_any (a, vidmap))
-		return FALSE;
+	  if (!copy_any (a, vidmap))
+	    return FALSE;
 	}
 	return a.seq_get_end() && seq_put_end();
     }
@@ -394,6 +423,39 @@ CORBA::Any::copy_any (Any &a, MapLL &vidmap, Boolean recurse)
 	ULong arrlen = atc->length();
 	if (!a.array_get_begin() || !array_put_begin())
 	    return FALSE;
+
+	/*
+	 * __FP__ begin
+	 */
+
+	CORBA::TypeCode_var ctc = atc->content_type ();
+	if (ctc->unalias()->kind() == CORBA::tk_octet) {
+	  CORBA::Buffer * buf = a.dc->buffer ();
+	  if (buf->length() < arrlen) {
+	    return FALSE;
+	  }
+	  /*
+	   * Copy all octets at once
+	   */
+	  ec->put_octets (buf->data(), arrlen);
+	  buf->rseek_rel (arrlen);
+	  /*
+	   * We've just confused the TypeCode Checker
+	   */
+	  Boolean r;
+	  r  = checker->arr_end (TRUE);
+	  r &= a.checker->arr_end (TRUE);
+	  r &= a.dc->arr_end ();
+	  ec->arr_end ();
+	  reset_extracted_value ();
+	  assert (r);
+	  return r;
+	}
+
+	/*
+	 * __FP__ end
+	 */
+
 	for (CORBA::ULong i = 0; i < arrlen; ++i) {
 	    if (!copy_any (a, vidmap))
 		return FALSE;
@@ -467,7 +529,7 @@ CORBA::Any::compare_any (Any &a, MapLL &vidmap)
      * if both typecodes are arithmetic types. Otherwise it returns
      * atc.
      */
-    switch (atc->maxtype(mtc)->kind()) {
+    switch (atc->kind()) {
     case tk_null:
     case tk_void:
 	return a.checker->basic (atc) && checker->basic (atc);
@@ -526,11 +588,11 @@ CORBA::Any::compare_any (Any &a, MapLL &vidmap)
 	return (a >>= to_octet (o1)) && (*this >>= to_octet (o2)) && o1 == o2;
     }
     case tk_any: {
-	Any any1, any2;
-	return (a >>= any1) && (*this >>= any2) && any1 == any2;
+	const Any *any1, *any2;
+	return (a >>= any1) && (*this >>= any2) && *any1 == *any2;
     }
     case tk_objref: {
-	Object_ptr objptr1, objptr2;
+	Object_var objptr1, objptr2;
 	return ((a >>= to_object (objptr1)) &&
 		(*this >>= to_object (objptr2)) &&
 		objptr1->_is_equivalent (objptr2));
@@ -541,13 +603,13 @@ CORBA::Any::compare_any (Any &a, MapLL &vidmap)
 	break;
     }
     case tk_string: {
-	char *str1, *str2;
+	const char *str1, *str2;
 	return ((a >>= to_string (str1, atc->length())) &&
 		(*this >>= to_string (str2, mtc->length())) &&
 		!strcmp (str1, str2));
     }
     case tk_wstring: {
-	wchar_t *str1, *str2;
+	const wchar_t *str1, *str2;
 	return ((a >>= to_wstring (str1, atc->length())) &&
 		(*this >>= to_wstring (str2, mtc->length())) &&
 		!xwcscmp (str1, str2));
@@ -573,8 +635,8 @@ CORBA::Any::compare_any (Any &a, MapLL &vidmap)
     }
     case tk_except: {
 	String_var repoid1, repoid2;
-	if (!a.except_get_begin (repoid1.out()) ||
-	    !except_get_begin (repoid2.out()) || strcmp (repoid1, repoid2))
+	if (!a.except_get_begin (repoid1) ||
+	    !except_get_begin (repoid2) || strcmp (repoid1, repoid2))
 	    return FALSE;
 	if (atc->member_count() != mtc->member_count())
 	    return FALSE;
@@ -702,7 +764,7 @@ void
 CORBA::Any::replace (TypeCode_ptr tc, void *value, Boolean release)
 {
     reset ();
-    type (tc);
+    set_type (tc);
 
     if (value) {
 	Buffer b (value);
@@ -723,7 +785,7 @@ void
 CORBA::Any::replace (TypeCode_ptr tc)
 {
     reset ();
-    type (tc);
+    set_type (tc);
 
     switch (tc->unalias()->kind()) {
     case CORBA::tk_void:
@@ -798,7 +860,7 @@ CORBA::Any::type () const
 }
 
 void
-CORBA::Any::type (TypeCode_ptr t)
+CORBA::Any::set_type (TypeCode_ptr t)
 {
     if (checker->level_count() == 0) {
 	value_estate.reset();
@@ -824,6 +886,31 @@ CORBA::Any::type (TypeCode_ptr t)
     }
 }
 
+void
+CORBA::Any::type (TypeCode_ptr t)
+{
+    if (checker->level_count() != 0)
+        mico_throw (CORBA::BAD_INV_ORDER());
+    if (!t->equivalent (tc()))
+        mico_throw (CORBA::BAD_TYPECODE());
+    tc (CORBA::TypeCode::_duplicate (t));
+    checker->restart (tc());
+
+    switch (tc()->unalias()->kind()) {
+    case CORBA::tk_void:
+    case CORBA::tk_null:
+        /*
+         * make sure checker is completed, otherwise marshal()
+         * will fail.
+         */
+        checker->finish();
+        break;
+
+    default:
+        break;
+    }
+}
+
 void
 CORBA::Any::reset ()
 {
@@ -878,10 +965,9 @@ CORBA::Boolean
 CORBA::Any::marshal (DataEncoder &e)
 {
     assert (checker->completed());
-    Any a (tc(), new MICO::SimpleDecoder,
+    Any a (tc(), new MICO::CDRDecoder,
 	   e.clone (e.buffer(), FALSE,
 		    e.converter(), FALSE,
-		    e.wconverter(), FALSE,
 		    e.valuestate(), FALSE));
 
     // cannot use a.prepare_write() since that will clear the buffer ...
@@ -895,12 +981,11 @@ CORBA::Boolean
 CORBA::Any::demarshal (TypeCode_ptr t, DataDecoder &d)
 {
     reset ();
-    type (t);
+    set_type (t);
     Any a (t, d.clone (d.buffer(), FALSE,
 		       d.converter(), FALSE,
-		       d.wconverter(), FALSE,
 		       d.valuestate(), FALSE),
-	   new MICO::SimpleEncoder);
+	   new MICO::CDREncoder);
 
     // cannot use a.pepare_read() ...
     a.value_dstate.reset();
@@ -1642,6 +1727,33 @@ CORBA::Any::operator<<= (Principal_ptr *t)
     insert (t);
 }
 
+CORBA::Boolean
+CORBA::Any::insert (AbstractBase_ptr t)
+{
+  CORBA::StaticAny _sa (CORBA::_stc_AbstractBase, &t);
+  return from_static_any (_sa);
+}
+
+void
+CORBA::Any::operator<<= (AbstractBase_ptr t)
+{
+  insert (t);
+}
+
+CORBA::Boolean
+CORBA::Any::insert (AbstractBase_ptr *t)
+{
+  CORBA::Boolean _ret = insert (*t);
+  CORBA::release (*t);
+  return _ret;
+}
+
+void
+CORBA::Any::operator<<= (AbstractBase_ptr *t)
+{
+  insert (t);
+}
+
 CORBA::Boolean
 CORBA::Any::get_short (Short &s)
 {
@@ -1667,19 +1779,6 @@ CORBA::Any::get_long (Long &l)
     if (checker->basic (_tc_long)) {
 	return dc->get_long (l);
     }
-#ifdef ALLOW_SUBTYPING
-    else {
-	Short s;
-	UShort us;
-	if (get_short (s)) {
-	    l = s;
-	    return TRUE;
-	} else if (get_ushort (us)) {
-	    l = us;
-	    return TRUE;
-	}
-    }
-#endif
     return FALSE;
 }
 
@@ -1702,19 +1801,6 @@ CORBA::Any::get_longlong (LongLong &l)
     if (checker->basic (_tc_longlong)) {
 	return dc->get_longlong (l);
     }
-#ifdef ALLOW_SUBTYPING
-    else {
-	Long _l;
-	ULong _ul;
-	if (get_long (_l)) {
-	    l = _l;
-	    return TRUE;
-	} else if (get_ulong (_ul)) {
-	    l = _ul;
-	    return TRUE;
-	}
-    }
-#endif
     return FALSE;
 }
 
@@ -1756,15 +1842,6 @@ CORBA::Any::get_ulong (ULong &ul)
     if (checker->basic (_tc_ulong)) {
 	return dc->get_ulong (ul);
     }
-#ifdef ALLOW_SUBTYPING
-    else {
-	UShort us;
-	if (get_ushort (us)) {
-	    ul = us;
-	    return TRUE;
-	}
-    }
-#endif
     return FALSE;
 }
 
@@ -1787,15 +1864,6 @@ CORBA::Any::get_ulonglong (ULongLong &ul)
     if (checker->basic (_tc_ulonglong)) {
 	return dc->get_ulonglong (ul);
     }
-#ifdef ALLOW_SUBTYPING
-    else {
-	ULong _ul;
-	if (get_ulong (_ul)) {
-	    ul = _ul;
-	    return TRUE;
-	}
-    }
-#endif
     return FALSE;
 }
 
@@ -1818,19 +1886,6 @@ CORBA::Any::get_float (Float &f)
     if (checker->basic (_tc_float)) {
 	return dc->get_float (f);
     }
-#ifdef ALLOW_SUBTYPING
-    else {
-	Short s;
-	UShort us;
-	if (get_short (s)) {
-	    f = s;
-	    return TRUE;
-	} else if (get_ushort (us)) {
-	    f = us;
-	    return TRUE;
-	}
-    }
-#endif
     return FALSE;
 }
 
@@ -1853,23 +1908,6 @@ CORBA::Any::get_double (Double &d)
     if (checker->basic (_tc_double)) {
 	return dc->get_double (d);
     }
-#ifdef ALLOW_SUBTYPING
-    else {
-	Long l;
-	ULong ul;
-	Float f;
-	if (get_long (l)) {
-	    d = l;
-	    return TRUE;
-	} else if (get_ulong (ul)) {
-	    d = ul;
-	    return TRUE;
-	} else if (get_float (f)) {
-	    d = f;
-	    return TRUE;
-	}
-    }
-#endif
     return FALSE;
 }
 
@@ -1892,27 +1930,6 @@ CORBA::Any::get_longdouble (LongDouble &d)
     if (checker->basic (_tc_longdouble)) {
 	return dc->get_longdouble (d);
     }
-#ifdef ALLOW_SUBTYPING
-    else {
-	LongLong _l;
-	ULongLong _ul;
-	Double _d;
-	if (get_longlong (_l)) {
-	    d = _l;
-	    return TRUE;
-	} else if (get_ulonglong (_ul)) {
-#ifdef _WINDOWS
-	    d = __int64( _ul );
-#else
-	    d = _ul;
-#endif
-	    return TRUE;
-	} else if (get_double (_d)) {
-	    d = _d;
-	    return TRUE;
-	}
-    }
-#endif
     return FALSE;
 }
 
@@ -1930,26 +1947,13 @@ CORBA::Any::operator>>= (LongDouble &d) const
 }
 
 CORBA::Boolean
-CORBA::Any::operator>>= (Any &a) const
-{
-    Any &me = (Any &)*this;
-
-    me.prepare_read ();
-    if (!me.checker->basic (_tc_any) || !me.dc->get_any (a)) {
-	me.rewind ();
-	return FALSE;
-    }
-    return TRUE;
-}
-
-CORBA::Boolean
-CORBA::Any::operator>>= (Any *&a) const
+CORBA::Any::operator>>= (const Any *&a) const
 {
     return to_static_any (_stc_any, (void *&)a);
 }
 
 CORBA::Boolean
-CORBA::Any::operator>>= (char * &s) const
+CORBA::Any::operator>>= (const char * &s) const
 {
     char **p;
     if (to_static_any (_stc_string, (void *&)p)) {
@@ -1960,7 +1964,7 @@ CORBA::Any::operator>>= (char * &s) const
 }
 
 CORBA::Boolean
-CORBA::Any::operator>>= (wchar_t * &s) const
+CORBA::Any::operator>>= (const wchar_t * &s) const
 {
     wchar_t **p;
     if (to_static_any (_stc_wstring, (void *&)p)) {
@@ -2076,7 +2080,7 @@ CORBA::Any::operator>>= (to_object o) const
 
     Object_ptr *p;
     if (to_static_any (_stc_Object, checker->tc(), (void *&)p)) {
-	o.ref = *p;
+	o.ref = CORBA::Object::_duplicate (*p);
 	return TRUE;
     }
     return FALSE;
@@ -2095,7 +2099,7 @@ CORBA::Any::operator>>= (to_abstract_base o) const
 
     AbstractBase_ptr *p;
     if (!to_static_any (_stc_AbstractBase, checker->tc(), (void *&)p)) {
-	o.ref = *p;
+	o.ref = CORBA::AbstractBase::_duplicate (*p);
         return FALSE;
     }
     return TRUE;
@@ -2104,7 +2108,13 @@ CORBA::Any::operator>>= (to_abstract_base o) const
 CORBA::Boolean
 CORBA::Any::operator>>= (Object_ptr &t) const
 {
-    return *this >>= to_object (t);
+    CORBA::Object_var tv;
+    if (!(*this >>= to_object (tv))) {
+        return FALSE;
+    }
+    // to_object extraction returns a duplicate ...
+    t = tv.in();
+    return TRUE;
 }
 
 CORBA::Boolean
@@ -2412,24 +2422,6 @@ CORBA::Any::struct_get_end () const
 {
     Any &me = (Any &)*this;
 
-#ifdef ALLOW_SUBTYPING
-    if (me.checker->struct_end()) {
-	if (!me.dc->struct_end()) {
-	    me.rewind ();
-	    return FALSE;
-	}
-	return TRUE;
-    }
-    if (!me.checker->inside (tk_struct)) {
-	me.rewind ();
-	return FALSE;
-    }
-    Any a;
-    while (!me.checker->level_finished ()) {
-	CORBA::Boolean r = any_get (a);
-	assert (r);
-    }
-#endif
     if (!me.checker->struct_end() || !me.dc->struct_end()) {
 	me.rewind ();
 	return FALSE;
@@ -2493,7 +2485,7 @@ CORBA::Any::any_get (Any &a, Boolean recurse) const
     Any &me = (Any &)*this;
 
     me.prepare_read ();
-    a.type (checker->tc());
+    a.set_type (checker->tc());
     a.prepare_write ();
     if (!a.copy_any (me, recurse)) {
 	me.rewind ();
@@ -2554,7 +2546,7 @@ CORBA::Any::array_get_end () const
 }
 
 CORBA::Boolean
-CORBA::Any::except_get_begin (char *&repoid) const
+CORBA::Any::except_get_begin (String_out repoid) const
 {
     Any &me = (Any &)*this;
 
@@ -2573,24 +2565,6 @@ CORBA::Any::except_get_end () const
 {
     Any &me = (Any &)*this;
 
-#ifdef ALLOW_SUBTYPING
-    if (me.checker->except_end()) {
-	if (!dc->except_end()) {
-	    me.rewind ();
-	    return FALSE;
-	}
-	return TRUE;
-    }
-    if (!me.checker->inside (tk_except)) {
-	me.rewind ();
-	return FALSE;
-    }
-    Any a;
-    while (!me.checker->level_finished()) {
-	CORBA::Boolean r = any_get (a);
-        assert (r);
-    }
-#endif
     if (!me.checker->except_end() || !me.dc->except_end()) {
 	me.rewind ();
 	return FALSE;
@@ -2841,21 +2815,40 @@ CORBA::Any::coerce (CORBA::Any &dst) const
 	return TRUE;
     }
     case CORBA::tk_string: {
+        if (dtype->kind() == CORBA::tk_string) {
+	  // either stype or dtype must be a bounded string
+	  const char *s;
+	  src >>= CORBA::Any::to_string (s, stype->length());
+	  if (dtype->length() != 0 && dtype->length() < strlen (s))
+	    return FALSE;
+	  dst <<= CORBA::Any::from_string (s, dtype->length());
+	  return TRUE;
+        }
 	if (dtype->kind() != CORBA::tk_wstring)
 	    return FALSE;
-	char *s;
+	const char *s;
 	src >>= CORBA::Any::to_string (s, 0);
 	CORBA::WString_var w = CORBA::wstring_alloc (strlen (s));
 	for (size_t i = 0; i < strlen (s); i++)
 	    w[(CORBA::ULong)i] = s[i] & 0xff;
 	w[(CORBA::ULong)strlen(s)] = 0;
-	dst <<= CORBA::Any::from_wstring (w, 0);
+	//dst <<= CORBA::Any::from_wstring (w, 0);
+	dst <<= w.in();
 	return TRUE;
     }
     case CORBA::tk_wstring: {
+        if (dtype->kind() == CORBA::tk_wstring) {
+	  // either stype or dtype must be a bounded wstring
+	  const wchar_t *w;
+	  src >>= CORBA::Any::to_wstring (w, stype->length());
+	  if (dtype->length() != 0 && dtype->length() < xwcslen (w))
+	    return FALSE;
+	  dst <<= CORBA::Any::from_wstring (w, dtype->length());
+	  return TRUE;
+        }
 	if (dtype->kind() != CORBA::tk_string)
 	    return FALSE;
-	wchar_t *w;
+	const wchar_t *w;
 	src >>= CORBA::Any::to_wstring (w, 0);
 	CORBA::String_var s = CORBA::string_alloc (xwcslen (w));
 	for (size_t i = 0; i < xwcslen (w); i++) {
@@ -2933,6 +2926,13 @@ CORBA::Any::coerce (CORBA::Any &dst) const
     }
 
     switch (dtype->kind()) {
+    case CORBA::tk_octet: {
+	CORBA::ULongLong upper = 0xfful;
+	if (!cv.unsigned_range (upper))
+	    return FALSE;
+	dst <<= CORBA::Any::from_octet ((CORBA::Octet)cv.to_unsigned());
+	break;
+    }
     case CORBA::tk_short: {
 	CORBA::LongLong upper = 0x7fffl;
 	CORBA::LongLong lower = - upper - 1;
