@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #define MICO_CONF_IMR
@@ -28,7 +28,6 @@
 #include <string.h>
 #endif
 #include <iostream.h>
-#include <algorithm>
 #include <mico/impl.h>
 #include <mico/intercept.h>
 #include <mico/template_impl.h>
@@ -259,7 +258,7 @@ void
 MICO::BOAServImpl::restore_request (const CORBA::OAServer::ObjSeq &objs)
 {
     for (CORBA::ULong i = 0; i < objs.length(); ++i)
-	_boa->restore_internal (objs[i]);
+	_boa->restore_internal (objs[i].in());
 }
 
 void
@@ -300,7 +299,7 @@ MICO::BOAImpl::BOAImpl (CORBA::ORB_ptr theorb, int &argc, char **argv)
 
     _orb = theorb;
     _theid = 0;
-    _orb->register_oa (this, OAPriorityLocal);
+    _orb->register_oa (this);
 
     vector<CORBA::Octet> hostid = MICO::InetAddress::hostid();
     CORBA::ULong pid = OSMisc::getpid();
@@ -499,10 +498,11 @@ MICO::BOAImpl::find_impl ()
 {
     CORBA::Object_var obj =
 	orb()->resolve_initial_references ("ImplementationRepository");
-    assert (!CORBA::is_nil (obj));
-
     CORBA::ImplRepository_var imr = CORBA::ImplRepository::_narrow (obj);
-    assert (!CORBA::is_nil (imr));
+
+    if (CORBA::is_nil (imr)) {
+      return CORBA::ImplementationDef::_nil ();
+    }
 
     CORBA::ImplRepository::ImplDefSeq_var impls =
 	imr->find_by_name (impl_name());
@@ -672,7 +672,7 @@ MICO::BOAImpl::restore_internal (CORBA::Object_ptr orig)
 
     ReferenceData_var id;
     CORBA::Object_var remote_obj = CORBA::Object::_duplicate (orig);
-    _oamed->restore_obj (obj, remote_obj, id, _oasrv_id);
+    _oamed->restore_obj (obj, remote_obj.inout(), id, _oasrv_id);
     assert (!CORBA::is_nil (remote_obj));
     _oamed->activate_obj (remote_obj, _oasrv_id);
 
@@ -1012,9 +1012,9 @@ MICO::BOAImpl::builtin_invoke (CORBA::Object_ptr o,
 	CORBA::NVList_ptr args;
 	_orb->create_list (0, args);
 	args->add (CORBA::ARG_IN);
-	args->item(0)->value()->type (CORBA::_tc_string);
+	args->item(0)->value()->set_type (CORBA::_tc_string);
 	if (svreq->params (args)) {
-            char *repoid;
+            const char *repoid;
             CORBA::Boolean r = (*args->item(0)->value() >>= repoid);
             assert (r);
 
@@ -1077,7 +1077,7 @@ MICO::BOAImpl::invoke (MsgId msgid,
 	unqueue();
 	assert (!CORBA::is_nil (fwd_obj));
 
-	_orb->answer_invoke (msgid, CORBA::InvokeForward, fwd_obj, req);
+	_orb->answer_invoke (msgid, CORBA::InvokeForward, fwd_obj, req, 0);
 	return TRUE;
     }
     if (!rec) {
@@ -1088,7 +1088,7 @@ MICO::BOAImpl::invoke (MsgId msgid,
         CORBA::OBJECT_NOT_EXIST ex;
 	req->set_out_args (&ex);
 	_orb->answer_invoke (msgid, CORBA::InvokeSysEx,
-			     CORBA::Object::_nil(), req);
+			     CORBA::Object::_nil(), req, 0);
 	return TRUE;
     }
 
@@ -1125,7 +1125,7 @@ MICO::BOAImpl::answer_invoke (MsgId msgid,
                               CORBA::ORBRequest *req,
 			      CORBA::InvokeStatus stat)
 {
-    _orb->answer_invoke (msgid, stat, CORBA::Object::_nil(), req);
+    _orb->answer_invoke (msgid, stat, CORBA::Object::_nil(), req, 0);
 
     if (_amode == CORBA::ImplementationDef::ActivatePerMethod) {
 	// shutdown per-method servers after one (non-builtin) invocation
@@ -1140,7 +1140,9 @@ MICO::BOAImpl::activate (const char *repoid)
     CORBA::Object_var obj =
 	_orb->resolve_initial_references ("ImplementationRepository");
     CORBA::ImplRepository_var imr = CORBA::ImplRepository::_narrow (obj);
-    assert (!CORBA::is_nil (imr));
+    if (CORBA::is_nil (imr)) {
+      return FALSE;
+    }
 
     CORBA::ImplRepository::ImplDefSeq_var ims = imr->find_by_repoid (repoid);
 
@@ -1159,16 +1161,20 @@ MICO::BOAImpl::activate (const char *repoid)
 		continue;
 
 	    // load it
-	    MICODebug::instance()->tracer()
-		<< "BOA: loading shlib " << name.in() << endl;
+	    if (MICO::Logger::IsLogged (MICO::Logger::Info)) {
+	      MICO::Logger::Stream (MICO::Logger::Info)
+		<< "Info: BOA: loading shlib " << name.in() << endl;
+	    }
 
 	    MICO::SharedLib *shlib = new UnixSharedLib (name);
 	    if (!*shlib) {
-		MICODebug::instance()->printer()
-		    << "BOA: could not load shlib " << name.in() << ": "
-		    << shlib->error() << endl;
-		delete shlib;
-		continue;
+	      if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+		MICO::Logger::Stream (MICO::Logger::Error)
+		  << "Info: BOA: could not load shlib " << name.in() << ": "
+		  << shlib->error() << endl;
+	      }
+	      delete shlib;
+	      continue;
 	    }
 
 	    /*
@@ -1181,9 +1187,12 @@ MICO::BOAImpl::activate (const char *repoid)
 	    _shlibs.push_back (shlib);
 
 	    if (!shlib->init ()) {
-		MICODebug::instance()->printer()
-		    << "BOA: could not init shlib " << name.in() << endl;
-		continue;
+	      if (MICO::Logger::IsLogged (MICO::Logger::Warning)) {
+		MICO::Logger::Stream (MICO::Logger::Warning)
+		  << "Warning: BOA: could not init shlib " << name.in()
+		  << endl;
+	      }
+	      continue;
 	    }
 	    return TRUE;
 	}
@@ -1250,7 +1259,7 @@ MICO::BOAImpl::bind (MsgId msgid, const char *repoid,
 CORBA::Boolean
 MICO::BOAImpl::locate (MsgId msgid, CORBA::Object_ptr o)
 {
-    _orb->answer_locate (msgid, CORBA::LocateHere, CORBA::Object::_nil());
+    _orb->answer_locate (msgid, CORBA::LocateHere, CORBA::Object::_nil(), 0);
     return TRUE;
 }
 
