@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #define MICO_CONF_IMR
@@ -40,23 +40,18 @@ CORBA::DataEncoder::DataEncoder ()
     dofree_buf = TRUE;
     conv = 0;
     dofree_conv = FALSE;
-    wconv = 0;
-    dofree_wconv = FALSE;
     vstate = 0;
     dofree_vstate = FALSE;
 }
 
 CORBA::DataEncoder::DataEncoder (Buffer *b, Boolean dofree_b,
-                                 CodesetConv *c, Boolean dofree_c,
-                                 CodesetConv *wc, Boolean dofree_wc,
+                                 CodeSetCoder *c, Boolean dofree_c,
 				 ValueState *vs, Boolean dofree_vs)
 {
     buf = b;
     dofree_buf = dofree_b;
     conv = c;
     dofree_conv = dofree_c;
-    wconv = wc;
-    dofree_wconv = dofree_wc;
     vstate = vs;
     dofree_vstate = dofree_vs;
 }
@@ -67,8 +62,6 @@ CORBA::DataEncoder::~DataEncoder ()
 	delete buf;
     if (dofree_conv && conv)
         delete conv;
-    if (dofree_wconv && wconv)
-        delete wconv;
     if (dofree_vstate && vstate)
 	delete vstate;
 }
@@ -386,7 +379,7 @@ CORBA::DataEncoder::buffer (Buffer *b, Boolean release)
 }
 
 void
-CORBA::DataEncoder::converter (CodesetConv *c, Boolean release)
+CORBA::DataEncoder::converter (CodeSetCoder *c, Boolean release)
 {
     if (dofree_conv && conv)
 	delete conv;
@@ -394,15 +387,6 @@ CORBA::DataEncoder::converter (CodesetConv *c, Boolean release)
     dofree_conv = release;
 }
 
-void
-CORBA::DataEncoder::wconverter (CodesetConv *c, Boolean release)
-{
-    if (dofree_wconv && wconv)
-	delete wconv;
-    wconv = c;
-    dofree_wconv = release;
-}
-
 void
 CORBA::DataEncoder::valuestate (ValueState *vs, Boolean release)
 {
@@ -433,16 +417,13 @@ CORBA::DataEncoder::byteorder (CORBA::ByteOrder bo)
 
 
 CORBA::DataDecoder::DataDecoder (Buffer *b, Boolean dofree_b,
-                                 CodesetConv *c, Boolean dofree_c,
-                                 CodesetConv *wc, Boolean dofree_wc,
+                                 CodeSetCoder *c, Boolean dofree_c,
 				 ValueState *vs, Boolean dofree_vs)
 {
     buf = b;
     dofree_buf = dofree_b;
     conv = c;
     dofree_conv = dofree_c;
-    wconv = wc;
-    dofree_wconv = dofree_wc;
     vstate = vs;
     dofree_vstate = dofree_vs;
 }
@@ -453,8 +434,6 @@ CORBA::DataDecoder::~DataDecoder ()
 	delete buf;
     if (dofree_conv && conv)
         delete conv;
-    if (dofree_wconv && wconv)
-        delete wconv;
     if (dofree_vstate && vstate)
 	delete vstate;
 }
@@ -466,7 +445,7 @@ CORBA::DataDecoder::get_octets (void *data, ULong len)
 }
 
 CORBA::Boolean
-CORBA::DataDecoder::get_string (string &str)
+CORBA::DataDecoder::get_string_stl (string &str)
 {
     char *s;
     if (!get_string (s))
@@ -477,7 +456,7 @@ CORBA::DataDecoder::get_string (string &str)
 }
 
 CORBA::Boolean
-CORBA::DataDecoder::get_string_raw (string &str)
+CORBA::DataDecoder::get_string_raw_stl (string &str)
 {
     char *s;
     if (!get_string_raw (s))
@@ -538,7 +517,7 @@ CORBA::DataDecoder::struct_end ()
 CORBA::Boolean
 CORBA::DataDecoder::except_begin (string &repoid)
 {
-    return get_string (repoid);
+    return get_string_stl (repoid);
 }
 
 CORBA::Boolean
@@ -550,7 +529,26 @@ CORBA::DataDecoder::except_end ()
 CORBA::Boolean
 CORBA::DataDecoder::seq_begin (ULong &l)
 {
-    return get_ulong (l);
+  /*
+   * We want to be protected from ill-formed messages that advertise
+   * an invalid sequence length (e.g. with 1G members). Ultimately,
+   * this would fail at the end of the message while still reading
+   * members, but at that point, the program has already failed from
+   * a memory allocation error.
+   *
+   * We do a crude check here whether the number of elements is larger
+   * than the number of bytes left in the message, which is impossible.
+   */
+
+  if (!get_ulong (l)) {
+    return FALSE;
+  }
+
+  if (l > buf->length()) {
+    return FALSE;
+  }
+
+  return TRUE;
 }
 
 CORBA::Boolean
@@ -571,10 +569,20 @@ CORBA::DataDecoder::encaps_begin (EncapsState &state, ULong &len)
 
     if (!get_ulong (len))
 	return FALSE;
-    --len;
+
+    if (len > buf->length()) {
+      return FALSE;
+    }
 
     buffer()->ralign_base (buffer()->rpos());
 
+    if (len == 0) {
+        // if encapsulation contains no data then we dont need to
+        // know the byteorder ...
+        return TRUE;
+    }
+    --len;
+
     Octet bo;
     if (!get_octet (bo))
 	return FALSE;
@@ -632,13 +640,13 @@ CORBA::DataDecoder::get_indirect_string (string &s)
 	rpos = buf->rpos();
 	if (!buf->rseek_beg (indir))
 	    return FALSE;
-	if (!get_string (s))
+	if (!get_string_stl (s))
 	    return FALSE;
 	buf->rseek_beg (rpos);
     } else {
 	// string
 	buf->rseek_beg (rpos);
-	if (!get_string (s))
+	if (!get_string_stl (s))
 	    return FALSE;
     }
     return TRUE;
@@ -667,7 +675,7 @@ CORBA::DataDecoder::get_indirect_string_seq (vector<string> &v)
 	if (!seq_begin (len))
 	    return FALSE;
 	for (ULong i = 0; i < len; ++i) {
-	    if (!get_string (s))
+	    if (!get_string_stl (s))
 		return FALSE;
 	    v.push_back (s);
 	}
@@ -683,7 +691,7 @@ CORBA::DataDecoder::get_indirect_string_seq (vector<string> &v)
 	if (!seq_begin (len))
 	    return FALSE;
 	for (ULong i = 0; i < len; ++i) {
-	    if (!get_string (s))
+	    if (!get_string_stl (s))
 		return FALSE;
 	    v.push_back (s);
 	}
@@ -750,10 +758,12 @@ CORBA::DataDecoder::value_begin (string &url,
 	return TRUE;
     }
     if (!(0x7fffff00 <= value_tag && value_tag <= 0x7fffffff)) {
-	MICODebug::instance()->tracer()
-	    << "DataDecoder::value_begin(): bad tag: "
-	    << "0x" << hex << value_tag << endl;
-        return FALSE;
+      if (MICO::Logger::IsLogged (MICO::Logger::Info)) {
+	MICO::Logger::Stream (MICO::Logger::Info)
+	  << "Info: DataDecoder::value_begin(): bad tag: "
+	  << "0x" << hex << value_tag << endl;
+      }
+      return FALSE;
     }
     
     // value
@@ -809,7 +819,8 @@ CORBA::DataDecoder::value_begin (string &url,
 		return FALSE;
 	    if (!get_long (tag))
 		return FALSE;
-	    assert (!(0 < tag && tag < 0x7fffff00));
+	    if (0 < tag && tag < 0x7fffff00)
+	      return FALSE;
 	}
 	buf->rseek_beg (rpos);
     }
@@ -870,9 +881,11 @@ CORBA::DataDecoder::value_end (Long finished_value_id, ValueState &state)
 	    if (state.s.data_nesting_level > -tag-1) {
 		state.s.data_nesting_level = -tag-1;
 	    } else {
-		MICODebug::instance()->tracer()
-		    << "DataDecoder::value_end(): bad end tag: "
+	      if (MICO::Logger::IsLogged (MICO::Logger::Info)) {
+		MICO::Logger::Stream (MICO::Logger::Info)
+		    << "Info: DataDecoder::value_end(): bad end tag: "
 		    << tag << " (ignored)" << endl;
+	      }
 	    }
 	} else {
 	    // value or value_ref or nil
@@ -913,9 +926,11 @@ CORBA::DataDecoder::value_end (Long finished_value_id, ValueState &state)
 	    if (state.s.data_nesting_level > -tag-1) {
 		state.s.data_nesting_level = -tag-1;
 	    } else {
-		MICODebug::instance()->tracer()
-		    << "DataDecoder::value_end(): bad end tag: "
-		    << tag << " (ignored)" << endl;
+	      if (MICO::Logger::IsLogged (MICO::Logger::Info)) {
+		MICO::Logger::Stream (MICO::Logger::Info)
+		  << "Info: DataDecoder::value_end(): bad end tag: "
+		  << tag << " (ignored)" << endl;
+	      }
 	    }
 	} else {
 	    // some other tag
@@ -942,7 +957,8 @@ CORBA::DataDecoder::value_end (Long finished_value_id, ValueState &state)
 		return FALSE;
 	    if (!get_long (tag))
 		return FALSE;
-	    assert (!(0 < tag && tag < 0x7fffff00));
+	    if (0 < tag && tag < 0x7fffff00)
+	      return FALSE;
 	}
 	buf->rseek_beg (rpos);
     } else {
@@ -962,7 +978,7 @@ CORBA::DataDecoder::buffer (Buffer *b, Boolean release)
 }
 
 void
-CORBA::DataDecoder::converter (CodesetConv *c, Boolean release)
+CORBA::DataDecoder::converter (CodeSetCoder *c, Boolean release)
 {
     if (dofree_conv && conv)
 	delete conv;
@@ -970,15 +986,6 @@ CORBA::DataDecoder::converter (CodesetConv *c, Boolean release)
     dofree_conv = release;
 }
 
-void
-CORBA::DataDecoder::wconverter (CodesetConv *c, Boolean release)
-{
-    if (dofree_wconv && wconv)
-	delete wconv;
-    wconv = c;
-    dofree_wconv = release;
-}
-
 void
 CORBA::DataDecoder::valuestate (ValueState *vs, Boolean release)
 {
@@ -1005,587 +1012,6 @@ CORBA::DataDecoder::byteorder (CORBA::ByteOrder bo)
 }
 
 
-/**************************** SimpleEncoder ******************************/
-
-
-MICO::SimpleEncoder::SimpleEncoder ()
-{
-}
-
-MICO::SimpleEncoder::SimpleEncoder (CORBA::Buffer *b, CORBA::Boolean dofree_b,
-                                    CORBA::CodesetConv *c,
-                                    CORBA::Boolean dofree_c,
-                                    CORBA::CodesetConv *wc,
-                                    CORBA::Boolean dofree_wc,
-				    ValueState *vs,
-				    CORBA::Boolean dofree_vs)
-    : CORBA::DataEncoder (b, dofree_b, c, dofree_c, wc, dofree_wc,
-			  vs, dofree_vs)
-{
-}
-
-MICO::SimpleEncoder::~SimpleEncoder ()
-{
-}
-
-void
-MICO::SimpleEncoder::put_short (CORBA::Short s)
-{
-    buf->put (&s, sizeof (s));
-}
-
-void
-MICO::SimpleEncoder::put_ushort (CORBA::UShort us)
-{
-    buf->put (&us, sizeof (us));
-}
-
-void
-MICO::SimpleEncoder::put_long (CORBA::Long l)
-{
-    buf->put (&l, sizeof (l));
-}
-
-void
-MICO::SimpleEncoder::put_longlong (CORBA::LongLong l)
-{
-    buf->put (&l, sizeof (l));
-}
-
-void
-MICO::SimpleEncoder::put_ulong (CORBA::ULong ul)
-{
-    buf->put (&ul, sizeof (ul));
-}
-
-void
-MICO::SimpleEncoder::put_ulonglong (CORBA::ULongLong ul)
-{
-    buf->put (&ul, sizeof (ul));
-}
-
-void
-MICO::SimpleEncoder::put_float (CORBA::Float f)
-{
-    buf->put (&f, sizeof (f));
-}
-
-void
-MICO::SimpleEncoder::put_double (CORBA::Double d)
-{
-    buf->put (&d, sizeof (d));
-}
-
-void
-MICO::SimpleEncoder::put_longdouble (CORBA::LongDouble d)
-{
-    buf->put (&d, sizeof (d));
-}
-
-void
-MICO::SimpleEncoder::put_char (CORBA::Char c)
-{
-    buf->put (&c, sizeof (c));
-}
-
-void
-MICO::SimpleEncoder::put_char_raw (CORBA::Char c)
-{
-    buf->put (&c, sizeof (c));
-}
-
-void
-MICO::SimpleEncoder::put_wchar (CORBA::WChar c)
-{
-    buf->put (&c, sizeof (c));
-}
-
-void
-MICO::SimpleEncoder::put_octet (CORBA::Octet o)
-{
-    buf->put (&o, sizeof (o));
-}
-
-void
-MICO::SimpleEncoder::put_boolean (CORBA::Boolean b)
-{
-    buf->put (&b, sizeof (b));
-}
-
-void
-MICO::SimpleEncoder::put_shorts (const CORBA::Short *p, CORBA::ULong l)
-{
-    buf->put (p, l*sizeof(CORBA::Short));
-}
-
-void
-MICO::SimpleEncoder::put_ushorts (const CORBA::UShort *p, CORBA::ULong l)
-{
-    buf->put (p, l*sizeof(CORBA::UShort));
-}
-
-void
-MICO::SimpleEncoder::put_longs (const CORBA::Long *p, CORBA::ULong l)
-{
-    buf->put (p, l*sizeof(CORBA::Long));
-}
-
-void
-MICO::SimpleEncoder::put_longlongs (const CORBA::LongLong *p, CORBA::ULong l)
-{
-    buf->put (p, l*sizeof(CORBA::LongLong));
-}
-
-void
-MICO::SimpleEncoder::put_ulongs (const CORBA::ULong *p, CORBA::ULong l)
-{
-    buf->put (p, l*sizeof(CORBA::ULong));
-}
-
-void
-MICO::SimpleEncoder::put_ulonglongs (const CORBA::ULongLong *p, CORBA::ULong l)
-{
-    buf->put (p, l*sizeof(CORBA::ULongLong));
-}
-
-void
-MICO::SimpleEncoder::put_floats (const CORBA::Float *p, CORBA::ULong l)
-{
-    buf->put (p, l*sizeof(CORBA::Float));
-}
-
-void
-MICO::SimpleEncoder::put_doubles (const CORBA::Double *p, CORBA::ULong l)
-{
-    buf->put (p, l*sizeof(CORBA::Double));
-}
-
-void
-MICO::SimpleEncoder::put_longdoubles (const CORBA::LongDouble *p,
-				      CORBA::ULong l)
-{
-    buf->put (p, l*sizeof(CORBA::LongDouble));
-}
-
-void
-MICO::SimpleEncoder::put_chars (const CORBA::Char *p, CORBA::ULong l)
-{
-    buf->put (p, l);
-}
-
-void
-MICO::SimpleEncoder::put_chars_raw (const CORBA::Char *p, CORBA::ULong l)
-{
-    buf->put (p, l);
-}
-
-void
-MICO::SimpleEncoder::put_wchars (const CORBA::WChar *p, CORBA::ULong l)
-{
-    buf->put (p, l*sizeof(CORBA::WChar));
-}
-
-void
-MICO::SimpleEncoder::put_booleans (const CORBA::Boolean *p, CORBA::ULong l)
-{
-    buf->put (p, l);
-}
-
-void
-MICO::SimpleEncoder::put_string (const char *s)
-{
-    CORBA::ULong len = strlen (s) + 1;
-    put_ulong (len);
-    buf->put (s, len);
-}
-
-void
-MICO::SimpleEncoder::put_string_raw (const char *s)
-{
-    CORBA::ULong len = strlen (s) + 1;
-    put_ulong (len);
-    buf->put (s, len);
-}
-
-void
-MICO::SimpleEncoder::put_wstring (const wchar_t *s)
-{
-    CORBA::ULong len = xwcslen (s) + 1;
-    put_ulong (len);
-    buf->put (s, len*sizeof(wchar_t));
-}
-
-void
-MICO::SimpleEncoder::put_fixed (const FixedBase::FixedValue &value,
-				CORBA::UShort digits,
-				CORBA::Short scale)
-{
-    assert (value.length() == (CORBA::ULong)digits+1);
-    put_octets (&value[0], value.length());
-}
-
-CORBA::DataEncoder *
-MICO::SimpleEncoder::clone () const
-{
-    return new SimpleEncoder (new CORBA::Buffer (*buf), TRUE,
-			      (conv ? conv->clone() : 0), TRUE,
-			      (wconv ? wconv->clone() : 0), TRUE);
-}
-
-CORBA::DataEncoder *
-MICO::SimpleEncoder::clone (CORBA::Buffer *b, CORBA::Boolean dofree_b,
-                            CORBA::CodesetConv *c,
-                            CORBA::Boolean dofree_c,
-                            CORBA::CodesetConv *wc,
-                            CORBA::Boolean dofree_wc,
-			    ValueState *vs,
-			    CORBA::Boolean dofree_vs) const
-{
-    return new SimpleEncoder (b, dofree_b, c, dofree_c, wc, dofree_wc,
-			      vs, dofree_vs);
-}
-
-CORBA::DataDecoder *
-MICO::SimpleEncoder::decoder () const
-{
-    return new SimpleDecoder (new CORBA::Buffer (*buf), TRUE,
-			      (conv ? conv->clone() : 0), TRUE,
-			      (wconv ? wconv->clone() : 0), TRUE,
-			      0, TRUE);
-}
-
-CORBA::DataDecoder *
-MICO::SimpleEncoder::decoder (CORBA::Buffer *b, CORBA::Boolean dofree_b,
-			      CORBA::CodesetConv *c,
-			      CORBA::Boolean dofree_c,
-			      CORBA::CodesetConv *wc,
-			      CORBA::Boolean dofree_wc) const
-{
-    return new SimpleDecoder (b, dofree_b, c, dofree_c, wc, dofree_wc,
-			      0, TRUE);
-}
-
-const char *
-MICO::SimpleEncoder::type () const
-{
-    return "simple";
-}
-
-CORBA::ULong
-MICO::SimpleEncoder::max_alignment () const
-{
-    return 1;
-}
-
-
-/**************************** SimpleDecoder ******************************/
-
-
-MICO::SimpleDecoder::SimpleDecoder ()
-    : CORBA::DataDecoder (new CORBA::Buffer, TRUE)
-{
-}
-
-MICO::SimpleDecoder::SimpleDecoder (CORBA::Buffer *b, CORBA::Boolean dofree_b,
-                                    CORBA::CodesetConv *c,
-                                    CORBA::Boolean dofree_c,
-                                    CORBA::CodesetConv *wc,
-                                    CORBA::Boolean dofree_wc,
-				    ValueState *vs,
-				    CORBA::Boolean dofree_vs)
-    : CORBA::DataDecoder (b, dofree_b, c, dofree_c, wc, dofree_wc,
-			  vs, dofree_vs)
-{
-}
-
-MICO::SimpleDecoder::~SimpleDecoder ()
-{
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_short (CORBA::Short &s)
-{
-    return buf->get (&s, sizeof (s));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_ushort (CORBA::UShort &us)
-{
-    return buf->get (&us, sizeof (us));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_long (CORBA::Long &l)
-{
-    return buf->get (&l, sizeof (l));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_longlong (CORBA::LongLong &l)
-{
-    return buf->get (&l, sizeof (l));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_ulong (CORBA::ULong &ul)
-{
-    return buf->get (&ul, sizeof (ul));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_ulonglong (CORBA::ULongLong &ul)
-{
-    return buf->get (&ul, sizeof (ul));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_float (CORBA::Float &f)
-{
-    return buf->get (&f, sizeof (f));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_double (CORBA::Double &d)
-{
-    return buf->get (&d, sizeof (d));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_longdouble (CORBA::LongDouble &d)
-{
-    return buf->get (&d, sizeof (d));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_char (CORBA::Char &c)
-{
-    return buf->get (&c, sizeof (c));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_char_raw (CORBA::Char &c)
-{
-    return buf->get (&c, sizeof (c));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_wchar (CORBA::WChar &c)
-{
-    return buf->get (&c, sizeof (c));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_octet (CORBA::Octet &o)
-{
-    return buf->get (&o, sizeof (o));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_boolean (CORBA::Boolean &b)
-{
-    return buf->get (&b, sizeof (b));
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_shorts (CORBA::Short *p, CORBA::ULong l)
-{
-    l *= sizeof (CORBA::Short);
-    return buf->get (p, l);
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_ushorts (CORBA::UShort *p, CORBA::ULong l)
-{
-    l *= sizeof (CORBA::UShort);
-    return buf->get (p, l);
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_longs (CORBA::Long *p, CORBA::ULong l)
-{
-    l *= sizeof (CORBA::Long);
-    return buf->get (p, l);
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_longlongs (CORBA::LongLong *p, CORBA::ULong l)
-{
-    l *= sizeof (CORBA::LongLong);
-    return buf->get (p, l);
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_ulongs (CORBA::ULong *p, CORBA::ULong l)
-{
-    l *= sizeof (CORBA::ULong);
-    return buf->get (p, l);
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_ulonglongs (CORBA::ULongLong *p, CORBA::ULong l)
-{
-    l *= sizeof (CORBA::ULongLong);
-    return buf->get (p, l);
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_floats (CORBA::Float *p, CORBA::ULong l)
-{
-    l *= sizeof (CORBA::Float);
-    return buf->get (p, l);
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_doubles (CORBA::Double *p, CORBA::ULong l)
-{
-    l *= sizeof (CORBA::Double);
-    return buf->get (p, l);
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_longdoubles (CORBA::LongDouble *p,
-				      CORBA::ULong l)
-{
-    l *= sizeof (CORBA::LongDouble);
-    return buf->get (p, l);
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_chars (CORBA::Char *p, CORBA::ULong l)
-{
-    return buf->get (p, l);
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_chars_raw (CORBA::Char *p, CORBA::ULong l)
-{
-    return buf->get (p, l);
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_wchars (CORBA::WChar *p, CORBA::ULong l)
-{
-    l *= sizeof (CORBA::WChar);
-    return buf->get (p, l);
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_booleans (CORBA::Boolean *p, CORBA::ULong l)
-{
-    return buf->get (p, l);
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_string (char *&s)
-{
-    CORBA::ULong len;
-    if (!get_ulong (len))
-	return FALSE;
-    assert (len >= 1);
-    s = CORBA::string_alloc (len-1);
-    if (!buf->get (s, len)) {
-	CORBA::string_free (s);
-	return FALSE;
-    }
-    assert (s[len-1] == 0);
-    return TRUE;
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_string_raw (char *&s)
-{
-    CORBA::ULong len;
-    if (!get_ulong (len))
-	return FALSE;
-    assert (len >= 1);
-    s = CORBA::string_alloc (len-1);
-    if (!buf->get (s, len)) {
-	CORBA::string_free (s);
-	return FALSE;
-    }
-    assert (s[len-1] == 0);
-    return TRUE;
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_wstring (wchar_t *&s)
-{
-    CORBA::ULong len;
-    if (!get_ulong (len))
-	return FALSE;
-    assert (len >= 1);
-    s = CORBA::wstring_alloc (len-1);
-    if (!buf->get (s, len*sizeof(wchar_t))) {
-	CORBA::wstring_free (s);
-	return FALSE;
-    }
-    assert (s[len-1] == 0);
-    return TRUE;
-}
-
-CORBA::Boolean
-MICO::SimpleDecoder::get_fixed (FixedBase::FixedValue &value,
-				CORBA::UShort digits,
-				CORBA::Short scale)
-{
-    value.length (digits+1);
-    return get_octets (&value[0], value.length());
-}
-
-CORBA::DataDecoder *
-MICO::SimpleDecoder::clone () const
-{
-    return new SimpleDecoder (new CORBA::Buffer (*buf), TRUE,
-			      (conv ? conv->clone() : 0), TRUE,
-			      (wconv ? wconv->clone() : 0), TRUE);
-}
-
-CORBA::DataDecoder *
-MICO::SimpleDecoder::clone (CORBA::Buffer *b, CORBA::Boolean dofree_b,
-                            CORBA::CodesetConv *c,
-                            CORBA::Boolean dofree_c,
-                            CORBA::CodesetConv *wc,
-                            CORBA::Boolean dofree_wc,
-			    ValueState *vs,
-			    CORBA::Boolean dofree_vs) const
-{
-    return new SimpleDecoder (b, dofree_b, c, dofree_c, wc, dofree_wc,
-			      vs, dofree_vs);
-}
-
-CORBA::DataEncoder *
-MICO::SimpleDecoder::encoder () const
-{
-    return new SimpleEncoder (new CORBA::Buffer (*buf), TRUE,
-			      (conv ? conv->clone() : 0), TRUE,
-			      (wconv ? wconv->clone() : 0), TRUE,
-			      0, TRUE);
-}
-
-CORBA::DataEncoder *
-MICO::SimpleDecoder::encoder (CORBA::Buffer *b, CORBA::Boolean dofree_b,
-			      CORBA::CodesetConv *c,
-			      CORBA::Boolean dofree_c,
-			      CORBA::CodesetConv *wc,
-			      CORBA::Boolean dofree_wc) const
-{
-    return new SimpleEncoder (b, dofree_b, c, dofree_c, wc, dofree_wc,
-			      0, TRUE);
-}
-
-const char *
-MICO::SimpleDecoder::type () const
-{
-    return "simple";
-}
-
-CORBA::ULong
-MICO::SimpleDecoder::max_alignment () const
-{
-    return 1;
-}
-
-
 /**************************** CDREncoder ******************************/
 
 static inline void swap2 (void *d, const void *s)
@@ -1646,10 +1072,9 @@ MICO::CDREncoder::CDREncoder ()
 
 MICO::CDREncoder::CDREncoder (CORBA::Buffer *b, CORBA::Boolean dofree_b,
 			      CORBA::ByteOrder _bo,
-                              CORBA::CodesetConv *c, CORBA::Boolean dofree_c,
-                              CORBA::CodesetConv *wc, CORBA::Boolean dofree_wc,
+                              CORBA::CodeSetCoder *c, CORBA::Boolean dofree_c,
 			      ValueState *vs, CORBA::Boolean dofree_vs)
-    : CORBA::DataEncoder (b, dofree_b, c, dofree_c, wc, dofree_wc,
+    : CORBA::DataEncoder (b, dofree_b, c, dofree_c,
 			  vs, dofree_vs)
 {
 #ifdef HAVE_BYTEORDER_BE
@@ -1811,15 +1236,11 @@ MICO::CDREncoder::put_longdouble (CORBA::LongDouble d)
 void
 MICO::CDREncoder::put_char (CORBA::Char c)
 {
-    if (!conv || conv->from()->id() == conv->to()->id()) {
-        buf->put1 (&c);
-        return;
-    }
-
-    CORBA::Long written = conv->encode (&c, 1, *this, FALSE);
-
-    // XXX multibyte chars are not allowed ...
-    assert (written == 1);
+  if (!conv) {
+    buf->put1 (&c);
+    return;
+  }
+  conv->put_char (*this, c);
 }
 
 void
@@ -1831,16 +1252,11 @@ MICO::CDREncoder::put_char_raw (CORBA::Char c)
 void
 MICO::CDREncoder::put_wchar (CORBA::WChar c)
 {
-    if (!wconv) {
-        // XXX fallback codeset is UTF-16, encoded as unsigned short
-        put_ushort ((CORBA::UShort)c);
-        return;
-    }
-
-    CORBA::Long written = wconv->encode (&c, 1, *this, FALSE);
-
-    // XXX multibyte chars are not allowed ...
-    assert (written == 1);
+  if (!conv) {
+    buf->put (&c, sizeof (CORBA::WChar));
+    return;
+  }
+  conv->put_wchar (*this, c);
 }
 
 void
@@ -1969,8 +1385,11 @@ MICO::CDREncoder::put_longdoubles (const CORBA::LongDouble *p, CORBA::ULong l)
 void
 MICO::CDREncoder::put_chars (const CORBA::Char *p, CORBA::ULong l)
 {
-    for (CORBA::Long i = l; --i >= 0; ++p)
-	put_char (*p);
+  if (!conv) {
+    buf->put (p, l);
+    return;
+  }
+  conv->put_chars (*this, p, l);
 }
 
 void
@@ -1982,8 +1401,11 @@ MICO::CDREncoder::put_chars_raw (const CORBA::Char *p, CORBA::ULong l)
 void
 MICO::CDREncoder::put_wchars (const CORBA::WChar *p, CORBA::ULong l)
 {
-    for (CORBA::Long i = l; --i >= 0; ++p)
-	put_wchar (*p);
+  if (!conv) {
+    buf->put (p, l * sizeof (CORBA::WChar));
+    return;
+  }
+  conv->put_wchars (*this, p, l);
 }
 
 void
@@ -1995,18 +1417,14 @@ MICO::CDREncoder::put_booleans (const CORBA::Boolean *p, CORBA::ULong l)
 void
 MICO::CDREncoder::put_string (const char *s)
 {
-    if (!conv || conv->from()->id() == conv->to()->id()) {
-        CORBA::ULong len = strlen (s) + 1;
-        put_ulong (len);
-        put_octets (s, len);
-        return;
-    }
-
-    DelayedSeqState state;
-    delayed_seq_begin (state);
-    CORBA::Long written = conv->encode ((CORBA::Char *)s, strlen (s), *this);
-    assert (written >= 0);
-    delayed_seq_end (state, written);
+  if (!conv) {
+    CORBA::ULong len = strlen (s) + 1;
+    put_ulong (len);
+    put_octets (s, len);
+    return;
+  }
+  
+  conv->put_string (*this, s, 0);
 }
 
 void
@@ -2020,20 +1438,14 @@ MICO::CDREncoder::put_string_raw (const char *s)
 void
 MICO::CDREncoder::put_wstring (const wchar_t *s)
 {
-    if (!wconv) {
-        CORBA::ULong len = xwcslen (s) + 1;
-        put_ulong (len);
-        for (CORBA::ULong i = 0; i < len; ++i)
-            // fallback codeset is UTF-16
-            put_ushort (s[i]);
-        return;
+    if (!conv) {
+      CORBA::ULong len = xwcslen (s) + 1;
+      put_ulong (len);
+      buf->put (s, len * sizeof (wchar_t));
+      return;
     }
 
-    DelayedSeqState state;
-    delayed_seq_begin (state);
-    CORBA::Long written = wconv->encode ((CORBA::WChar *)s, xwcslen (s),*this);
-    assert (written >= 0);
-    delayed_seq_end (state, written);
+    conv->put_wstring (*this, s, 0);
 }
 
 void
@@ -2064,18 +1476,15 @@ MICO::CDREncoder::clone () const
 {
     return new CDREncoder (new CORBA::Buffer (*buf), TRUE, data_bo,
 			   (conv ? conv->clone() : 0), TRUE,
-			   (wconv ? wconv->clone() : 0), TRUE,
 			   (vstate ? new ValueState (*vstate) : 0), TRUE);
 }
 
 CORBA::DataEncoder *
 MICO::CDREncoder::clone (CORBA::Buffer *b, CORBA::Boolean dofree_b,
-                         CORBA::CodesetConv *c, CORBA::Boolean dofree_c,
-                         CORBA::CodesetConv *wc,
-			 CORBA::Boolean dofree_wc,
+                         CORBA::CodeSetCoder *c, CORBA::Boolean dofree_c,
 			 ValueState *vs, CORBA::Boolean dofree_vs) const
 {
-    return new CDREncoder (b, dofree_b, data_bo, c, dofree_c, wc, dofree_wc,
+    return new CDREncoder (b, dofree_b, data_bo, c, dofree_c,
 			   vs, dofree_vs);
 }
 
@@ -2084,18 +1493,15 @@ MICO::CDREncoder::decoder () const
 {
     return new CDRDecoder (new CORBA::Buffer (*buf), TRUE, data_bo,
 			   (conv ? conv->clone() : 0), TRUE,
-			   (wconv ? wconv->clone() : 0), TRUE,
 			   0, TRUE);
 }
 
 CORBA::DataDecoder *
 MICO::CDREncoder::decoder (CORBA::Buffer *b, CORBA::Boolean dofree_b,
-			   CORBA::CodesetConv *c, CORBA::Boolean dofree_c,
-			   CORBA::CodesetConv *wc,
-			   CORBA::Boolean dofree_wc) const
+			   CORBA::CodeSetCoder *c, CORBA::Boolean dofree_c
+			   ) const
 {
-    return new CDRDecoder (b, dofree_b, data_bo, c, dofree_c, wc, dofree_wc,
-			   0, TRUE);
+    return new CDRDecoder (b, dofree_b, data_bo, c, dofree_c, 0, 1);
 }
 
 const char *
@@ -2139,10 +1545,9 @@ MICO::CDRDecoder::CDRDecoder ()
 
 MICO::CDRDecoder::CDRDecoder (CORBA::Buffer *b, CORBA::Boolean dofree_b,
 			      CORBA::ByteOrder _bo,
-                              CORBA::CodesetConv *c, CORBA::Boolean dofree_c,
-                              CORBA::CodesetConv *wc, CORBA::Boolean dofree_wc,
+                              CORBA::CodeSetCoder *c, CORBA::Boolean dofree_c,
 			      ValueState *vs, CORBA::Boolean dofree_vs)
-    : CORBA::DataDecoder (b, dofree_b, c, dofree_c, wc, dofree_wc,
+    : CORBA::DataDecoder (b, dofree_b, c, dofree_c,
 			  vs, dofree_vs)
 {
 #ifdef HAVE_BYTEORDER_BE
@@ -2325,21 +1730,11 @@ MICO::CDRDecoder::get_longdouble (CORBA::LongDouble &d)
 CORBA::Boolean
 MICO::CDRDecoder::get_char (CORBA::Char &c)
 {
-    if (!conv || conv->from()->id() == conv->to()->id()) {
-        return buf->get1 (&c);
-    }
+  if (!conv) {
+    return buf->get1 (&c);
+  }
 
-    assert (conv->to()->max_codepoints() <= 25);
-    CORBA::Char to[50];
-
-    // XXX only one byte characters allowed ...
-    CORBA::Long read = conv->decode (*this, 1, to, FALSE);
-    if (read < 0)
-	return FALSE;
-
-    c = *to;
-
-    return TRUE;
+  return conv->get_char (*this, c);
 }
 
 CORBA::Boolean
@@ -2351,26 +1746,11 @@ MICO::CDRDecoder::get_char_raw (CORBA::Char &c)
 CORBA::Boolean
 MICO::CDRDecoder::get_wchar (CORBA::WChar &c)
 {
-    if (!wconv) {
-        // XXX fallback codeset is UTF-16, encoded as unsigned short
-        CORBA::UShort us;
-        if (!get_ushort (us))
-            return FALSE;
-        c = (CORBA::WChar)us;
-        return TRUE;
-    }
+  if (!conv) {
+    return buf->get (&c, sizeof (CORBA::WChar));
+  }
 
-    assert (wconv->to()->max_codepoints() <= 25);
-    CORBA::WChar to[50];
-
-    // XXX only one byte characters allowed ...
-    CORBA::Long read = wconv->decode (*this, 1, to, FALSE);
-    if (read < 0)
-	return FALSE;
-
-    c = *to;
-
-    return TRUE;
+  return conv->get_wchar (*this, c);
 }
 
 CORBA::Boolean
@@ -2538,11 +1918,10 @@ MICO::CDRDecoder::get_longdoubles (CORBA::LongDouble *p, CORBA::ULong l)
 CORBA::Boolean
 MICO::CDRDecoder::get_chars (CORBA::Char *p, CORBA::ULong l)
 {
-    for (CORBA::Long i = l; --i >= 0; ++p) {
-	if (!get_char (*p))
-	    return FALSE;
-    }
-    return TRUE;
+  if (!conv) {
+    return buf->get (p, l);
+  }
+  return conv->get_chars (*this, p, l);
 }
 
 CORBA::Boolean
@@ -2554,11 +1933,10 @@ MICO::CDRDecoder::get_chars_raw (CORBA::Char *p, CORBA::ULong l)
 CORBA::Boolean
 MICO::CDRDecoder::get_wchars (CORBA::WChar *p, CORBA::ULong l)
 {
-    for (CORBA::Long i = l; --i >= 0; ++p) {
-	if (!get_wchar (*p))
-	    return FALSE;
-    }
-    return TRUE;
+  if (!conv) {
+    return buf->get (p, l * sizeof (CORBA::WChar));
+  }
+  return conv->get_wchars (*this, p, l);
 }
 
 CORBA::Boolean
@@ -2568,79 +1946,68 @@ MICO::CDRDecoder::get_booleans (CORBA::Boolean *p, CORBA::ULong l)
 }
 
 CORBA::Boolean
-MICO::CDRDecoder::get_string (char *&s)
+MICO::CDRDecoder::get_string (CORBA::String_out s)
 {
+  if (!conv) {
     CORBA::ULong len;
     if (!get_ulong (len))
         return FALSE;
-    assert (len >= 1);
+    if (len == 0 || len > buf->length())
+      return FALSE;
 
-    if (!conv || conv->from()->id() == conv->to()->id()) {
-        s = CORBA::string_alloc (len-1);
-        if (!buf->get (s, len)) {
-            CORBA::string_free (s);
-            return FALSE;
-        }
-        assert (s[len-1] == 0);
-        return TRUE;
+    char * p = CORBA::string_alloc (len-1);
+    if (!buf->get (p, len)) {
+      CORBA::string_free (p);
+      return FALSE;
     }
-
-    s = CORBA::string_alloc (conv->to()->guess_size (len-1));
-
-    CORBA::Long read = conv->decode (*this, len, (CORBA::Char *)s, FALSE);
-    if (read < 0)
-	return FALSE;
-
+    s = p;
     return TRUE;
+  }
+
+  return conv->get_string (*this, s, 0);
 }
 
 CORBA::Boolean
-MICO::CDRDecoder::get_string_raw (char *&s)
+MICO::CDRDecoder::get_string_raw (CORBA::String_out s)
 {
     CORBA::ULong len;
     if (!get_ulong (len))
         return FALSE;
-    assert (len >= 1);
+    if (len == 0 || len > buf->length())
+      return FALSE;
 
     s = CORBA::string_alloc (len-1);
-    if (!buf->get (s, len)) {
+    if (!buf->get (s.ptr(), len)) {
 	CORBA::string_free (s);
 	return FALSE;
     }
-    assert (s[len-1] == 0);
+    if (s[len-1] != 0) {
+      CORBA::string_free (s);
+      return FALSE;
+    }
     return TRUE;
 }
 
 CORBA::Boolean
-MICO::CDRDecoder::get_wstring (wchar_t *&s)
+MICO::CDRDecoder::get_wstring (CORBA::WString_out s)
 {
+  if (!conv) {
     CORBA::ULong len;
     if (!get_ulong (len))
         return FALSE;
-    assert (len >= 1);
+    if (len == 0 || len*sizeof(CORBA::WChar) > buf->length())
+      return FALSE;
 
-    if (!wconv) {
-        s = CORBA::wstring_alloc (len-1);
-	CORBA::UShort u;
-        for (CORBA::ULong i = 0; i < len; ++i) {
-            // default code set if UTF-16 ...
-            if (!get_ushort (u)) {
-                CORBA::wstring_free (s);
-                return FALSE;
-            }
-            s[i] = u;
-        }
-        assert (s[len-1] == 0);
-        return TRUE;
+    CORBA::WChar * p = CORBA::wstring_alloc (len-1);
+    if (!buf->get (p, len * sizeof (CORBA::WChar))) {
+      CORBA::wstring_free (p);
+      return FALSE;
     }
-
-    s = CORBA::wstring_alloc (wconv->to()->guess_size (len-1));
-
-    CORBA::Long read = wconv->decode (*this, len, (CORBA::WChar *)s, FALSE);
-    if (read < 0)
-	return FALSE;
-
+    s = p;
     return TRUE;
+  }
+
+  return conv->get_wstring (*this, s, 0);
 }
 
 CORBA::Boolean
@@ -2683,39 +2050,31 @@ MICO::CDRDecoder::clone () const
 {
     return new CDRDecoder (new CORBA::Buffer (*buf), TRUE, data_bo,
 			   (conv ? conv->clone() : 0), TRUE,
-			   (wconv ? wconv->clone() : 0), TRUE,
 			   (vstate ? new ValueState (*vstate) : 0), TRUE);
 }
 
 CORBA::DataDecoder *
 MICO::CDRDecoder::clone (CORBA::Buffer *b, CORBA::Boolean dofree_b,
-                         CORBA::CodesetConv *c, CORBA::Boolean dofree_c,
-                         CORBA::CodesetConv *wc,
-			 CORBA::Boolean dofree_wc,
-			 ValueState *vs,
-			 CORBA::Boolean dofree_vs) const
+                         CORBA::CodeSetCoder *c, CORBA::Boolean dofree_c,
+			 ValueState *vs, CORBA::Boolean dofree_vs) const
 {
-    return new CDRDecoder (b, dofree_b, data_bo, c, dofree_c, wc, dofree_wc,
-			   vs, dofree_vs);
+    return new CDRDecoder (b, dofree_b, data_bo, c, dofree_c, vs, dofree_vs);
 }
 
 CORBA::DataEncoder *
 MICO::CDRDecoder::encoder () const
 {
-    return new CDREncoder (new CORBA::Buffer (*buf), TRUE, data_bo,
-			   (conv ? conv->clone() : 0), TRUE,
-			   (wconv ? wconv->clone() : 0), TRUE,
-			   0, TRUE);
+  return new CDREncoder (new CORBA::Buffer (*buf), TRUE, data_bo,
+			 (conv ? conv->clone() : 0), TRUE,
+			 0, TRUE);
 }
 
 CORBA::DataEncoder *
 MICO::CDRDecoder::encoder (CORBA::Buffer *b, CORBA::Boolean dofree_b,
-			   CORBA::CodesetConv *c, CORBA::Boolean dofree_c,
-			   CORBA::CodesetConv *wc,
-			   CORBA::Boolean dofree_wc) const
+			   CORBA::CodeSetCoder *c, CORBA::Boolean dofree_c
+			   ) const
 {
-    return new CDREncoder (b, dofree_b, data_bo, c, dofree_c, wc, dofree_wc,
-			   0, TRUE);
+    return new CDREncoder (b, dofree_b, data_bo, c, dofree_c, 0, 1);
 }
 
 const char *
