@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #define MICO_CONF_IMR
@@ -36,7 +36,7 @@
 #include <mico/os-misc.h>
 #include <mico/basic_seq.h>
 
-#ifdef _WINDOWS
+#if defined(_WINDOWS) && !defined(__MINGW32__) && !defined(__BORLANDC__)
 #include <atlbase.h>
 #if ( !defined(_MSC_VER) || !defined(_ATL_VER) || (_MSC_VER < 1100) || (_ATL_VER < 0x0202))
 #error "You need Visual C++ 5.0 with Service Pack 3"
@@ -52,13 +52,13 @@
 /*****************************  NamedValue  ***************************/
 
 
-CORBA::NamedValue::NamedValue (const char *name, const Any &val, Flags flags)
-    : _n (name),  _v (val), _f (flags)
+CORBA::NamedValue::NamedValue (const char *nameval, const Any &val, Flags flagsval)
+    : _n (nameval),  _v (val), _f (flagsval)
 {
 }
 
-CORBA::NamedValue::NamedValue (const char *name, Flags flags)
-    : _n (name), _f (flags)
+CORBA::NamedValue::NamedValue (const char *nameval, Flags flagsval)
+    : _n (nameval), _f (flagsval)
 {
 }
 
@@ -85,11 +85,9 @@ CORBA::NamedValue::~NamedValue ()
 /*****************************  NVList  ***************************/
 
 
-CORBA::NVList::NVList (ULong count)
+CORBA::NVList::NVList (ULong countval)
 {
-    _vec.reserve (count);
-    for (CORBA::ULong i = 0; i < count; ++i)
-	_vec.push_back (new NamedValue);
+    _vec.reserve (countval);
 }
 
 CORBA::NVList::~NVList ()
@@ -480,8 +478,8 @@ CORBA::Request::Request ()
 
 CORBA::Request::Request (Object_ptr o, Context_ptr c,
 			 const char *op,
-			 NVList_ptr arguments,
-			 NamedValue_ptr result,
+			 NVList_ptr argumentsval,
+			 NamedValue_ptr resultval,
 			 Flags f)
 {
     if (CORBA::is_nil (o) || !op)
@@ -496,8 +494,8 @@ CORBA::Request::Request (Object_ptr o, Context_ptr c,
     _object = Object::_duplicate (o);
     _context = Context::_duplicate (c);
     _opname = op;
-    _args = NVList::_duplicate (arguments);
-    _res = NamedValue::_duplicate (result);
+    _args = NVList::_duplicate (argumentsval);
+    _res = NamedValue::_duplicate (resultval);
     _flags = f;
 
     _invoke_pending = FALSE;
@@ -521,8 +519,8 @@ CORBA::Request::Request (Object_ptr o, Context_ptr c,
 CORBA::Request::Request (Object_ptr o,
 			 Context_ptr c,
 			 const char *op,
-			 NVList_ptr arguments,
-			 NamedValue_ptr result,
+			 NVList_ptr argumentsval,
+			 NamedValue_ptr resultval,
 			 ExceptionList_ptr el,
 			 ContextList_ptr cl,
 			 Flags f)
@@ -539,8 +537,8 @@ CORBA::Request::Request (Object_ptr o,
     _object = Object::_duplicate (o);
     _context = Context::_duplicate (c);
     _opname = op;
-    _args = NVList::_duplicate (arguments);
-    _res = NamedValue::_duplicate (result);
+    _args = NVList::_duplicate (argumentsval);
+    _res = NamedValue::_duplicate (resultval);
     _flags = f;
     _elist = ExceptionList::_duplicate (el);
     _clist = ContextList::_duplicate (cl);
@@ -617,7 +615,7 @@ CORBA::Request::decode_user_exception ()
     Exception *ex = _environm->exception();
     if (ex && _elist->count() > 0) {
 	UnknownUserException *uuex =
-	  UnknownUserException::EXCEPTION_DOWNCAST (ex);
+	  UnknownUserException::_downcast (ex);
 	if (uuex) {
 	    // decode UnknownUserException using ExceptionList ...
 	    const char *repoid = uuex->_except_repoid ();
@@ -638,8 +636,9 @@ CORBA::Request::callback (ORB_ptr, ULong, ORBCallback::Event ev)
 {
     assert (ev == ORBCallback::Invoke);
     if (_cb) {
-        _cb->callback (this, RequestCallback::RequestDone);
+        RequestCallback *tmpcb = _cb;
         _cb = 0;
+        tmpcb->callback (this, RequestCallback::RequestDone);
     }
 }
 
@@ -691,7 +690,7 @@ CORBA::Request::add_out_arg (const char *name)
 void
 CORBA::Request::set_return_type (TypeCode_ptr tc)
 {
-    result()->value()->type (tc);
+    result()->value()->set_type (tc);
 }
 
 CORBA::Any &
@@ -768,7 +767,7 @@ CORBA::Request::get_response (Boolean block)
 	return;
 
     ORB_ptr orb = _object->_orbnc();
-    InvokeStatus rs;
+    InvokeStatus rs = InvokeOk;
 
     while (_invoke_pending) {
 	Object_var nobj;
@@ -778,7 +777,7 @@ CORBA::Request::get_response (Boolean block)
 	    CORBA::Boolean r = orb->wait (_msgid);
 	    assert (r);
 	} else {
-	    if (!orb->wait (_msgid, 1))
+	    if (!orb->wait (_msgid, 0))
 		return;
 	}
 
@@ -792,7 +791,8 @@ CORBA::Request::get_response (Boolean block)
 	    return;
 	}
 
-	rs = orb->get_invoke_reply (_msgid, nobj, dummy);
+	GIOP::AddressingDisposition ad;
+	rs = orb->get_invoke_reply (_msgid, nobj, dummy, ad);
 
 	switch (rs) {
 	case InvokeForward:
@@ -821,13 +821,16 @@ CORBA::Request::get_response (Boolean block)
 		 * invocation.
 		 */
 		CORBA::SystemException *sysex =
-		    CORBA::SystemException::EXCEPTION_DOWNCAST (env()->exception());
+		    CORBA::SystemException::_downcast (env()->exception());
 		assert (sysex);
-		if (sysex->completed() == CORBA::COMPLETED_NO) {
+		if (sysex->completed() == CORBA::COMPLETED_NO &&
+                    (CORBA::COMM_FAILURE::_downcast (sysex) ||
+                     CORBA::TRANSIENT::_downcast (sysex) ||
+                     CORBA::OBJECT_NOT_EXIST::_downcast (sysex))) {
 		    _object->_unforward();
 		    env()->clear();
 		    _msgid = orb->invoke_async (_object, _orbreq,
-					       Principal::_nil());
+                                                Principal::_nil());
 		    break;
 		}
 	    }
@@ -839,6 +842,11 @@ CORBA::Request::get_response (Boolean block)
 	    _invoke_pending = FALSE;
 	    break;
 
+	case InvokeAddrDisp:
+	    _object->_ior_fwd()->addressing_disposition (ad);
+	    _msgid = orb->invoke_async (_object, _orbreq, Principal::_nil());
+	    break;
+
 	default:
 	    assert (0);
 	}
