@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #define MICO_CONF_IMR
@@ -415,6 +415,16 @@ DynAny_impl::insert_dyn_any (DynamicAny::DynAny_ptr value)
     insert_any (a.in());
 }
 
+void
+DynAny_impl::insert_abstract (CORBA::AbstractBase_ptr value)
+{
+  CORBA::TypeCode_var tc = _elements[_index]->type ();
+  CORBA::Any a;
+  a.set_type (tc);
+  a <<= value;
+  _elements[_index]->from_any (a);
+}
+
 CORBA::Boolean
 DynAny_impl::get_boolean ()
 {
@@ -703,6 +713,20 @@ DynAny_impl::get_val ()
     return value;
 }
 
+CORBA::AbstractBase_ptr
+DynAny_impl::get_abstract ()
+{
+    if (_index < 0)
+        mico_throw (TypeMismatch());
+
+    update_element (_index);
+    CORBA::Any_var a = _elements[_index]->to_any();
+    CORBA::AbstractBase_ptr value;
+    if (!((CORBA::Any &)a >>= CORBA::Any::to_abstract_base (value)))
+	mico_throw (TypeMismatch ());
+    return value;
+}
+
 DynamicAny::DynAny_ptr
 DynAny_impl::current_component ()
 {
@@ -2141,6 +2165,33 @@ DynArray_impl::set_elements_as_dyn_any (const DynamicAny::DynAnySeq& value)
 	_elements[i]->assign (value[i].in());
 }
 
+/************************** DynValueCommon *************************/
+
+DynValueCommon_impl::DynValueCommon_impl ()
+{
+  _is_null = 1;
+  _index = -1;
+}
+
+CORBA::Boolean
+DynValueCommon_impl::is_null ()
+{
+  return _is_null;
+}
+
+void
+DynValueCommon_impl::set_to_null ()
+{
+  _is_null = 1;
+  _index = -1;
+}
+
+void
+DynValueCommon_impl::set_to_value ()
+{
+  _is_null = 0;
+  _index = 0;
+}
 
 /************************** DynValue *******************************/
 
@@ -2153,46 +2204,51 @@ DynValue_impl::DynValue_impl (CORBA::TypeCode_ptr tc)
 {
     _type = CORBA::TypeCode::_duplicate (tc);
     tc = tc->unalias();
-    if (tc->kind() != CORBA::tk_value && tc->kind() != CORBA::tk_value_box)
+    if (tc->kind() != CORBA::tk_value)
 	mico_throw (DynamicAny::DynAnyFactory::InconsistentTypeCode ());
-    _isvalue = (tc->kind() == CORBA::tk_value);
     for (CORBA::ULong i = 0; i < tc->member_count_inherited(); ++i) {
 	CORBA::TypeCode_var memtc = tc->member_type_inherited(i);
 	_elements.push_back(_factory()->create_dyn_any_from_type_code (memtc));
     }
-    if (_elements.size() == 0)
-        _index = -1;
+    _index = -1;
 }
 
 DynValue_impl::DynValue_impl (const CORBA::Any &a)
 {
     _type = a.type();
     CORBA::TypeCode_ptr tc = _type->unalias();
-    if (tc->kind() != CORBA::tk_value && tc->kind() != CORBA::tk_value_box)
+    if (tc->kind() != CORBA::tk_value)
 	mico_throw (DynamicAny::DynAnyFactory::InconsistentTypeCode ());
-    _isvalue = (tc->kind() == CORBA::tk_value);
 
     CORBA::Long value_id;
     CORBA::Boolean is_ref;
 
-    if (_isvalue) {
-	CORBA::Boolean r = a.value_get_begin (value_id, is_ref);
-	assert (r);
-    } else {
-	CORBA::Boolean r = a.valuebox_get_begin (value_id, is_ref);
-	assert (r);
-    }
+    CORBA::Boolean r = a.value_get_begin (value_id, is_ref);
+    assert (r);
 
     if (is_ref) {
-	assert (!value_id);
 
-	// XXX what shall we do with NIL values??
+      for (CORBA::ULong i = 0; i < tc->member_count_inherited(); ++i) {
+	CORBA::TypeCode_var memtc = tc->member_type_inherited(i);
+	_elements.push_back(_factory()->create_dyn_any_from_type_code (memtc));
+      }
+
+      if (!value_id) {
+	_is_null = 1;
+	_index = -1;
+      }
+      else {
+	_is_null = 0;
+	// XXX shared value
 	assert (0);
+      }
     }
+    else {
+      _is_null = 0;
 
-    for (CORBA::ULong i = 0; i < tc->member_count_inherited(); ++i) {
+      for (CORBA::ULong i = 0; i < tc->member_count_inherited(); ++i) {
 	CORBA::Any el;
-	CORBA::Boolean r = a.any_get (el);
+	r = a.any_get (el);
 	assert (r);
 // Here it is needed to set the Any type after calling any_get()
 // else there is no deal with aliased typecodes.
@@ -2201,15 +2257,12 @@ DynValue_impl::DynValue_impl (const CORBA::Any &a)
 	el.type(member_tc);
 //
 	_elements.push_back (_factory()->create_dyn_any (el));
+      }
+
+      r = a.value_get_end (value_id, is_ref);
+      assert (r);
     }
 
-    if (_isvalue) {
-	CORBA::Boolean r = a.value_get_end (value_id, is_ref);
-	assert (r);
-    } else {
-	CORBA::Boolean r = a.valuebox_get_end (value_id, is_ref);
-	assert (r);
-    }
     if (_elements.size() == 0)
         _index = -1;
 }
@@ -2228,42 +2281,33 @@ DynValue_impl::from_any (const CORBA::Any& a)
     CORBA::Long value_id;
     CORBA::Boolean is_ref;
 
-    if (_isvalue) {
-	CORBA::Boolean r = a.value_get_begin (value_id, is_ref);
-	assert (r);
-    } else {
-	CORBA::Boolean r = a.valuebox_get_begin (value_id, is_ref);
-	assert (r);
-    }
+    CORBA::Boolean r = a.value_get_begin (value_id, is_ref);
+    assert (r);
 
     if (is_ref) {
-	assert (!value_id);
+      if (!value_id) {
+	_is_null = 1;
+	return;
+      }
 
-	// XXX what shall we do with NIL values??
-	assert (0);
+      // XXX shared value
+      assert (0);
     }
 
+    _is_null = 0;
+
     CORBA::TypeCode_ptr tc2 = tc->unalias();
     for (CORBA::ULong i = 0; i < tc2->member_count_inherited(); ++i) {
 	CORBA::Any el;
-// Here it is needed to set the Any type before calling any_get()
-// else there is no deal with aliased typecodes.
-// Note that calling el.set_type() before calling any_get() doesn't resolve the alias problem.
+	r = a.any_get (el);
+	assert (r);
 	CORBA::TypeCode_var member_tc = tc2->member_type_inherited(i);
 	el.type(member_tc);
-//
-	CORBA::Boolean r = a.any_get (el);
-	assert (r);
 	_elements[i]->from_any (el);
     }
 
-    if (_isvalue) {
-	CORBA::Boolean r = a.value_get_end (value_id, is_ref);
-	assert (r);
-    } else {
-	CORBA::Boolean r = a.valuebox_get_end (value_id, is_ref);
-	assert (r);
-    }
+    r = a.value_get_end (value_id, is_ref);
+    assert (r);
 }
 
 CORBA::Any*
@@ -2275,33 +2319,30 @@ DynValue_impl::to_any ()
 
     CORBA::Long value_id;
 
-    if (_isvalue) {
-	CORBA::Boolean r = a->value_put_begin (value_id);
-	assert (r);
-    } else {
-	CORBA::Boolean r = a->valuebox_put_begin (value_id);
-	assert (r);
+    if (_is_null) {
+      a->value_put_ref (0);
+      return a;
     }
 
+    CORBA::Boolean r = a->value_put_begin (value_id);
+    assert (r);
+
     for (CORBA::ULong i = 0; i < _elements.size(); ++i) {
 	CORBA::Any_var el = _elements[i]->to_any ();
-	CORBA::Boolean r = a->any_put (el.inout());
-	assert (r);
-    }
-    if (_isvalue) {
-	CORBA::Boolean r = a->value_put_end (value_id);
-	assert (r);
-    } else {
-	CORBA::Boolean r = a->valuebox_put_end (value_id);
+	r = a->any_put (el.inout());
 	assert (r);
     }
+
+    r = a->value_put_end (value_id);
+    assert (r);
+
     return a;
 }
 
 DynamicAny::FieldName
 DynValue_impl::current_member_name ()
 {
-    if (_index < 0)
+    if (_is_null || _index < 0)
         mico_throw (TypeMismatch());
     return CORBA::string_dup (_type->unalias()->member_name_inherited(_index));
 }
@@ -2309,7 +2350,7 @@ DynValue_impl::current_member_name ()
 CORBA::TCKind
 DynValue_impl::current_member_kind ()
 {
-    if (_index < 0)
+    if (_is_null || _index < 0)
         mico_throw (TypeMismatch());
     // XXX unlias ???
     CORBA::TypeCode_var tc = _type->unalias()->member_type_inherited (_index);
@@ -2319,6 +2360,10 @@ DynValue_impl::current_member_kind ()
 DynamicAny::NameValuePairSeq*
 DynValue_impl::get_members ()
 {
+  if (_is_null) {
+    mico_throw (InvalidValue());
+  }
+
     DynamicAny::NameValuePairSeq *seq = new DynamicAny::NameValuePairSeq;
     seq->length (_elements.size());
     CORBA::TypeCode_ptr tc = _type->unalias ();
@@ -2334,7 +2379,7 @@ void
 DynValue_impl::set_members (const DynamicAny::NameValuePairSeq& value)
 {
     CORBA::TypeCode_ptr tc = _type->unalias ();
-    if (value.length() != tc->member_count())
+    if (value.length() != tc->member_count_inherited())
 	mico_throw (TypeMismatch ());
 
     for (CORBA::ULong i = 0; i < value.length(); ++i) {
@@ -2344,11 +2389,16 @@ DynValue_impl::set_members (const DynamicAny::NameValuePairSeq& value)
 	_elements[i]->from_any (value[i].value);
     }
     _index = _elements.size() > 0 ? 0 : -1;
+    _is_null = 0;
 }
 
 DynamicAny::NameDynAnyPairSeq*
 DynValue_impl::get_members_as_dyn_any ()
 {
+  if (_is_null) {
+    mico_throw (InvalidValue());
+  }
+
     DynamicAny::NameDynAnyPairSeq *seq = new DynamicAny::NameDynAnyPairSeq;
     seq->length (_elements.size());
     CORBA::TypeCode_ptr tc = _type->unalias ();
@@ -2374,9 +2424,174 @@ DynValue_impl::set_members_as_dyn_any (
 	_elements[i]->assign (value[i].value.in());
     }
     _index = _elements.size() > 0 ? 0 : -1;
+    _is_null = 0;
 }
 
 
+/************************** DynValue *******************************/
+
+/*
+ * XXX Shared valuetypes end up in separate copies when using
+ *     DynValue. This is due to the way CORBA::Any is implemented.
+ */
+
+DynValueBox_impl::DynValueBox_impl (CORBA::TypeCode_ptr tc)
+{
+    _type = CORBA::TypeCode::_duplicate (tc);
+    tc = tc->unalias();
+    if (tc->kind() != CORBA::tk_value_box)
+	mico_throw (DynamicAny::DynAnyFactory::InconsistentTypeCode ());
+    CORBA::TypeCode_var memtc = tc->content_type ();
+    _elements.push_back(_factory()->create_dyn_any_from_type_code (memtc));
+    _index = -1;
+}
+
+DynValueBox_impl::DynValueBox_impl (const CORBA::Any &a)
+{
+    _type = a.type();
+    CORBA::TypeCode_ptr tc = _type->unalias();
+    if (tc->kind() != CORBA::tk_value_box)
+	mico_throw (DynamicAny::DynAnyFactory::InconsistentTypeCode ());
+
+    CORBA::Long value_id;
+    CORBA::Boolean is_ref;
+
+    CORBA::Boolean r = a.valuebox_get_begin (value_id, is_ref);
+    assert (r);
+
+    if (is_ref) {
+      CORBA::TypeCode_var memtc = tc->content_type ();
+      _elements.push_back(_factory()->create_dyn_any_from_type_code (memtc));
+
+      if (!value_id) {
+        _index = -1;
+	_is_null = 1;
+      }
+      else {
+	_is_null = 0;
+	// XXX shared value
+	assert (0);
+      }
+    }
+    else {
+      _is_null = 0;
+
+      CORBA::TypeCode_var memtc = tc->content_type ();
+      CORBA::Any el;
+      r = a.any_get (el);
+      assert (r);
+      el.type (memtc);
+      
+      _elements.push_back (_factory()->create_dyn_any (el));
+      
+      r = a.valuebox_get_end (value_id, is_ref);
+      assert (r);
+    }
+}
+
+DynValueBox_impl::~DynValueBox_impl ()
+{
+}
+
+void
+DynValueBox_impl::from_any (const CORBA::Any& a)
+{
+    CORBA::TypeCode_var tc = a.type();
+    if (!_type->equaltype (tc))
+	mico_throw (TypeMismatch ());
+
+    CORBA::Long value_id;
+    CORBA::Boolean is_ref;
+    CORBA::Boolean r = a.valuebox_get_begin (value_id, is_ref);
+    assert (r);
+
+    if (is_ref) {
+      if (!value_id) {
+	_is_null = 1;
+	return;
+      }
+
+      // XXX shared value
+      assert (0);
+    }
+
+    _is_null = 0;
+
+    CORBA::TypeCode_ptr tc2 = tc->unalias();
+    CORBA::TypeCode_var member_tc = tc2->content_type();
+    CORBA::Any el;
+    r = a.any_get (el);
+    assert (r);
+    el.type (member_tc);
+    _elements[0]->from_any (el);
+
+    r = a.valuebox_get_end (value_id, is_ref);
+    assert (r);
+}
+
+CORBA::Any*
+DynValueBox_impl::to_any ()
+{
+    CORBA::Any *a = new CORBA::Any;
+
+    a->set_type (_type);
+
+    CORBA::Long value_id;
+
+    if (_is_null) {
+      a->value_put_ref (0);
+      return a;
+    }
+
+    CORBA::Boolean r = a->valuebox_put_begin (value_id);
+    assert (r);
+
+    CORBA::Any_var el = _elements[0]->to_any ();
+    r = a->any_put (el.inout());
+    assert (r);
+
+    r = a->valuebox_put_end (value_id);
+    assert (r);
+
+    return a;
+}
+
+CORBA::Any *
+DynValueBox_impl::get_boxed_value ()
+{
+  if (_is_null) {
+    mico_throw (InvalidValue());
+  }
+
+  return _elements[0]->to_any ();
+}
+
+void
+DynValueBox_impl::set_boxed_value (const CORBA::Any & boxed)
+{
+  _elements[0]->from_any (boxed);
+  _is_null = 0;
+  _index = 0;
+}
+
+DynamicAny::DynAny_ptr
+DynValueBox_impl::get_boxed_value_as_dyn_any ()
+{
+  if (_is_null) {
+    mico_throw (InvalidValue());
+  }
+
+  return _elements[0]->copy ();
+}
+
+void
+DynValueBox_impl::set_boxed_value_as_dyn_any (DynamicAny::DynAny_ptr boxed)
+{
+  _elements[0]->assign (boxed);
+  _is_null = 0;
+  _index = 0;
+}
+
 /************************* DynAnyFactory ***************************/
 
 
@@ -2429,8 +2644,9 @@ DynAnyFactory_impl::create_dyn_any (const CORBA::Any& value)
     case CORBA::tk_fixed:
 	return DynamicAny::DynFixed::_duplicate (new DynFixed_impl (value));
     case CORBA::tk_value:
-    case CORBA::tk_value_box:
 	return DynamicAny::DynValue::_duplicate (new DynValue_impl (value));
+    case CORBA::tk_value_box:
+	return DynamicAny::DynValueBox::_duplicate (new DynValueBox_impl (value));
     default:
 	mico_throw (InconsistentTypeCode());
     }
@@ -2477,8 +2693,9 @@ DynAnyFactory_impl::create_dyn_any_from_type_code (CORBA::TypeCode_ptr type)
     case CORBA::tk_fixed:
 	return DynamicAny::DynFixed::_duplicate (new DynFixed_impl (type));
     case CORBA::tk_value:
-    case CORBA::tk_value_box:
 	return DynamicAny::DynValue::_duplicate (new DynValue_impl (type));
+    case CORBA::tk_value_box:
+      return DynamicAny::DynValueBox::_duplicate (new DynValueBox_impl (type));
     default:
 	mico_throw (InconsistentTypeCode());
     }
