@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,15 +16,15 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #include <CORBA-SMALL.h>
 #include <mico/template_impl.h>
 #include <iostream.h>
-#include <mico/os-math.h>
 #include <ctype.h>
+#include <mico/os-math.h>
 
 /*
  * one would usually use pow(10,x) but for some strange
@@ -34,7 +34,7 @@
  * works fine. This is what is called a Heisenbug.
  */
 static MICO_LongDouble
-pow10 (int x)
+power10 (int x)
 {
     if (x == 0)
         return 1;
@@ -169,7 +169,7 @@ FixedBase::FixedBase (MICO_Long v)
     char buf[100];
     sprintf (buf, "%i", v);
     compute_params (buf, _digits, _scale);
-    _val = v * pow10 (_scale);
+    _val = v * power10 (_scale);
     adjust ();
 }
 
@@ -178,7 +178,7 @@ FixedBase::FixedBase (MICO_ULong v)
     char buf[100];
     sprintf (buf, "%u", v);
     compute_params (buf, _digits, _scale);
-    _val = v * pow10 (_scale);
+    _val = v * power10 (_scale);
     adjust ();
 }
 
@@ -188,7 +188,7 @@ FixedBase::FixedBase (MICO_LongLong v)
     // XXX
     sprintf (buf, "%i", (MICO_Long)v);
     compute_params (buf, _digits, _scale);
-    _val = v * pow10 (_scale);
+    _val = v * power10 (_scale);
     adjust ();
 }
 
@@ -200,9 +200,9 @@ FixedBase::FixedBase (MICO_ULongLong v)
     compute_params (buf, _digits, _scale);
 #ifdef _WINDOWS
     // VC++ 5.0 does not support unsigned __int64
-    _val = ((MICO_LongLong)v) * pow10 (_scale);
+    _val = ((MICO_LongLong)v) * power10 (_scale);
 #else
-    _val = v * pow10 (_scale);
+    _val = v * power10 (_scale);
 #endif
     adjust ();
 }
@@ -214,7 +214,7 @@ FixedBase::FixedBase (MICO_Double v)
     char buf[100];
     sprintf (buf, "%.31le", v2);
     compute_params (buf, _digits, _scale, digits());
-    _val = v * pow10 (_scale);
+    _val = v * power10 (_scale);
     adjust ();
 }
 
@@ -230,7 +230,7 @@ FixedBase::FixedBase (MICO_LongDouble v)
     sprintf (buf, "%.31le", (MICO_Double)v2);
     compute_params (buf, _digits, _scale, digits());
 #endif
-    _val = v * pow10 (_scale);
+    _val = v * power10 (_scale);
     adjust ();
 }
 
@@ -250,7 +250,7 @@ FixedBase::FixedBase (const char *s)
     v = strtod ((char *)s, NULL);
 #endif
     compute_params (s, _digits, _scale);
-    _val = v * pow10 (_scale);
+    _val = v * power10 (_scale);
     adjust ();
 }
 
@@ -271,12 +271,12 @@ FixedBase::~FixedBase ()
 
 FixedBase::operator CORBA::LongLong () const
 {
-    return (CORBA::LongLong)(_val / pow10 (_scale));
+    return (CORBA::LongLong)(_val / power10 (_scale));
 }
 
 FixedBase::operator CORBA::LongDouble () const
 {
-    return _val / pow10 (_scale);
+    return _val / power10 (_scale);
 }
 
 FixedBase
@@ -300,7 +300,7 @@ FixedBase::truncate (MICO_UShort nscale) const
 void
 FixedBase::adjust (CORBA::Boolean round)
 {
-    _val = fmodl (_val, pow10 (_digits));
+    _val = fmodl (_val, power10 (_digits));
     CORBA::LongDouble delta = round ? 0.5 : 0.0;
     if (_val < 0) {
 	_val = ceill (_val - delta);
@@ -314,7 +314,7 @@ FixedBase::transform (MICO_Short nscale) const
 {
     if (_scale == nscale)
 	return _val;
-    return _val * pow10 (nscale - _scale);
+    return _val * power10 (nscale - _scale);
 }
 
 void
@@ -348,7 +348,8 @@ FixedBase::compute_params (const char *str,
     if (first_significant_pos < 0)
 	first_significant_pos = 0;
     if (dot_pos < 0)
-	dot_pos = last_significant_pos;
+	// rfweber@bluewin.de; was dot_pos = last_significant_pos;
+	dot_pos = pos;
 
     if (*str == 'e' || *str == 'E')
 	dot_pos += atol (++str);
@@ -467,7 +468,7 @@ FixedBase::to_digits () const
     digs->length (_digits+1);
 
     MICO_LongDouble d = fabsl (_val) + 0.5;
-    MICO_LongDouble q = pow10 (_digits-1);
+    MICO_LongDouble q = power10 (_digits-1);
 
     for (int i = 0; i < _digits; ++i) {
 	int r = (int)(d/q);
@@ -508,7 +509,7 @@ FixedBase::read (istream &i)
      */
     CORBA::Double d;
     i >> d;
-    _val = d * pow10 (_scale);
+    _val = d * power10 (_scale);
     adjust ();
     return i;
 }
@@ -550,7 +551,7 @@ FixedBase::add (FixedBase &res, const FixedBase &a1, const FixedBase a2)
 	res_scale = a2._scale;
     }
     if (res._scale != res_scale)
-	res._val *= pow10 (res._scale - res_scale);
+	res._val *= power10 (res._scale - res_scale);
     res.adjust ();
 }
 
@@ -570,25 +571,25 @@ FixedBase::sub (FixedBase &res, const FixedBase &a1, const FixedBase a2)
 	res_scale = a2._scale;
     }
     if (res._scale != res_scale)
-	res._val *= pow10 (res._scale - res_scale);
+	res._val *= power10 (res._scale - res_scale);
     res.adjust ();
 }
 
 void
 FixedBase::mul (FixedBase &res, const FixedBase &a1, const FixedBase a2)
 {
-    res._val = (a1._val * a2._val) / pow10 (a2._scale);
+    res._val = (a1._val * a2._val) / power10 (a2._scale);
     if (res._scale != a1._scale)
-	res._val *= pow10 (res._scale - a1._scale);
+	res._val *= power10 (res._scale - a1._scale);
     res.adjust ();
 }
 
 void
 FixedBase::div (FixedBase &res, const FixedBase &a1, const FixedBase a2)
 {
-    res._val = (a1._val * pow10 (a2._scale)) / a2._val;
+    res._val = (a1._val * power10 (a2._scale)) / a2._val;
     if (res._scale != a1._scale)
-	res._val *= pow10 (res._scale - a1._scale);
+	res._val *= power10 (res._scale - a1._scale);
     res.adjust ();
 }
 
@@ -597,7 +598,7 @@ FixedBase::neg (FixedBase &res, const FixedBase &a)
 {
     res._val = - a._val;
     if (res._scale != a._scale)
-	res._val *= pow10 (res._scale - a._scale);
+	res._val *= power10 (res._scale - a._scale);
 }
 
 int
@@ -725,7 +726,7 @@ FixedBase::operator- () const
 CORBA::Boolean
 FixedBase::operator!() const
 {
-    return _val != 0;
+    return _val == 0;
 }
 
 #define FMAX(a,b) ((a)>(b)?(a):(b))
