@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #define MICO_CONF_INTERCEPT
@@ -151,7 +151,7 @@ void Interceptor::Context::destroy ()
 Interceptor::LWRootRequest::LWRootRequest (
     CORBA::Object_var &obj,
     CORBA::String_var &op,
-    CORBA::IOP::ServiceContextList &svc)
+    IOP::ServiceContextList &svc)
     : _svc (svc), _op (op), _obj (obj)
 {
 }
@@ -270,7 +270,7 @@ Interceptor::LWRootRequest::get_context (Root_ptr interceptor)
 Interceptor::LWRequest::LWRequest (
     CORBA::Object_var &obj,
     CORBA::String_var &op,
-    CORBA::IOP::ServiceContextList &svc,
+    IOP::ServiceContextList &svc,
     CORBA::Request_ptr req)
     : Interceptor::LWRootRequest (obj, op, svc), _req (req)
 {
@@ -309,7 +309,7 @@ Interceptor::LWRequest::request ()
 Interceptor::LWServerRequest::LWServerRequest (
     CORBA::Object_ptr obj,
     const char *op,
-    CORBA::IOP::ServiceContextList &svc,
+    IOP::ServiceContextList &svc,
     CORBA::ServerRequest_ptr req)
     : Interceptor::LWRootRequest (_obj, _op, svc), _req (req),
       _obj (CORBA::Object::_duplicate (obj)), _op (op)
@@ -554,7 +554,7 @@ Interceptor::LWRequest_ptr
 Interceptor::ClientInterceptor::_create_request (
     CORBA::Object_var &obj,
     CORBA::String_var &op,
-    CORBA::IOP::ServiceContextList &svc,
+    IOP::ServiceContextList &svc,
     CORBA::Request_ptr req)
 {
     if (_ics().size() == 0)
@@ -773,7 +773,7 @@ Interceptor::LWServerRequest_ptr
 Interceptor::ServerInterceptor::_create_request (
     CORBA::Object_ptr obj,
     const char *op,
-    CORBA::IOP::ServiceContextList &svc,
+    IOP::ServiceContextList &svc,
     CORBA::ServerRequest_ptr req)
 {
     if (_ics().size() == 0)
