@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #define MICO_CONF_IMR
@@ -61,29 +61,44 @@ CORBA::IORProfileDecoder::~IORProfileDecoder ()
 CORBA::IOR::IOR (const char *objid, const IORProfileVec &t)
     : oid (objid)
 {
+  _active_profile = 0;
+  _active_profile_index = 0;
+  _addressing_disposition = 0;
     for (mico_vec_size_type i = 0; i < t.size(); ++i)
 	add_profile (t[i]);
 }
 
 CORBA::IOR::IOR (DataDecoder &dc)
 {
+  _active_profile = 0;
+  _active_profile_index = 0;
+  _addressing_disposition = 0;
     CORBA::Boolean r = dc.get_ior (*this);
     assert (r);
 }
 
 CORBA::IOR::IOR (const char *_ior)
 {
+  _active_profile = 0;
+  _active_profile_index = 0;
+  _addressing_disposition = 0;
     CORBA::Boolean r = from_string (_ior);
     assert (r);
 }
 
 CORBA::IOR::IOR (const IOR &ior)
 {
+  _active_profile = 0;
+  _active_profile_index = 0;
+  _addressing_disposition = 0;
     copy (ior);
 }
 
 CORBA::IOR::IOR ()
 {
+  _active_profile = 0;
+  _active_profile_index = 0;
+  _addressing_disposition = 0;
 }
 
 CORBA::IOR::~IOR ()
@@ -95,6 +110,8 @@ CORBA::IOR &
 CORBA::IOR::operator= (const IOR &ior)
 {
     if (this != &ior) {
+      _active_profile_index = 0;
+      _addressing_disposition = 0;
 	free ();
 	copy (ior);
     }
@@ -104,6 +121,10 @@ CORBA::IOR::operator= (const IOR &ior)
 CORBA::Boolean
 CORBA::IOR::from_string (const char *_ior)
 {
+  _active_profile = 0;
+  _active_profile_index = 0;
+  _addressing_disposition = 0;
+
     string ior (_ior);
     if (ior.find ("IOR:") != 0)
 	return FALSE;
@@ -120,7 +141,7 @@ CORBA::IOR::from_string (const char *_ior)
     }
 #ifdef _WINDOWS
     MICO::CDRDecoder dc (&b, FALSE, CORBA::DefaultEndian,
-			 0, TRUE, 0, TRUE, 0, TRUE);
+			 0, TRUE, 0, TRUE);
 #else
     MICO::CDRDecoder dc (&b, FALSE);
 #endif
@@ -199,6 +220,10 @@ CORBA::IOR::operator< (const IOR &ior) const
 void
 CORBA::IOR::free ()
 {
+  _active_profile = 0;
+  _active_profile_index = 0;
+  _addressing_disposition = 0;
+
     for (ULong i = 0; i < tags.size(); ++i)
 	delete tags[i];
     tags.erase (tags.begin(), tags.end());
@@ -224,7 +249,7 @@ CORBA::IOR::decode (DataDecoder &dc)
 	ULong len;
 
 	check (dc.struct_begin ());
-	check (dc.get_string_raw (oid));
+	check (dc.get_string_raw_stl (oid));
 	check (dc.seq_begin (len));
 
 	IORProfile *tag;
@@ -289,6 +314,48 @@ CORBA::IOR::profile (IORProfile::ProfileId id, Boolean find_unreach,
     return 0;
 }
 
+CORBA::IORProfile *
+CORBA::IOR::active_profile (CORBA::ULong * index)
+{
+  if (_active_profile && index) {
+    *index = _active_profile_index;
+  }
+  return _active_profile;
+}
+
+void
+CORBA::IOR::active_profile (CORBA::IORProfile * prof)
+{
+  CORBA::ULong i;
+
+  if (!prof) {
+    _active_profile = prof;
+    return;
+  }
+
+  for (i=0; i<tags.size(); i++) {
+    if (tags[i] == prof) {
+      _active_profile_index = i;
+      break;
+    }
+  }
+
+  assert (i != tags.size());
+  _active_profile = prof;
+}
+
+GIOP::AddressingDisposition
+CORBA::IOR::addressing_disposition ()
+{
+  return _addressing_disposition;
+}
+
+void
+CORBA::IOR::addressing_disposition (GIOP::AddressingDisposition disp)
+{
+  _addressing_disposition = disp;
+}
+
 void
 CORBA::IOR::add_profile (IORProfile *prof)
 {
@@ -307,6 +374,16 @@ CORBA::IOR::del_profile (IORProfile *prof)
 	    break;
 	}
     }
+    if (prof == _active_profile) {
+      _active_profile = 0;
+    }
+}
+
+CORBA::IORProfile *
+CORBA::IOR::get_profile (ULong i)
+{
+  if (i >= tags.size()) return 0;
+  return tags[i];
 }
 
 const char *
@@ -696,7 +773,7 @@ MICO::LocalProfileDecoder::decode (CORBA::DataDecoder &dc, ProfileId,
 
     check (dc.struct_begin ());
     {
-	check (dc.get_string_raw (host));
+	check (dc.get_string_raw_stl (host));
 	check (dc.get_long (pid));
 
 	check (dc.seq_begin (len));
@@ -858,7 +935,7 @@ MICO::IIOPProfile::print (ostream &o) const
     << (CORBA::Long) (version & 0xff) << endl;
   o << "    Address:  " << myaddr.stringify() << endl;
 
-  o << "   Location:  iioploc://";
+  o << "   Location:  corbaloc::";
   if (version != 0x0100) {
     o << (CORBA::Long) (version >> 8) << "."
       << (CORBA::Long) (version & 0xff) << "@";
@@ -953,11 +1030,10 @@ MICO::IIOPProfileDecoder::decode (CORBA::DataDecoder &dc, ProfileId,
 	    check (dc.get_octet (major));
 	    check (dc.get_octet (minor));
 	    version = ((major << 8) | minor);
-	    check (version <= 0x0102);
 	}
 	check (dc.struct_end ());
 
-	check (dc.get_string_raw (host));
+	check (dc.get_string_raw_stl (host));
 	check (dc.get_ushort (port));
 
 	check (dc.seq_begin (len));
@@ -1221,8 +1297,8 @@ MICO::UIOPProfileDecoder::decode (CORBA::DataDecoder &dc, ProfileId,
 	}
 	check (dc.struct_end ());
 
-	check (dc.get_string_raw (host));
-	check (dc.get_string_raw (filename));
+	check (dc.get_string_raw_stl (host));
+	check (dc.get_string_raw_stl (filename));
 
 	check (dc.seq_begin (len));
 	{
@@ -1439,7 +1515,7 @@ MICO::UnknownProfile::decode (CORBA::DataDecoder &dc, ProfileId id,
     p->tagdata.erase (p->tagdata.begin(), p->tagdata.end());
     // include byteorder octet from encapsulated sequence
     p->tagdata.insert (p->tagdata.begin(),
-		       dc.buffer()->data() - 1, dc.buffer()->data() + len + 1);
+		       dc.buffer()->data() - 1, dc.buffer()->data() + len);
     dc.buffer()->rseek_rel (len);
 
     return p;
@@ -1845,7 +1921,7 @@ MICO::UnknownComponent::decode (CORBA::DataDecoder &dc, ComponentId id,
     p->tagdata.erase (p->tagdata.begin(), p->tagdata.end());
     // include byteorder octet from encapsulated sequence
     p->tagdata.insert (p->tagdata.begin(),
-		       dc.buffer()->data() - 1, dc.buffer()->data() + len + 1);
+		       dc.buffer()->data() - 1, dc.buffer()->data() + len);
     dc.buffer()->rseek_rel (len);
 
     return p;
@@ -1913,7 +1989,7 @@ MICO::MultiCompProfile::objectkey (CORBA::Octet *, CORBA::Long length)
 const CORBA::Octet *
 MICO::MultiCompProfile::objectkey (CORBA::Long &length) const
 {
-    length = 1;
+    length = 0;
     return (const CORBA::Octet *)"";
 }
 
