@@ -1,5 +1,5 @@
 /*
- *  MICO --- a free CORBA implementation
+ *  MICO --- an Open Source CORBA implementation
  *  Copyright (C) 1998 Frank Pilhofer
  *
  *  This library is free software; you can redistribute it and/or
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #include <CORBA.h>
@@ -78,10 +78,10 @@ PortableServer::ServantBase::_is_a (const char *)
   return FALSE;
 }
 
-CORBA::Boolean
-PortableServer::ServantBase::_non_existent ()
+CORBA::Object_ptr
+PortableServer::ServantBase::_get_component ()
 {
-  return FALSE;
+  return CORBA::Object::_nil ();
 }
 
 CORBA::InterfaceDef_ptr
@@ -91,12 +91,20 @@ PortableServer::ServantBase::_get_interface (const char *repoid)
   CORBA::Object_var irobj = 
     orb->resolve_initial_references ("InterfaceRepository");
   CORBA::Repository_var ifr = CORBA::Repository::_narrow (irobj);
-  assert (!CORBA::is_nil (ifr));
+  if (CORBA::is_nil (ifr)) {
+    return CORBA::InterfaceDef::_nil ();
+  }
 
   CORBA::Contained_var cv = ifr->lookup_id (repoid);
   return CORBA::InterfaceDef::_narrow (cv);
 }
 
+CORBA::Boolean
+PortableServer::ServantBase::_non_existent ()
+{
+  return FALSE;
+}
+
 CORBA::Object_ptr
 PortableServer::ServantBase::_this ()
 {
@@ -266,6 +274,69 @@ PortableServer::DynamicImplementation::doinvoke (CORBA::ServerRequestBase_ptr re
   invoke ((CORBA::ServerRequest_ptr) req);
 }
 
+CORBA::Object_ptr
+PortableServer::DynamicImplementation::_this ()
+{
+  return MICO_SCOPE(PortableServer,ServantBase::_this) ();
+}
+
+CORBA::Boolean
+PortableServer::DynamicImplementation::_is_a (const char * repoid)
+{
+  /*
+   * To call _primary_interface, we need a POA and an ObjectId.
+   * If called within the context of a method invocation, we use
+   * the "Current" information. Else, we activate the Servant.
+   */
+
+  PortableServer::ObjectId_var oid;
+  PortableServer::POA_var poa;
+
+  if (!CORBA::is_nil (_the_poa_current) &&
+      _the_poa_current->iscurrent() &&
+      _the_poa_current->get_serv() == (Servant) this) {
+    poa = _the_poa_current->get_POA ();
+    oid = _the_poa_current->get_object_id ();
+  }
+  else {
+    if (!CORBA::is_nil (_my_poa)) {
+      poa = PortableServer::POA::_duplicate (_my_poa);
+    }
+    else {
+      poa = _default_POA ();
+    }
+    CORBA::Object_var obj = poa->activate_for_this (this);
+    oid = poa->reference_to_id (obj);
+  }
+
+  CORBA::String_var pif = _primary_interface (oid.in(), poa.in());
+
+  if (strcmp (pif.in(), repoid) == 0) {
+    return TRUE;
+  }
+
+  /*
+   * Access IR
+   */
+
+  CORBA::ORB_var orb = CORBA::ORB_instance ("mico-local-orb");
+  CORBA::Object_var irobj =
+    orb->resolve_initial_references ("InterfaceRepository");
+  CORBA::Repository_var ifr = CORBA::Repository::_narrow (irobj);
+  if (CORBA::is_nil (ifr)) {
+    mico_throw (CORBA::OBJ_ADAPTER (0, CORBA::COMPLETED_NO));
+  }
+  
+  CORBA::Contained_var cv = ifr->lookup_id (pif.in());
+  CORBA::InterfaceDef_var ifd = CORBA::InterfaceDef::_narrow (cv);
+
+  if (CORBA::is_nil (ifd)) {
+    mico_throw (CORBA::OBJ_ADAPTER (0, CORBA::COMPLETED_NO));
+  }
+
+  return ifd->is_a (repoid);
+}
+
 CORBA::InterfaceDef_ptr
 PortableServer::DynamicImplementation::_get_interface ()
 {
@@ -305,7 +376,9 @@ PortableServer::DynamicImplementation::_get_interface ()
   CORBA::Object_var irobj =
     orb->resolve_initial_references ("InterfaceRepository");
   CORBA::Repository_var ifr = CORBA::Repository::_narrow (irobj);
-  assert (!CORBA::is_nil (ifr));
+  if (CORBA::is_nil (ifr)) {
+    mico_throw (CORBA::OBJ_ADAPTER (0, CORBA::COMPLETED_NO));
+  }
   
   CORBA::Contained_var cv = ifr->lookup_id (pif.in());
   CORBA::InterfaceDef_ptr ifd = CORBA::InterfaceDef::_narrow (cv);
@@ -341,95 +414,6 @@ PortableServer::StaticImplementation::doinvoke (CORBA::ServerRequestBase_ptr req
   invoke ((CORBA::StaticServerRequest_ptr) req);
 }
 
-/*
- * ServantBase_var
- */
-
-#if 0
-PortableServer::ServantBase_var::ServantBase_var ()
-{
-  _ptr = NULL;
-}
-
-PortableServer::ServantBase_var::ServantBase_var (ServantBase * serv)
-{
-  _ptr = serv;
-}
-
-PortableServer::ServantBase_var::ServantBase_var (const ServantBase_var & o)
-{
-  if ((_ptr = o._ptr) != NULL) {
-    _ptr->_add_ref ();
-  }
-}
-
-PortableServer::ServantBase_var::~ServantBase_var ()
-{
-  if (_ptr) {
-    _ptr->_remove_ref ();
-  }
-}
-
-PortableServer::ServantBase_var &
-PortableServer::ServantBase_var::operator= (ServantBase * p)
-{
-  if (_ptr) {
-    _ptr->_remove_ref ();
-  }
-  _ptr = p;
-  return *this;
-}
-
-PortableServer::ServantBase_var &
-PortableServer::ServantBase_var::operator= (const ServantBase_var & o)
-{
-  if (_ptr != o._ptr) {
-    if (_ptr) {
-      _ptr->_remove_ref ();
-    }
-    if ((_ptr = o._ptr) != NULL) {
-      _ptr->_add_ref ();
-    }
-  }
-  return *this;
-}
-
-PortableServer::ServantBase *
-PortableServer::ServantBase_var::operator-> () const
-{
-  _ptr->_check ();
-  return _ptr;
-}
-
-PortableServer::ServantBase *
-PortableServer::ServantBase_var::in () const
-{
-  _ptr->_check ();
-  return _ptr;
-}
-
-PortableServer::ServantBase *&
-PortableServer::ServantBase_var::inout ()
-{
-  _ptr->_check ();
-  return _ptr;
-}
-
-PortableServer::ServantBase *&
-PortableServer::ServantBase_var::out ()
-{
-  return _ptr;
-}
-
-PortableServer::ServantBase *
-PortableServer::ServantBase_var::_retn ()
-{
-  PortableServer::ServantBase * rv = _ptr;
-  _ptr = NULL;
-  rv->_check();
-  return rv;
-}
-#endif
 
 /*
  * Base class for stubs
@@ -792,7 +776,9 @@ POA_PortableServer::AdapterActivator::_get_interface ()
   CORBA::Object_var irobj = 
     orb->resolve_initial_references ("InterfaceRepository");
   CORBA::Repository_var ifr = CORBA::Repository::_narrow (irobj);
-  assert (!CORBA::is_nil (ifr));
+  if (CORBA::is_nil (ifr)) {
+    mico_throw (CORBA::OBJ_ADAPTER (0, CORBA::COMPLETED_NO));
+  }
 
   CORBA::Contained_var cv = ifr->lookup_id ("IDL:omg.org/PortableServer/AdapterActivator:1.0");
   CORBA::InterfaceDef_ptr ifd = CORBA::InterfaceDef::_narrow (cv);
@@ -813,7 +799,7 @@ POA_PortableServer::AdapterActivator::_primary_interface (const PortableServer::
 void *
 POA_PortableServer::AdapterActivator::_narrow_helper (const char * repoid)
 {
-  void * p;
+
   if (strcmp (repoid, "IDL:omg.org/PortableServer/AdapterActivator:1.0") == 0) {
     return (void *) this;
   }
@@ -878,7 +864,9 @@ POA_PortableServer::ServantManager::_get_interface ()
   CORBA::Object_var irobj = 
     orb->resolve_initial_references ("InterfaceRepository");
   CORBA::Repository_var ifr = CORBA::Repository::_narrow (irobj);
-  assert (!CORBA::is_nil (ifr));
+  if (CORBA::is_nil (ifr)) {
+    mico_throw (CORBA::OBJ_ADAPTER (0, CORBA::COMPLETED_NO));
+  }
 
   CORBA::Contained_var cv = ifr->lookup_id ("IDL:omg.org/PortableServer/ServantManager:1.0");
   CORBA::InterfaceDef_ptr ifd = CORBA::InterfaceDef::_narrow (cv);
@@ -899,7 +887,7 @@ POA_PortableServer::ServantManager::_primary_interface (const PortableServer::Ob
 void *
 POA_PortableServer::ServantManager::_narrow_helper (const char * repoid)
 {
-  void * p;
+
   if (strcmp (repoid, "IDL:omg.org/PortableServer/ServantManager:1.0") == 0) {
     return (void *) this;
   }
@@ -967,7 +955,9 @@ POA_PortableServer::ServantActivator::_get_interface ()
   CORBA::Object_var irobj = 
     orb->resolve_initial_references ("InterfaceRepository");
   CORBA::Repository_var ifr = CORBA::Repository::_narrow (irobj);
-  assert (!CORBA::is_nil (ifr));
+  if (CORBA::is_nil (ifr)) {
+    mico_throw (CORBA::OBJ_ADAPTER (0, CORBA::COMPLETED_NO));
+  }
 
   CORBA::Contained_var cv = ifr->lookup_id ("IDL:omg.org/PortableServer/ServantActivator:1.0");
   CORBA::InterfaceDef_ptr ifd = CORBA::InterfaceDef::_narrow (cv);
@@ -1059,7 +1049,9 @@ POA_PortableServer::ServantLocator::_get_interface ()
   CORBA::Object_var irobj = 
     orb->resolve_initial_references ("InterfaceRepository");
   CORBA::Repository_var ifr = CORBA::Repository::_narrow (irobj);
-  assert (!CORBA::is_nil (ifr));
+  if (CORBA::is_nil (ifr)) {
+    mico_throw (CORBA::OBJ_ADAPTER (0, CORBA::COMPLETED_NO));
+  }
 
   CORBA::Contained_var cv = ifr->lookup_id ("IDL:omg.org/PortableServer/ServantLocator:1.0");
   CORBA::InterfaceDef_ptr ifd = CORBA::InterfaceDef::_narrow (cv);
