@@ -1,5 +1,5 @@
 /*
- *  MICO --- a free CORBA implementation
+ *  MICO --- an Open Source CORBA implementation
  *  Copyright (C) 1998 Frank Pilhofer
  *
  *  This library is free software; you can redistribute it and/or
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 /*
@@ -46,8 +46,8 @@ MICOPOA::POAOptions MICOPOA::poaopts;
 
 MICOPOA::POA_impl::POAMap MICOPOA::POA_impl::AllPOAs;
 MICOPOA::UniqueIdGenerator MICOPOA::POA_impl::poauid;
+MICOPOA::UniqueIdGenerator MICOPOA::POA_impl::idfactory ("_");
 MICOPOA::POACurrent_impl * MICOPOA::POA_impl::current = NULL;
-MICOPOA::POA_impl::POAStateMap MICOPOA::POA_impl::persistent_state;
 
 string MICOPOA::POA_impl::oaprefix;
 string MICOPOA::POA_impl::impl_name;
@@ -78,7 +78,8 @@ public:
   Interceptor::Status initialize (CORBA::ORB_ptr orb, const char *orbid,
 				  int &argc, char *argv[])
   {
-    MICOPOA::poaopts.parse (orb, argc, argv);
+    if (!MICOPOA::poaopts.parse (orb, argc, argv))
+      return Interceptor::INVOKE_ABORT;
     return Interceptor::INVOKE_CONTINUE;
   }
 } InitPOA;
@@ -93,7 +94,7 @@ MICOPOA::_init ()
  * Options parser
  */
 
-void
+CORBA::Boolean
 MICOPOA::POAOptions::parse (CORBA::ORB_ptr orb, int &argc, char *argv[])
 {
   MICOGetOpt::OptMap opts;
@@ -103,9 +104,9 @@ MICOPOA::POAOptions::parse (CORBA::ORB_ptr orb, int &argc, char *argv[])
 
   MICOGetOpt opt_parser (opts);
   CORBA::Boolean r = opt_parser.parse (orb->rcfile(), TRUE);
-  assert (r);
+  if (!r) return FALSE;
   r = opt_parser.parse (argc, argv, TRUE);
-  assert (r);
+  if (!r) return FALSE;
   const MICOGetOpt::OptVec &o = opt_parser.opts();
 
   for (MICOGetOpt::OptVec::const_iterator i = o.begin(); i != o.end(); ++i) {
@@ -113,6 +114,7 @@ MICOPOA::POAOptions::parse (CORBA::ORB_ptr orb, int &argc, char *argv[])
     string val = (*i).second;
     options[arg] = val;
   }
+  return TRUE;
 }
 
 const char *
@@ -646,7 +648,7 @@ MICOPOA::POAObjectReference::decompose_ref ()
   for (i=length-1; i>0; i--) {
     if (key[i] == '/') {
       for (j=0; j<i; j++) {
-	if (key[i-j] != '\\') {
+	if (key[i-j-1] != '\\') {
 	  break;
 	}
       }
@@ -679,7 +681,7 @@ MICOPOA::POAObjectReference::decompose_ref ()
 
     for (i=j, idlength=length-j; i<length; i++) {
       if (key[i] == '\\') {
-	idlength--;
+	idlength--, i++;
       }
     }
 
@@ -1338,7 +1340,7 @@ MICOPOA::POA_impl::the_children ()
 
   res->length (children.size());
   while (it != children.end()) {
-    (*res)[i++] = PortableServer::POA::_duplicate ((*it).second);
+    (*res)[i++] = PortableServer::POA::_duplicate ((*it++).second);
   }
 
   return res;
@@ -1488,7 +1490,7 @@ MICOPOA::POA_impl::set_policies (const CORBA::PolicyList & policies)
   request_processing_policy = PortableServer::RequestProcessingPolicy::_nil();
 
   for (CORBA::ULong i=0; i<policies.length(); i++) {
-    CORBA::Policy_ptr pol = policies[i];
+    CORBA::Policy_ptr pol = policies[i].in();
 
     if (pol->policy_type() == PortableServer::THREAD_POLICY_ID) {
       thread_policy = PortableServer::ThreadPolicy::_narrow (pol);
@@ -1652,8 +1654,7 @@ MICOPOA::POA_impl::POA_impl (const char * _name,
 			     const CORBA::PolicyList & policies,
 			     POA_impl * _parent,
 			     CORBA::ORB_ptr _orb)
-  : name (_name), parent(_parent), orb(_orb),
-    idfactory ("_")
+  : name (_name), parent(_parent), orb(_orb)
 {
   destructed = 0;
   unique_id = 0;
@@ -1709,8 +1710,8 @@ MICOPOA::POA_impl::POA_impl (const char * _name,
    * A persistent POA should have received a unique name via the
    * -OAImplName command line option.
    *
-   *  / <ip-addr> / <pid> / <time> / <fqn> / <guid>   (Transient)
-   *  <impl-name> / <fqn>                             (Persistent)
+   *  / <pid> / <time> / <guid>    (Transient)
+   *  <impl-name> / <fqn>          (Persistent)
    *
    * For a persistent POA with <fqn> == <impl_name>, the unique name is
    * collapsed to <impl-name> as a special case
@@ -1720,8 +1721,6 @@ MICOPOA::POA_impl::POA_impl (const char * _name,
     CORBA::String_var uid = poauid.new_id ();
     oaid  = oaprefix;
     oaid += "/";
-    oaid += fqn;
-    oaid += "/";
     oaid += uid.in();
   }
   else {
@@ -1765,15 +1764,6 @@ MICOPOA::POA_impl::POA_impl (const char * _name,
       oaid += "/";
       oaid += fqn;
     }
-
-    /*
-     * Restore our state if necessary
-     */
-
-    POAStateMap::iterator mystate = persistent_state.find (oaid);
-    if (mystate != persistent_state.end()) {
-      idfactory.state ((*mystate).second.c_str());
-    }
   }
   
   /*
@@ -1790,7 +1780,7 @@ MICOPOA::POA_impl::POA_impl (const char * _name,
  */
 
 MICOPOA::POA_impl::POA_impl (CORBA::ORB_ptr _orb)
-  : orb (_orb), idfactory ("_")
+  : orb (_orb)
 {
   destructed = 0;
   unique_id = 0;
@@ -1828,28 +1818,20 @@ MICOPOA::POA_impl::POA_impl (CORBA::ORB_ptr _orb)
   state = PortableServer::POAManager::HOLDING;
 
   /*
-   * Generate a unique prefix for transient POAs
+   * Generate a unique prefix for transient POAs based on the PID and the
+   * current time.
    */
 
+  OSMisc::TimeVal ct = OSMisc::gettime();
+
   oaprefix  = "/";
-
-  vector<CORBA::Octet> host = MICO::InetAddress::hostid();
-
-  for (CORBA::ULong i2=0; i2<host.size(); i2++) {
-    if (i2 > 0) {
-      oaprefix += ".";
-    }
-    oaprefix += xdec ((int) host[i2]);
-  }
-
-  oaprefix += "/";
   oaprefix += xdec (OSMisc::getpid());
   oaprefix += "/";
-  oaprefix += xdec (OSMisc::gettime());
+  oaprefix += xdec (ct.tv_sec);
 
   /*
    * The Root POA has a TRANSIENT Lifespan Policy. Therefore, use
-   *  / <ip-addr> / <pid> / <time>
+   *  / <pid> / <time>
    * as the unique name
    */
 
@@ -1862,6 +1844,9 @@ MICOPOA::POA_impl::POA_impl (CORBA::ORB_ptr _orb)
   if (poaopts["-POAImplName"] != NULL) {
     impl_name = poaopts["-POAImplName"];
   }
+  else {
+    impl_name = "Default";
+  }
 
   /*
    * If we have an ImplName, connect to the Mediator
@@ -1892,20 +1877,7 @@ MICOPOA::POA_impl::POA_impl (CORBA::ORB_ptr _orb)
       string myref = orb->ior_template()->stringify();
       CORBA::String_var poamed_ref =
 	poamed->create_impl (impl_name.c_str(), myref.c_str());
-
       poamed_ior = CORBA::IOR (poamed_ref);
-
-      /*
-       * Restore state
-       */
-
-      CORBA::POAMediator::StateInf_var poastate =
-	poamed->restore_state (impl_name.c_str());
-
-      for (CORBA::ULong i=0; i<poastate->length(); i++) {
-	persistent_state[(const char *) poastate[i].name] =
-	  (const char *) poastate[i].info;
-      }
     }
   }
 
@@ -1927,7 +1899,7 @@ MICOPOA::POA_impl::POA_impl (CORBA::ORB_ptr _orb)
    */
 
   orb->set_initial_reference ("RootPOA", this);
-  orb->register_oa (this, OAPriorityLocal);
+  orb->register_oa (this);
 }
 
 MICOPOA::POA_impl::~POA_impl ()
@@ -2052,42 +2024,12 @@ MICOPOA::POA_impl::destroy (CORBA::Boolean etherealize_objects,
     child->destroy (etherealize_objects, wait_for_completion);
   }
 
-  /*
-   * Persistent POAs save their state
-   */
-
-  if (lifespan_policy->value() == PortableServer::PERSISTENT) {
-    CORBA::String_var mystate = idfactory.state();
-    persistent_state[oaid] = (const char *) mystate;
-  }
-
   /*
    * The RootPOA says goodbye to the POAMediator
    */
 
   if (!parent) {
     if (!CORBA::is_nil (poamed)) {
-      /*
-       * Save state information
-       */
-
-      CORBA::POAMediator::StateInf poastate;
-      POAStateMap::iterator it;
-      CORBA::ULong length=0;
-
-      for (it = persistent_state.begin(); it != persistent_state.end(); it++) {
-	poastate.length (length+1);
-	poastate[length].name = CORBA::string_dup ((*it).first.c_str());
-	poastate[length].info = CORBA::string_dup ((*it).second.c_str());
-	length++;
-      }
-      
-      poamed->save_state (impl_name.c_str(), poastate);
-
-      /*
-       * Deactivate myself
-       */
-
       poamed->deactivate_impl (impl_name.c_str());
     }
   }
@@ -2262,10 +2204,21 @@ MICOPOA::POA_impl::activate_object (PortableServer::Servant servant)
 
   /*
    * Generate Object Reference
+   *
+   * In a persistent POA, we prefix the Object Id with oaprefix. This
+   * takes care of Object Id Uniqueness in case of server restart.
    */
 
   CORBA::String_var uid = idfactory.new_id ();
-  PortableServer::ObjectId * id = PortableServer::string_to_ObjectId (uid);
+  string fulluid;
+
+  if (lifespan_policy->value() == PortableServer::PERSISTENT) {
+    fulluid = oaprefix;
+  }
+  fulluid += uid.in();
+
+  PortableServer::ObjectId * id =
+    PortableServer::string_to_ObjectId (fulluid.c_str());
   CORBA::String_var iface = servant->_primary_interface (*id, this);
   POAObjectReference * por = new POAObjectReference (this, *id, iface,
 						     servant);
@@ -2387,11 +2340,18 @@ MICOPOA::POA_impl::create_reference (const char * repoid)
   }
 
   CORBA::String_var uid = idfactory.new_id ();
-  PortableServer::ObjectId * id = PortableServer::string_to_ObjectId (uid);
-  POAObjectReference * por = new POAObjectReference (this, *id, repoid);
+  string fulluid;
+
+  if (lifespan_policy->value() == PortableServer::PERSISTENT) {
+    fulluid = oaprefix;
+  }
+  fulluid += uid.in();
+
+  PortableServer::ObjectId_var id =
+    PortableServer::string_to_ObjectId (fulluid.c_str());
+  POAObjectReference * por = new POAObjectReference (this, id.in(), repoid);
   CORBA::Object_ptr obj = por->ref();
   assert (!CORBA::is_nil (obj));
-  delete id;
   delete por;
   return obj;
 }
@@ -2542,7 +2502,7 @@ MICOPOA::POA_impl::servant_to_reference (PortableServer::Servant servant)
    */
 
   if (servant_retention_policy->value() != PortableServer::RETAIN ||
-      (id_uniqueness_policy->value() != PortableServer::MULTIPLE_ID &&
+      (id_uniqueness_policy->value() != PortableServer::UNIQUE_ID &&
        implicit_activation_policy->value() != PortableServer::IMPLICIT_ACTIVATION)) {
     mico_throw (PortableServer::POA::WrongPolicy());
   }
@@ -2712,14 +2672,19 @@ MICOPOA::POA_impl::id_to_reference (const PortableServer::ObjectId & id)
 PortableServer::Servant
 MICOPOA::POA_impl::preinvoke (CORBA::Object_ptr obj)
 {
+  switch (state) {
+  case PortableServer::POAManager::HOLDING:
+  case PortableServer::POAManager::DISCARDING:
+    mico_throw (CORBA::TRANSIENT());
+
+  case PortableServer::POAManager::INACTIVE:
+    return NULL;
+  }
+
   ObjectMap::ObjectRecord * orec = ActiveObjectMap.find (this, obj);
   PortableServer::Servant serv;
   POAObjectReference * por;
 
-  if (state != PortableServer::POAManager::ACTIVE) {
-    return NULL;
-  }
-
   if (orec) {
     serv = orec->serv;
     por = orec->por;
@@ -2787,7 +2752,8 @@ MICOPOA::POA_impl::has_object (CORBA::Object_ptr obj)
 
   if (impl_name.size() > 0 && (CORBA::ULong) length >= impl_name.size()) {
     if (strncmp ((const char *) key, impl_name.c_str(),
-		 impl_name.size()) == 0) {
+		 impl_name.size()) == 0 &&
+	(length==impl_name.size() || key[impl_name.size()] == '/')) {
       return TRUE;
     }
   }
@@ -2854,8 +2820,7 @@ MICOPOA::POA_impl::invoke (CORBA::ORB::MsgId msgid,
      * there's no sense in reactivating transient POA's.
      */
 
-    if (impl_name.size() == 0 ||
-	strncmp (por.poa_name(), impl_name.c_str(), impl_name.size()) != 0) {
+    if (impl_name.size() == 0 || *por.poa_name() == '/') {
       InvocationRecord_var ir = new InvocationRecord (msgid, &por, req, pr);
       CORBA::ServerRequest_ptr svr = ir->make_dyn_req (this);
       CORBA::Exception * ex =
@@ -2943,7 +2908,7 @@ MICOPOA::POA_impl::local_invoke (InvocationRecord_ptr ir)
       theior->objid (obj->_ior()->objid());
       CORBA::Object_var local_ref = new CORBA::Object (theior);
       orb->answer_invoke (ir->id(), CORBA::InvokeForward,
-			  local_ref, ir->get_or());
+			  local_ref, ir->get_or(), 0);
     }
     return;
   }
@@ -3023,6 +2988,12 @@ MICOPOA::POA_impl::local_invoke (InvocationRecord_ptr ir)
      * Okay, next POA in chain successfully activated
      */
 
+    // adam.stanbridge@roke.co.uk to fix crash on bad iioploc url
+    if (!poa) {
+      perform_invoke (ir);
+      return;
+    }
+
     poa->local_invoke (ir);
     return;
   }
@@ -3050,20 +3021,62 @@ MICOPOA::POA_impl::builtin_is_a (InvocationRecord_ptr ir,
    */
 
   CORBA::NVList_ptr args = new CORBA::NVList (0);
-  char *repoid;
+  const char *repoid;
 
   args->add (CORBA::ARG_IN);
-  args->item(0)->value()->type (CORBA::_tc_string);
-  r = svr->params (args);
-  assert (r);
-  r = (*args->item(0)->value() >>= repoid);
-  assert (r);
+  args->item(0)->value()->set_type (CORBA::_tc_string);
+  if (svr->params (args)) {
+    r = (*args->item(0)->value() >>= repoid);
+    assert (r);
 
-  isa = serv->_is_a (repoid);
+#ifdef HAVE_EXCEPTIONS
+    try {
+#endif
+      isa = serv->_is_a (repoid);
+#ifdef HAVE_EXCEPTIONS
+    } catch (CORBA::SystemException_catch & sex) {
+      svr->exception (sex->_clone());
+      return;
+    }
+#endif
 
-  CORBA::Any * res = new CORBA::Any;
-  *res <<= CORBA::Any::from_boolean (isa);
-  svr->result (res);
+    CORBA::Any * res = new CORBA::Any;
+    *res <<= CORBA::Any::from_boolean (isa);
+    svr->result (res);
+  }
+  else {
+    assert (0);
+  }
+}
+
+void
+MICOPOA::POA_impl::builtin_component (InvocationRecord_ptr ir,
+				      PortableServer::Servant serv)
+{
+  CORBA::ServerRequest_ptr svr = ir->make_dyn_req (this);
+  CORBA::Object_var comp;
+  CORBA::Boolean r;
+
+  CORBA::NVList_ptr args = new CORBA::NVList (0);
+  if (svr->params (args)) {
+#ifdef HAVE_EXCEPTIONS
+    try {
+#endif
+      comp = serv->_get_component ();
+#ifdef HAVE_EXCEPTIONS
+    } catch (CORBA::SystemException_catch & sex) {
+      svr->exception (sex->_clone());
+      return;
+    }
+#endif
+
+    CORBA::Any *res = new CORBA::Any;
+    *res <<= comp.in();
+    svr->result (res);
+  }
+  else {
+    assert (0);
+  }
 }
 
 void
@@ -3075,30 +3088,32 @@ MICOPOA::POA_impl::builtin_interface (InvocationRecord_ptr ir,
   CORBA::Boolean r;
 
   CORBA::NVList_ptr args = new CORBA::NVList (0);
-  r = svr->params (args);
-  assert (r);
-
+  if (svr->params (args)) {
 #ifdef HAVE_EXCEPTIONS
-  try {
+    try {
 #endif
-    ifd = serv->_get_interface ();
+      ifd = serv->_get_interface ();
 #ifdef HAVE_EXCEPTIONS
-  } catch (CORBA::SystemException_catch & sex) {
-    svr->exception (sex->_clone());
-    return;
-  }
+    } catch (CORBA::SystemException_catch & sex) {
+      svr->exception (sex->_clone());
+      return;
+    }
 #endif
 
-  if (CORBA::is_nil (ifd)) {
-    CORBA::Exception * ex =
-      new CORBA::OBJ_ADAPTER (0, CORBA::COMPLETED_NO);
-    svr->exception (ex);
-    return;
-  }
+    if (CORBA::is_nil (ifd)) {
+      CORBA::Exception * ex =
+        new CORBA::OBJ_ADAPTER (0, CORBA::COMPLETED_NO);
+      svr->exception (ex);
+      return;
+    }
 
-  CORBA::Any *res = new CORBA::Any;
-  *res <<= ifd;
-  svr->result (res);
+    CORBA::Any *res = new CORBA::Any;
+    *res <<= ifd.in();
+    svr->result (res);
+  }
+  else {
+    assert (0);
+  }
 }
 
 void
@@ -3109,14 +3124,25 @@ MICOPOA::POA_impl::builtin_non_existent (InvocationRecord_ptr ir,
   CORBA::Boolean r, non_existent=TRUE;
 
   CORBA::NVList_ptr args = new CORBA::NVList (0);
-  r = svr->params (args);
-  assert (r);
+  if (svr->params (args)) {
+#ifdef HAVE_EXCEPTIONS
+    try {
+#endif
+      non_existent = serv->_non_existent ();
+#ifdef HAVE_EXCEPTIONS
+    } catch (CORBA::SystemException_catch & sex) {
+      svr->exception (sex->_clone());
+      return;
+    }
+#endif
 
-  non_existent = serv->_non_existent ();
-
-  CORBA::Any * res = new CORBA::Any;
-  *res <<= CORBA::Any::from_boolean (non_existent);
-  svr->result (res);
+    CORBA::Any * res = new CORBA::Any;
+    *res <<= CORBA::Any::from_boolean (non_existent);
+    svr->result (res);
+  }
+  else {
+    assert (0);
+  }
 }
 
 bool
@@ -3125,20 +3151,34 @@ MICOPOA::POA_impl::builtin_invoke (InvocationRecord_ptr ir,
 {
   const char * opname = ir->get_or()->op_name();
 
-  if (strcmp (opname, "_is_a") == 0) {
-    builtin_is_a (ir, serv);
-    return true;
+#ifdef HAVE_EXCEPTIONS
+  try {
+#endif
+    if (strcmp (opname, "_is_a") == 0) {
+      builtin_is_a (ir, serv);
+      return true;
+    }
+    else if (strcmp (opname, "_interface") == 0) {
+      builtin_interface (ir, serv);
+      return true;
+    }
+    else if (strcmp (opname, "_component") == 0) {
+      builtin_component (ir, serv);
+      return true;
+    }
+    else if (strcmp (opname, "_non_existent") == 0) {
+      builtin_non_existent (ir, serv);
+      return true;
+    }
+    
+    return false;
+#ifdef HAVE_EXCEPTIONS
+  } catch (CORBA::SystemException_catch & sex) {
+    CORBA::ServerRequest_ptr svr = ir->make_dyn_req (this);
+    svr->exception (sex->_clone());
   }
-  else if (strcmp (opname, "_interface") == 0) {
-    builtin_interface (ir, serv);
-    return true;
-  }
-  else if (strcmp (opname, "_non_existent") == 0) {
-    builtin_non_existent (ir, serv);
-    return true;
-  }
-
-  return false;
+  return true;
+#endif
 }
 
 bool
@@ -3148,6 +3188,7 @@ MICOPOA::POA_impl::is_builtin (InvocationRecord_ptr ir)
 
   if (strcmp (opname, "_is_a")         == 0 ||
       strcmp (opname, "_interface")    == 0 ||
+      strcmp (opname, "_component")    == 0 ||
       strcmp (opname, "_non_existent") == 0) {
     return true;
   }
@@ -3229,7 +3270,7 @@ MICOPOA::POA_impl::perform_invoke (InvocationRecord_ptr ir)
       } catch (PortableServer::ForwardRequest_catch & fwr) {
 	orb->answer_invoke (ir->id(), CORBA::InvokeForward,
 			    fwr->forward_reference,
-			    ir->get_or());
+			    ir->get_or(), 0);
 	return;
       } catch (CORBA::SystemException_catch & sex1) {
 	CORBA::ServerRequest_ptr svr = ir->make_dyn_req (this);
@@ -3247,11 +3288,19 @@ MICOPOA::POA_impl::perform_invoke (InvocationRecord_ptr ir)
       }
 
       /*
-       * The new servant is placed in the active map
+       * Activate new servant and place in Active Object Map
        */
 
-      POAObjectReference * npor = new POAObjectReference (*ir->get_por());
-      ActiveObjectMap.add (npor, servant);
+#ifdef HAVE_EXCEPTIONS
+      try {
+#endif
+	activate_object_with_id (ir->get_por()->get_id(), servant);
+#ifdef HAVE_EXCEPTIONS
+      } catch (...) {
+	servant->_remove_ref ();
+	throw;
+      }
+#endif
     }
     else {
       slo = PortableServer::ServantLocator::_narrow (servant_manager);
@@ -3273,7 +3322,7 @@ MICOPOA::POA_impl::perform_invoke (InvocationRecord_ptr ir)
       } catch (PortableServer::ForwardRequest_catch & fwr) {
 	orb->answer_invoke (ir->id(), CORBA::InvokeForward,
 			    fwr->forward_reference,
-			    ir->get_or());
+			    ir->get_or(), 0);
 	return;
       } catch (CORBA::SystemException_catch & sex2) {
 	CORBA::ServerRequest_ptr svr = ir->make_dyn_req (this);
@@ -3390,7 +3439,7 @@ MICOPOA::POA_impl::bind (CORBA::ORB::MsgId msgid, const char *repoid,
 CORBA::Boolean
 MICOPOA::POA_impl::locate (CORBA::ORB::MsgId msgid, CORBA::Object_ptr obj)
 {
-  orb->answer_locate (msgid, CORBA::LocateHere, CORBA::Object::_nil());
+  orb->answer_locate (msgid, CORBA::LocateHere, CORBA::Object::_nil(), 0);
   return TRUE;
 }
 
@@ -3472,5 +3521,5 @@ MICOPOA::POA_impl::answer_invoke (CORBA::ORB::MsgId msgid,
 				  CORBA::ORBRequest * oreq,
 				  CORBA::InvokeStatus stat)
 {
-  orb->answer_invoke (msgid, stat, CORBA::Object::_nil(), oreq);
+  orb->answer_invoke (msgid, stat, CORBA::Object::_nil(), oreq, 0);
 }
