@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #define MICO_CONF_IMR
@@ -91,7 +91,7 @@ mico_bio_read (BIO *b, char *out, int len)
 }
 
 static int
-mico_bio_write (BIO *b, char *in, int len)
+mico_bio_write (BIO *b, const char *in, int len)
 {
     int ret;
 
@@ -108,13 +108,13 @@ mico_bio_write (BIO *b, char *in, int len)
 }
 
 static int
-mico_bio_puts (BIO *b, char *out)
+mico_bio_puts (BIO *b, const char *out)
 {
     return mico_bio_write (b, out, strlen (out));
 }
 
 static long
-mico_bio_ctrl (BIO *b, int cmd, long num, char *ptr)
+mico_bio_ctrl (BIO *b, int cmd, long num, void *ptr)
 {
     long ret = 1;
 
@@ -139,7 +139,7 @@ mico_bio_ctrl (BIO *b, int cmd, long num, char *ptr)
 	    if (!ptr)
 		ret = 0;
 	    else
-		*(char **)ptr = b->ptr;
+		*(char **)ptr = (char *)b->ptr;
 	} else {
 	    ret = -1;
 	}
@@ -172,13 +172,13 @@ mico_bio_ctrl (BIO *b, int cmd, long num, char *ptr)
 
 static BIO_METHOD mico_bio_methods = {
     BIO_TYPE_MEM, "mico_bio",
-    (ssl_int_t)  mico_bio_write,
-    (ssl_int_t)  mico_bio_read,
-    (ssl_int_t)  mico_bio_puts,
-    (ssl_int_t)  0, // mico_bio_gets
-    (ssl_long_t) mico_bio_ctrl,
-    (ssl_int_t)  mico_bio_new,
-    (ssl_int_t)  mico_bio_free
+    mico_bio_write,
+    mico_bio_read,
+    mico_bio_puts,
+    0, // mico_bio_gets
+    mico_bio_ctrl,
+    mico_bio_new,
+    mico_bio_free
 };
 
 static BIO_METHOD *BIO_mico ()
@@ -241,9 +241,10 @@ MICOSSL::SSLAddress::make_transport_server () const
 CORBA::IORProfile *
 MICOSSL::SSLAddress::make_ior_profile (CORBA::Octet *key,
 				       CORBA::ULong keylen,
-				       const CORBA::MultiComponent &mc) const
+				       const CORBA::MultiComponent &mc,
+                                       CORBA::UShort version) const
 {
-    return new SSLProfile (key, keylen, *this, mc);
+    return new SSLProfile (key, keylen, *this, mc, version);
 }
 
 CORBA::Boolean
@@ -383,9 +384,12 @@ MICOSSL::SSLTransport::ssl_verify_callback (int ok, X509_STORE_CTX *ctx)
     int depth = X509_STORE_CTX_get_error_depth (ctx);
 
     if (!ok) {
-	MICODebug::instance()->printer()
-	    << "SSL verify error: " << X509_verify_cert_error_string (error)
+        if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	  MICO::Logger::Stream (MICO::Logger::Error)
+	    << "SSL: verify error: " << X509_verify_cert_error_string (error)
 	    << endl;
+	}
+
 	// XXX
 	if (_ssl_verify_depth <= depth) {
 	    ok = 1;
@@ -437,26 +441,32 @@ MICOSSL::SSLTransport::setup_ctx ()
 
     if (cipher)
 	SSL_CTX_set_cipher_list (_ssl_ctx, (char *)cipher);
-    SSL_CTX_set_verify (_ssl_ctx, verify, (ssl_int_t)ssl_verify_callback);
+    SSL_CTX_set_verify (_ssl_ctx, verify, ssl_verify_callback);
 
     SSL_CTX_set_client_CA_list (_ssl_ctx,
-				SSL_load_client_CA_file ((char *)cert));
+				SSL_load_client_CA_file ((char *)cafile));
 
     if (SSL_CTX_use_certificate_file (_ssl_ctx, (char *)cert,
 				      SSL_FILETYPE_PEM) <= 0) {
-	MICODebug::instance()->printer()
-	    << "cannot set SSL certificate file: " << cert << endl;
+        if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	  MICO::Logger::Stream (MICO::Logger::Error)
+	    << "SSL: cannot set SSL certificate file: " << cert << endl;
+	}
 	return FALSE;
     }
     if (SSL_CTX_use_PrivateKey_file (_ssl_ctx, (char *)key,
 				     SSL_FILETYPE_PEM) <= 0) {
-	MICODebug::instance()->printer()
-	    << "cannot set SSL private key file: " << key << endl;
+        if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	  MICO::Logger::Stream (MICO::Logger::Error)
+	    << "SSL: cannot set SSL private key file: " << key << endl;
+	}
 	return FALSE;
     }
     if (SSL_CTX_check_private_key (_ssl_ctx) <= 0) {
-	MICODebug::instance()->printer()
-	    << "bad private key" << endl;
+        if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	  MICO::Logger::Stream (MICO::Logger::Error)
+	    << "SSL: bad private key" << endl;
+	}
 	return FALSE;
     }
 
@@ -467,8 +477,10 @@ MICOSSL::SSLTransport::setup_ctx ()
     
     if ((!ret && (capath || cafile)) ||
 	!SSL_CTX_set_default_verify_paths (_ssl_ctx)) {
-	MICODebug::instance()->printer()
-	    << "cannot set SSL verify locations" << endl;
+        if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	  MICO::Logger::Stream (MICO::Logger::Error)
+	    << "SSL: cannot set SSL verify locations" << endl;
+	}
 	return FALSE;
     }
     return TRUE;
@@ -587,6 +599,14 @@ MICOSSL::SSLTransport::isblocking ()
     return _transp->isblocking();
 }
 
+CORBA::Boolean
+MICOSSL::SSLTransport::isreadable ()
+{
+    // XXX if transport is readable that doesnt mean that SSL_read()
+    // can read something ...
+    return _transp->isreadable();
+}
+
 CORBA::Long
 MICOSSL::SSLTransport::read (void *_b, CORBA::Long len)
 {
@@ -597,7 +617,7 @@ MICOSSL::SSLTransport::read (void *_b, CORBA::Long len)
 }
 
 CORBA::Long
-MICOSSL::SSLTransport::write (void *_b, CORBA::Long len)
+MICOSSL::SSLTransport::write (const void *_b, CORBA::Long len)
 {
     int i = SSL_write (_ssl, (char *)_b, len);
     if (i < 0)
@@ -775,7 +795,8 @@ tag_to_ssltag (CORBA::IORProfile::ProfileId tag)
 
 MICOSSL::SSLProfile::SSLProfile (CORBA::Octet *o, CORBA::ULong l,
 				 const SSLAddress &ia,
-				 const CORBA::MultiComponent &mc)
+				 const CORBA::MultiComponent &mc,
+                                 CORBA::UShort version)
     : _myaddr (ia)
 {
     // copy port from IIOP profile to SSLComponent
@@ -786,7 +807,7 @@ MICOSSL::SSLProfile::SSLProfile (CORBA::Octet *o, CORBA::ULong l,
     CORBA::MultiComponent mc2 (mc);
     mc2.add_component (new SSLComponent (port));
 
-    _prof = ia.content()->make_ior_profile (o, l, mc2);
+    _prof = ia.content()->make_ior_profile (o, l, mc2, version);
 }
 
 MICOSSL::SSLProfile::SSLProfile (CORBA::IORProfile *ior, const SSLAddress &ia)
@@ -1056,7 +1077,19 @@ MICOSSL::SSLPrincipal::get_property (const char *prop_name)
 
 /***************************** SSLComponent **************************/
 
-
+#if 0
+MICOSSL::SSLComponent::SSLComponent (CORBA::UShort port)
+    : _port (port)
+{
+    // this is a guess, dont know how to setup these correctly
+    _target_supports =
+        Security::Integrity|
+        Security::Confidentiality|
+        Security::EstablishTrustInTarget|
+        Security::EstablishTrustInClient;
+    _target_requires = 0;
+}
+#endif
 MICOSSL::SSLComponent::SSLComponent (
     CORBA::UShort port,
     Security::AssociationOptions target_supports,
