@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #define MICO_CONF_IMR
@@ -52,6 +52,13 @@ CORBA::StaticTypeInfo::typecode ()
     return CORBA::TypeCode::_nil();
 }
 
+void
+CORBA::StaticTypeInfo::release (StaticValueType v) const
+{
+    // We only release parameters which are managed indirectly through
+    // a pointer (e.g., string). Default is to do nothing
+}
+
 
 //----------------
 
@@ -466,6 +473,8 @@ public:
     { *(_MICO_T *)d = CORBA::string_dup (*(_MICO_T *)s); }
     void free (StaticValueType v) const
     { CORBA::string_free (*(_MICO_T *)v); delete (_MICO_T *)v; }
+    void release (StaticValueType v) const
+    { CORBA::string_free (*(_MICO_T *)v); }
     // XXX calling this multiple times with same v causes mem leaks
     CORBA::Boolean demarshal (CORBA::DataDecoder &dc, StaticValueType v) const
     { return dc.get_string (*(_MICO_T *)v); }
@@ -490,6 +499,8 @@ public:
     { *(_MICO_T *)d = CORBA::wstring_dup (*(_MICO_T *)s); }
     void free (StaticValueType v) const
     { CORBA::wstring_free (*(_MICO_T *)v); delete (_MICO_T *)v; }
+    void release (StaticValueType v) const
+    { CORBA::wstring_free (*(_MICO_T *)v); }
     // XXX calling this multiple times with same v causes mem leaks
     CORBA::Boolean demarshal (CORBA::DataDecoder &dc, StaticValueType v) const
     { return dc.get_wstring (*(_MICO_T *)v); }
@@ -537,6 +548,8 @@ public:
     { *(_MICO_T *)d = CORBA::TypeCode::_duplicate (*(_MICO_T *)s); }
     void free (StaticValueType v) const
     { CORBA::release (*(_MICO_T *)v); delete (_MICO_T *)v; }
+    void release (StaticValueType v) const
+    { CORBA::release (*(_MICO_T *)v); }
     // XXX calling this multiple times with same v causes mem leaks
     CORBA::Boolean demarshal (CORBA::DataDecoder &dc, StaticValueType v) const
     {
@@ -571,6 +584,8 @@ public:
     { *(_MICO_T *)d = CORBA::Principal::_duplicate (*(_MICO_T *)s); }
     void free (StaticValueType v) const
     { CORBA::release (*(_MICO_T *)v); delete (_MICO_T *)v; }
+    void release (StaticValueType v) const
+    { CORBA::release (*(_MICO_T *)v); }
     // XXX calling this multiple times with same v causes mem leaks
     CORBA::Boolean demarshal (CORBA::DataDecoder &dc, StaticValueType v) const
     {
@@ -605,6 +620,8 @@ public:
     { *(_MICO_T *)d = CORBA::Context::_duplicate (*(_MICO_T *)s); }
     void free (StaticValueType v) const
     { CORBA::release (*(_MICO_T *)v); delete (_MICO_T *)v; }
+    void release (StaticValueType v) const
+    { CORBA::release (*(_MICO_T *)v); }
     // XXX calling this multiple times with same v causes mem leaks
     CORBA::Boolean demarshal (CORBA::DataDecoder &dc, StaticValueType v) const
     {
@@ -639,6 +656,8 @@ public:
     { *(_MICO_T *)d = CORBA::Object::_duplicate (*(_MICO_T *)s); }
     void free (StaticValueType v) const
     { CORBA::release (*(_MICO_T *)v); delete (_MICO_T *)v; }
+    void release (StaticValueType v) const
+    { CORBA::release (*(_MICO_T *)v); }
     // XXX calling this multiple times with same v causes mem leaks
     CORBA::Boolean demarshal (CORBA::DataDecoder &dc, StaticValueType v) const
     {
@@ -685,6 +704,8 @@ public:
     { CORBA::add_ref (*(_MICO_T *)s); *(_MICO_T *)d = *(_MICO_T *)s; }
     void free (StaticValueType v) const
     { CORBA::remove_ref (*(_MICO_T *)v); delete (_MICO_T *)v; }
+    void release (StaticValueType v) const
+    { CORBA::remove_ref (*(_MICO_T *)v); }
     CORBA::Boolean demarshal (CORBA::DataDecoder &dc, StaticValueType v) const
     {
 	return CORBA::ValueBase::_demarshal (
@@ -713,6 +734,8 @@ public:
     { *(_MICO_T *)d = CORBA::AbstractBase::_duplicate (*(_MICO_T *)s); }
     void free (StaticValueType v) const
     { CORBA::release (*(_MICO_T *)v); delete (_MICO_T *)v; }
+    void release (StaticValueType v) const
+    { CORBA::release (*(_MICO_T *)v); }
     CORBA::Boolean demarshal (CORBA::DataDecoder &dc, StaticValueType v) const
     {
 	return CORBA::AbstractBase::_demarshal (dc, *(_MICO_T *)v);
@@ -1852,6 +1875,13 @@ CORBA::StaticRequest::get_in_args (DataEncoder *ec)
 	if (_args[i]->flags() & (CORBA::ARG_IN|CORBA::ARG_INOUT)) {
 	    if (!_args[i]->marshal (*ec))
 		return FALSE;
+	    // Release the memory of inout parameters which are
+	    // managed through a pointer (e.g., string, interface,...)
+	    // If we don't do this, we will have a memory leak when
+	    // the return parameters overwrite the original input
+	    // parameter
+	    if (_args[i]->flags() & CORBA::ARG_INOUT)
+		_args[i]->release();
 	}
     }
 
@@ -2144,7 +2174,8 @@ CORBA::StaticRequest::invoke ()
 	    return;
 	}
 
-	CORBA::InvokeStatus rs = orb->get_invoke_reply (msgid, obj, dummy);
+	GIOP::AddressingDisposition ad;
+	CORBA::InvokeStatus rs = orb->get_invoke_reply (msgid, obj, dummy, ad);
 	switch (rs) {
 	case CORBA::InvokeForward:
 	    // XXX what if _obj is not a stub ???
@@ -2153,6 +2184,11 @@ CORBA::StaticRequest::invoke ()
 	    msgid = orb->invoke_async (_obj, this, Principal::_nil());
 	    break;
 
+	case CORBA::InvokeAddrDisp:
+	  _obj->_ior_fwd()->addressing_disposition (ad);
+	  msgid = orb->invoke_async (_obj, this, Principal::_nil());
+	  break;
+
 	case CORBA::InvokeOk:
 	case CORBA::InvokeUsrEx:
 	    done = TRUE;
@@ -2247,6 +2283,7 @@ CORBA::StaticRequest::get_response ()
     CORBA::Object_var obj;
     CORBA::ORBRequest *dummy;
     CORBA::Boolean done = FALSE;
+    GIOP::AddressingDisposition ad;
 
     assert (_msgid);
 
@@ -2263,7 +2300,8 @@ CORBA::StaticRequest::get_response ()
 	    return;
 	}
 
-	CORBA::InvokeStatus rs = orb->get_invoke_reply (_msgid, obj, dummy);
+	CORBA::InvokeStatus rs = orb->get_invoke_reply (_msgid, obj,
+							dummy, ad);
 	switch (rs) {
 	case CORBA::InvokeForward:
 	    // XXX what if _obj is not a stub ???
@@ -2272,6 +2310,11 @@ CORBA::StaticRequest::get_response ()
 	    _msgid = orb->invoke_async (obj, this, Principal::_nil());
 	    break;
 
+	case CORBA::InvokeAddrDisp:
+	  _obj->_ior_fwd()->addressing_disposition (ad);
+	  _msgid = orb->invoke_async (_obj, this, Principal::_nil());
+	  break;
+
 	case CORBA::InvokeOk:
 	case CORBA::InvokeUsrEx:
 	    done = TRUE;
@@ -2432,8 +2475,10 @@ CORBA::StaticServerRequest::read_args ()
     }
 
     if (!_req->get_in_args (&_args, _ctx)) {
-        MICODebug::instance()->debugger()
-            << "cannot decode args" << endl;
+        if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	  MICO::Logger::Stream (MICO::Logger::Error)
+            << "Error: cannot decode args in StaticServerRequest" << endl;
+	}
         _aborted = TRUE;
         set_exception (new CORBA::MARSHAL());
 	write_results ();
@@ -2471,10 +2516,12 @@ CORBA::StaticServerRequest::write_results ()
 	_req->set_out_args (_env.exception());
     } else {
         if (!_req->set_out_args (_res, &_args)) {
-            MICODebug::instance()->debugger()
-                << "cannot set out args" << endl;
-            CORBA::MARSHAL ex (0, CORBA::COMPLETED_YES);
-            _req->set_out_args (&ex);
+	  if (MICO::Logger::IsLogged (MICO::Logger::Error)) {
+	    MICO::Logger::Stream (MICO::Logger::Error)
+	      << "Error: cannot set out args in StaticServerRequest" << endl;
+	  }
+	  CORBA::MARSHAL ex (0, CORBA::COMPLETED_YES);
+	  _req->set_out_args (&ex);
         }
     }
 }
