@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #define MICO_CONF_IMR
@@ -208,6 +208,11 @@ MICO::TCPTransport::connect (const CORBA::Address *a)
     assert (!strcmp (a->proto(), "inet"));
     InetAddress *ia = (InetAddress *)a;
 
+    if (!ia->valid()) {
+      err = "invalid address";
+      return FALSE;
+    }
+
     struct sockaddr_in sin = ia->sockaddr();
     CORBA::Long r = ::connect (fd, (socket_addr_t)&sin, sizeof (sin));
     if (r < 0) {
@@ -314,7 +319,7 @@ MICO::TCPTransport::read (void *_b, CORBA::Long len)
 }
 
 CORBA::Long
-MICO::TCPTransport::write (void *_b, CORBA::Long len)
+MICO::TCPTransport::write (const void *_b, CORBA::Long len)
 {
     CORBA::Long todo = len;
     CORBA::Octet *b = (CORBA::Octet *)_b;
@@ -881,7 +886,7 @@ MICO::UDPTransport::read (void *b, CORBA::Long len)
 }
 
 CORBA::Long
-MICO::UDPTransport::write (void *_b, CORBA::Long len)
+MICO::UDPTransport::write (const void *_b, CORBA::Long len)
 {
     CORBA::Octet *b = (CORBA::Octet *)_b;
 
@@ -912,19 +917,19 @@ MICO::UDPTransport::write (void *_b, CORBA::Long len)
 const CORBA::Address *
 MICO::UDPTransport::addr ()
 {
+    struct sockaddr_in sin;
+    socket_size_t sz = sizeof (sin);
+    CORBA::Long r = ::getsockname (fd, (socket_addr_t)&sin, &sz);
+    if (r < 0) {
+        OSNet::set_errno();
+        err = xstrerror (errno);
+        return 0;
+    }
+    local_addr.family (MICO::InetAddress::DGRAM);
     if (is_bound) {
-	local_addr.family (MICO::InetAddress::DGRAM);
         local_addr.sockaddr (*addr_sin);
+        local_addr.port (ntohs (sin.sin_port));
     } else {
-        struct sockaddr_in sin;
-        socket_size_t sz = sizeof (sin);
-        CORBA::Long r = ::getsockname (fd, (socket_addr_t)&sin, &sz);
-        if (r < 0) {
-            OSNet::set_errno();
-            err = xstrerror (errno);
-            return 0;
-        }
-        local_addr.family (MICO::InetAddress::DGRAM);
         local_addr.sockaddr (sin);
     }
     return &local_addr;
@@ -1139,19 +1144,19 @@ MICO::UDPTransportServer::accept ()
 const CORBA::Address *
 MICO::UDPTransportServer::addr ()
 {
+    struct sockaddr_in sin;
+    socket_size_t sz = sizeof (sin);
+    CORBA::Long r = ::getsockname (fd, (socket_addr_t)&sin, &sz);
+    if (r < 0) {
+        OSNet::set_errno();
+        err = xstrerror (errno);
+        return 0;
+    }
+    local_addr.family (MICO::InetAddress::DGRAM);
     if (is_bound) {
-	local_addr.family (MICO::InetAddress::DGRAM);
         local_addr.sockaddr (*addr_sin);
+        local_addr.port (ntohs (sin.sin_port));
     } else {
-        struct sockaddr_in sin;
-        socket_size_t sz = sizeof (sin);
-        CORBA::Long r = ::getsockname (fd, (socket_addr_t)&sin, &sz);
-        if (r < 0) {
-            OSNet::set_errno();
-            err = xstrerror (errno);
-            return 0;
-        }
-        local_addr.family (MICO::InetAddress::DGRAM);
         local_addr.sockaddr (sin);
     }
     return &local_addr;
@@ -1372,7 +1377,7 @@ MICO::UnixTransport::read (void *_b, CORBA::Long len)
 }
 
 CORBA::Long
-MICO::UnixTransport::write (void *_b, CORBA::Long len)
+MICO::UnixTransport::write (const void *_b, CORBA::Long len)
 {
     CORBA::Long todo = len;
     CORBA::Octet *b = (CORBA::Octet *)_b;
