@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #define MICO_CONF_IMR
@@ -33,161 +33,12 @@
 #include <mico/util.h>
 #include <mico/throw.h>
 
-#define TK_RECURSIVE 0xffffffff
+#define TK_RECURSIVE ((int)0xffffffff)
 
 
 /********************************************************************/
 
 
-static inline CORBA::Boolean
-is_short_suptype (CORBA::TCKind k, CORBA::Boolean proper = FALSE)
-{
-    switch (k) {
-    case CORBA::tk_short:
-        return !proper;
-
-    case CORBA::tk_long:
-    case CORBA::tk_float:
-    case CORBA::tk_longlong:
-    case CORBA::tk_double:
-    case CORBA::tk_longdouble:
-        return TRUE;
-
-    default:
-        return FALSE;
-    }
-}
-
-static inline CORBA::Boolean
-is_ushort_suptype (CORBA::TCKind k, CORBA::Boolean proper = FALSE)
-{
-    switch (k) {
-    case CORBA::tk_ushort:
-        return !proper;
-
-    case CORBA::tk_long:
-    case CORBA::tk_float:
-    case CORBA::tk_ulong:
-    case CORBA::tk_longlong:
-    case CORBA::tk_double:
-    case CORBA::tk_ulonglong:
-    case CORBA::tk_longdouble:
-        return TRUE;
-
-    default:
-        return FALSE;
-    }
-}
-
-static inline CORBA::Boolean
-is_long_suptype (CORBA::TCKind k, CORBA::Boolean proper = FALSE)
-{
-    switch (k) {
-    case CORBA::tk_long:
-        return !proper;
-
-    case CORBA::tk_longlong:
-    case CORBA::tk_double:
-    case CORBA::tk_longdouble:
-        return TRUE;
-
-    default:
-        return FALSE;
-    }
-}
-
-static inline CORBA::Boolean
-is_float_suptype (CORBA::TCKind k, CORBA::Boolean proper = FALSE)
-{
-    switch (k) {
-    case CORBA::tk_float:
-        return !proper;
-
-    case CORBA::tk_double:
-    case CORBA::tk_longdouble:
-        return TRUE;
-
-    default:
-        return FALSE;
-    }
-}
-
-static inline CORBA::Boolean
-is_ulong_suptype (CORBA::TCKind k, CORBA::Boolean proper = FALSE)
-{
-    switch (k) {
-    case CORBA::tk_ulong:
-        return !proper;
-
-    case CORBA::tk_longlong:
-    case CORBA::tk_double:
-    case CORBA::tk_ulonglong:
-    case CORBA::tk_longdouble:
-        return TRUE;
-
-    default:
-        return FALSE;
-    }
-}
-
-static inline CORBA::Boolean
-is_longlong_suptype (CORBA::TCKind k, CORBA::Boolean proper = FALSE)
-{
-    switch (k) {
-    case CORBA::tk_longlong:
-        return !proper;
-
-    case CORBA::tk_longdouble:
-        return TRUE;
-
-    default:
-        return FALSE;
-    }
-}
-
-static inline CORBA::Boolean
-is_double_suptype (CORBA::TCKind k, CORBA::Boolean proper = FALSE)
-{
-    switch (k) {
-    case CORBA::tk_double:
-        return !proper;
-
-    case CORBA::tk_longdouble:
-        return TRUE;
-
-    default:
-        return FALSE;
-    }
-}
-
-static inline CORBA::Boolean
-is_ulonglong_suptype (CORBA::TCKind k, CORBA::Boolean proper = FALSE)
-{
-    switch (k) {
-    case CORBA::tk_ulonglong:
-        return !proper;
-
-    case CORBA::tk_longdouble:
-        return TRUE;
-
-    default:
-        return FALSE;
-    }
-}
-
-static inline CORBA::Boolean
-is_longdouble_suptype (CORBA::TCKind k, CORBA::Boolean proper = FALSE)
-{
-    switch (k) {
-    case CORBA::tk_longdouble:
-        return !proper;
-
-    default:
-        return FALSE;
-    }
-}
-
-
 /****************************** TypeCode ******************************/
 
 
@@ -230,8 +81,8 @@ CORBA::TypeCode::stringify () const
     return s;
 }
 
-CORBA::TypeCode::TypeCode (TCKind kind)
-    : tckind (kind), len (0), isconst (FALSE)
+CORBA::TypeCode::TypeCode (TCKind kindval)
+    : tckind (kindval), len (0), isconst (FALSE)
 {
     init ();
 }
@@ -274,7 +125,7 @@ CORBA::TypeCode::from_string (const char *_ior)
 
 #ifdef _WINDOWS
     MICO::CDRDecoder dc (&b, FALSE, CORBA::DefaultEndian,
-			 0, TRUE, 0, TRUE, 0, TRUE);
+			 0, TRUE, 0, TRUE);
 #else
     MICO::CDRDecoder dc (&b, FALSE);
 #endif
@@ -350,12 +201,12 @@ CORBA::TypeCode::create_basic_tc (TCKind k)
 
 CORBA::TypeCode_ptr
 CORBA::TypeCode::create_struct_tc (const char *rep_id,
-				   const char *name,
+				   const char *nameval,
 				   const StructMemberSeq &members)
 {
     CORBA::TypeCode_ptr tc = new TypeCode (tk_struct);
     tc->repoid = rep_id ? rep_id : "";
-    tc->tcname = name ? name : "";
+    tc->tcname = nameval ? nameval : "";
 
     for (CORBA::ULong i = 0; i < members.length(); ++i) {
         tc->namevec.push_back (members[i].name.in());
@@ -367,12 +218,12 @@ CORBA::TypeCode::create_struct_tc (const char *rep_id,
 
 CORBA::TypeCode_ptr
 CORBA::TypeCode::create_exception_tc (const char *rep_id,
-				      const char *name,
+				      const char *nameval,
 				      const StructMemberSeq &members)
 {
     CORBA::TypeCode_ptr tc = new TypeCode (tk_except);
     tc->repoid = rep_id ? rep_id : "";
-    tc->tcname = name ? name : "";
+    tc->tcname = nameval ? nameval : "";
 
     for (CORBA::ULong i = 0; i < members.length(); ++i) {
         tc->namevec.push_back (members[i].name.in());
@@ -425,24 +276,15 @@ CORBA::TypeCode::convert_case_label (CORBA::TypeCode_ptr disc,
 }
 
 CORBA::TypeCode_ptr
-CORBA::TypeCode::create_union_tc (const char *rep_id, const char *name,
+CORBA::TypeCode::create_union_tc (const char *rep_id, const char *nameval,
 				  TypeCode_ptr disc_type,
 				  const UnionMemberSeq &members)
 {
     MICO_OBJ_CHECK (disc_type);
 
-#if USE_CORBA_2_1
-    // tset/c++/api/pObject/ORB/VScreate_union_tc1 considers a union
-    // with no members an error. There is probably something broken
-    // in vsOrb. This test will stop some of the MICO tests (those
-    // with recursion) from working. That's why I enclosed it with #ifdef
-    if( members.length() == 0 )
-        mico_throw (CORBA::BAD_PARAM());
-#endif
-
     CORBA::TypeCode_var tc = new TypeCode (tk_union);
     tc->repoid = rep_id ? rep_id : "";
-    tc->tcname = name ? name : "";
+    tc->tcname = nameval ? nameval : "";
     tc->defidx = -1;
 
     tc->discriminator = CORBA::TypeCode::_duplicate (disc_type);
@@ -463,12 +305,12 @@ CORBA::TypeCode::create_union_tc (const char *rep_id, const char *name,
 }
 
 CORBA::TypeCode_ptr
-CORBA::TypeCode::create_enum_tc (const char *rep_id, const char *name,
+CORBA::TypeCode::create_enum_tc (const char *rep_id, const char *nameval,
 				 const EnumMemberSeq &members)
 {
     CORBA::TypeCode_ptr tc = new TypeCode (tk_enum);
     tc->repoid = rep_id ? rep_id : "";
-    tc->tcname = name ? name : "";
+    tc->tcname = nameval ? nameval : "";
 
     for (CORBA::ULong i = 0; i < members.length(); ++i)
         tc->namevec.push_back (members[i].in());
@@ -477,14 +319,14 @@ CORBA::TypeCode::create_enum_tc (const char *rep_id, const char *name,
 }
 
 CORBA::TypeCode_ptr
-CORBA::TypeCode::create_alias_tc (const char *rep_id, const char *name,
+CORBA::TypeCode::create_alias_tc (const char *rep_id, const char *nameval,
 				  TypeCode_ptr orig_type)
 {
     MICO_OBJ_CHECK (orig_type);
 
     CORBA::TypeCode_ptr tc = new TypeCode (tk_alias);
     tc->repoid = rep_id ? rep_id : "";
-    tc->tcname = name ? name : "";
+    tc->tcname = nameval ? nameval : "";
 
     tc->content = CORBA::TypeCode::_duplicate (orig_type);
     tc->content->connect (tc);
@@ -493,11 +335,11 @@ CORBA::TypeCode::create_alias_tc (const char *rep_id, const char *name,
 }
 
 CORBA::TypeCode_ptr
-CORBA::TypeCode::create_interface_tc (const char *rep_id, const char *name)
+CORBA::TypeCode::create_interface_tc (const char *rep_id, const char *nameval)
 {
     CORBA::TypeCode_ptr tc = new TypeCode (tk_objref);
     tc->repoid = rep_id ? rep_id : "";
-    tc->tcname = name ? name : "";
+    tc->tcname = nameval ? nameval : "";
 
     return tc;
 }
@@ -573,7 +415,7 @@ CORBA::TypeCode::create_recursive_sequence_tc (ULong bound, ULong offset)
 }
 
 CORBA::TypeCode_ptr
-CORBA::TypeCode::create_value_tc (const char *rep_id, const char *name,
+CORBA::TypeCode::create_value_tc (const char *rep_id, const char *nameval,
 				  ValueModifier mod,
 				  TypeCode_ptr concrete_base,
 				  const ValueMemberSeq &members)
@@ -583,7 +425,7 @@ CORBA::TypeCode::create_value_tc (const char *rep_id, const char *name,
 
     CORBA::TypeCode_ptr tc = new CORBA::TypeCode (tk_value);
     tc->repoid = rep_id ? rep_id : "";
-    tc->tcname = name ? name : "";
+    tc->tcname = nameval ? nameval : "";
     tc->value_mod = mod;
     tc->len = -1;
     if (!CORBA::is_nil (concrete_base)) {
@@ -596,7 +438,7 @@ CORBA::TypeCode::create_value_tc (const char *rep_id, const char *name,
     for (CORBA::ULong i = 0; i < members.length(); ++i) {
         tc->visvec.push_back (members[i].access);
         tc->namevec.push_back (members[i].name.in());
-        tc->tcvec.push_back (CORBA::TypeCode::_duplicate (members[i].type));
+        tc->tcvec.push_back (CORBA::TypeCode::_duplicate (members[i].type.in()));
         tc->tcvec.back()->connect (tc);
     }
 
@@ -605,14 +447,14 @@ CORBA::TypeCode::create_value_tc (const char *rep_id, const char *name,
 
 CORBA::TypeCode_ptr
 CORBA::TypeCode::create_value_box_tc (const char *rep_id,
-				      const char *name,
+				      const char *nameval,
 				      TypeCode_ptr boxed_type)
 {
     MICO_OBJ_CHECK (boxed_type);
 
     CORBA::TypeCode_ptr tc = new CORBA::TypeCode (tk_value_box);
     tc->repoid = rep_id ? rep_id : "";
-    tc->tcname = name ? name : "";
+    tc->tcname = nameval ? nameval : "";
 
     tc->content = CORBA::TypeCode::_duplicate (boxed_type);
     tc->content->connect (tc);
@@ -622,11 +464,11 @@ CORBA::TypeCode::create_value_box_tc (const char *rep_id,
 
 CORBA::TypeCode_ptr
 CORBA::TypeCode::create_native_tc (const char *rep_id,
-				   const char *name)
+				   const char *nameval)
 {
     CORBA::TypeCode_ptr tc = new CORBA::TypeCode (tk_native);
     tc->repoid = rep_id ? rep_id : "";
-    tc->tcname = name ? name : "";
+    tc->tcname = nameval ? nameval : "";
 
     return tc;
 }
@@ -648,12 +490,26 @@ CORBA::TypeCode::create_recursive_tc (const char *rep_id)
 
 CORBA::TypeCode_ptr
 CORBA::TypeCode::create_abstract_interface_tc (const char *rep_id,
-					       const char *name)
+					       const char *nameval)
+{
+    if (!rep_id || !nameval)
+	mico_throw (CORBA::BAD_PARAM());
+
+    CORBA::TypeCode_ptr tc = new CORBA::TypeCode (tk_abstract_interface);
+    tc->repoid = rep_id ? rep_id : "";
+    tc->tcname = nameval ? nameval : "";
+
+    return tc;
+}
+
+CORBA::TypeCode_ptr
+CORBA::TypeCode::create_local_interface_tc (const char *rep_id,
+					    const char *name)
 {
     if (!rep_id || !name)
 	mico_throw (CORBA::BAD_PARAM());
 
-    CORBA::TypeCode_ptr tc = new CORBA::TypeCode (tk_abstract_interface);
+    CORBA::TypeCode_ptr tc = new CORBA::TypeCode (tk_local_interface);
     tc->repoid = rep_id ? rep_id : "";
     tc->tcname = name ? name : "";
 
@@ -809,6 +665,7 @@ CORBA::TypeCode::equal (TypeCode_ptr tc,
 
     case tk_objref:
     case tk_abstract_interface:
+    case tk_local_interface:
     case tk_native:
         return repoid == tc->repoid;
 
@@ -862,7 +719,7 @@ CORBA::TypeCode::equal (TypeCode_ptr tc,
 		  namevec.size() == tc->namevec.size() &&
 		  discriminator->equal (tc->discriminator, remove_aliases,
                                                       ignore_string_bounds) &&
-		defidx == tc->defidx)) {
+		((defidx == tc->defidx) || (defidx < 0 && tc->defidx < 0)))) {
 		return FALSE;
 	    }
 	    assert (namevec.size() == tcvec.size());
@@ -890,6 +747,12 @@ CORBA::TypeCode::equal (TypeCode_ptr tc,
     }
 }
 
+CORBA::Boolean
+CORBA::TypeCode::equivalent (TypeCode_ptr tc)
+{
+  return equaltype (tc);
+}
+
 CORBA::Boolean
 CORBA::TypeCode::equaltype (TypeCode_ptr tc, SetTC *_cache)
 {
@@ -940,6 +803,7 @@ CORBA::TypeCode::equaltype (TypeCode_ptr tc, SetTC *_cache)
 
     case tk_objref:
     case tk_abstract_interface:
+    case tk_local_interface:
     case tk_native:
         return me->repoid == he->repoid;
 
@@ -1055,254 +919,6 @@ CORBA::TypeCode::equaltype (TypeCode_ptr tc, SetTC *_cache)
     return retval;
 }
 
-CORBA::Boolean
-CORBA::TypeCode::subtype (TypeCode_ptr tc, SetTC *_cache)
-{
-    if (this == tc)
-	return TRUE;
-
-    TypeCode_ptr me = unalias()->resolve_recursion();
-    while (me->tckind == tk_alias)
-	me = me->unalias()->resolve_recursion();
-
-    TypeCode_ptr he = tc->unalias()->resolve_recursion();
-    while (he->tckind == tk_alias)
-	he = he->unalias()->resolve_recursion();
-
-    /*
-     * first check for simple types ...
-     */
-    switch (me->tckind) {
-    case tk_void:
-	return TRUE;
-
-    case tk_null:
-    case tk_octet:
-    case tk_boolean:
-    case tk_char:
-    case tk_wchar:
-    case tk_any:
-    case tk_TypeCode:
-    case tk_Principal:
-	return me->tckind == he->tckind;
-
-    case tk_short:
-        return is_short_suptype (he->tckind);
-
-    case tk_ushort:
-        return is_ushort_suptype (he->tckind);
-
-    case tk_long:
-        return is_long_suptype (he->tckind);
-	
-    case tk_longlong:
-        return is_longlong_suptype (he->tckind);
-
-    case tk_ulong:
-        return is_ulong_suptype (he->tckind);
-
-    case tk_ulonglong:
-        return is_ulonglong_suptype (he->tckind);
-
-    case tk_float:
-        return is_float_suptype (he->tckind);
-
-    case tk_double:
-        return is_double_suptype (he->tckind);
-
-    case tk_longdouble:
-        return is_longdouble_suptype (he->tckind);
-
-    case tk_string:
-	return he->tckind == tk_string && me->len == he->len;
-
-    case tk_wstring:
-	return he->tckind == tk_wstring && me->len == he->len;
-
-    case tk_objref:
-    case tk_abstract_interface:
-    case tk_native:
-	// should use the IR to check whether me is derived from he ...
-        return he->tckind == me->tckind && me->repoid == he->repoid;
-
-
-    case tk_enum:
-	{
-	    if (he->tckind != tk_enum ||
-		me->namevec.size() < he->namevec.size())
-		return FALSE;
-	    for (mico_vec_size_type i = 0; i < he->namevec.size(); ++i) {
-		if (!(me->namevec[i] == he->namevec[i]))
-		    return FALSE;
-	    }
-	    return TRUE;
-	}
-    case tk_fixed:
-        // XXX perhaps support for more flexible subtyping
-        return he->tckind == tk_fixed &&
-            me->digits == he->digits &&
-            me->scale == he->scale;
-
-    case tk_alias:
-    case TK_RECURSIVE:
-	// cannot happen
-	assert (0);
-
-    default:
-	break;
-    }
-
-    /*
-     * for non-simple types the kinds must be the same to be subtypes ...
-     */
-    if (me->tckind != he->tckind)
-	return FALSE;
-
-    /*
-     * check the cache. this is needed for recursive types ...
-     */
-    if (_cache && _cache->count (make_pair (me, he)))
-	return TRUE;
-
-    /*
-     * insert ourself into cache
-     */
-    SetTC *cache = _cache ? _cache : new SetTC;
-    cache->insert (make_pair (me, he));
-
-    Boolean retval = FALSE;
-
-    switch (me->tckind) {
-    case tk_sequence:
-    case tk_array:
-	retval = (me->len == he->len &&
-		  me->content->subtype (he->content, cache));
-	break;
-
-    case tk_except:
-    case tk_struct:
-	{
-	    if (me->namevec.size() < he->namevec.size())
-		break;
-	    assert (me->namevec.size() == me->tcvec.size());
-	    assert (he->namevec.size() == he->tcvec.size());
-	    retval = TRUE;
-	    for (mico_vec_size_type i=0; i < he->tcvec.size(); ++i) {
-		if (!me->tcvec[i]->subtype (he->tcvec[i], cache)) {
-		    retval = FALSE;
-		    break;
-		}
-	    }
-	    break;
-	}
-    case tk_value_box:
-	retval = me->content->subtype (he->content, cache);
-	break;
-
-    case tk_value:
-	{
-	    if (me->value_mod != he->value_mod ||
-                me->namevec.size() < he->namevec.size())
-                break;
-
-	    assert (me->namevec.size() == me->tcvec.size());
-	    assert (me->namevec.size() == me->visvec.size());
-	    assert (he->namevec.size() == he->tcvec.size());
-	    assert (he->namevec.size() == he->visvec.size());
-            retval = TRUE;
-	    for (mico_vec_size_type i = 0; i < me->tcvec.size(); ++i) {
-		if (!me->tcvec[i]->equaltype (he->tcvec[i]) ||
-                    me->visvec[i] != he->visvec[i]) {
-		    retval = FALSE;
-                    break;
-		}
-	    }
-            break;
-	}
-    case tk_union:
-	{
-	    if (me->namevec.size() < he->namevec.size() ||
-		!me->discriminator->subtype (he->discriminator, cache))
-		break;
-	    
-	    assert (me->namevec.size() == me->tcvec.size());
-	    assert (me->namevec.size() == me->labelvec.size());
-	    assert (he->namevec.size() == he->tcvec.size());
-	    assert (he->namevec.size() == he->labelvec.size());
-	    retval = TRUE;
-	    for (mico_vec_size_type i=0; i < he->tcvec.size(); ++i) {
-	    /*
-	    * use Any::equivalent() to compare the label values
-	    * to take care of different discriminant types
-		*/
-		if (!me->tcvec[i]->subtype (he->tcvec[i], cache) ||
-		    !me->labelvec[i]->equivalent (*he->labelvec[i])) {
-		    retval = FALSE;
-		    break;
-		}
-	    }
-	    break;
-	}
-    default:
-	assert (0);
-    }
-
-    /*
-     * remove ourself from cache only if subtype checking failed ...
-     */
-    if (!retval)
-	cache->erase (make_pair (me, he));
-
-    if (!_cache)
-	delete cache;
-
-    return retval;
-}
-
-CORBA::TypeCode_ptr
-CORBA::TypeCode::maxtype (TypeCode_ptr tc)
-{
-    TypeCode_ptr me = unalias();
-    TypeCode_ptr he = tc->unalias();
-
-
-    if (me->tckind == he->tckind)
-	return this;
-
-    switch (me->tckind) {
-    case tk_short:
-        return is_short_suptype (he->tckind, TRUE) ? tc : this;
-
-    case tk_ushort:
-        return is_ushort_suptype (he->tckind, TRUE) ? tc : this;
-
-    case tk_long:
-        return is_long_suptype (he->tckind, TRUE) ? tc : this;
-	
-    case tk_longlong:
-        return is_longlong_suptype (he->tckind, TRUE) ? tc : this;
-	
-    case tk_ulong:
-        return is_ulong_suptype (he->tckind, TRUE) ? tc : this;
-
-    case tk_ulonglong:
-        return is_ulonglong_suptype (he->tckind, TRUE) ? tc : this;
-
-    case tk_float:
-        return is_float_suptype (he->tckind, TRUE) ? tc : this;
-
-    case tk_double:
-        return is_double_suptype (he->tckind, TRUE) ? tc : this;
-
-    case tk_longdouble:
-        return is_longdouble_suptype (he->tckind, TRUE) ? tc : this;
-
-    default:
-	break;
-    }
-    return this;
-}
-
 CORBA::TypeCode_ptr
 CORBA::TypeCode::resolve_recursion() const
 {
@@ -1322,6 +938,7 @@ CORBA::TypeCode::id () const
 	  tckind == tk_union  || tckind == tk_enum ||
 	  tckind == tk_alias  || tckind == tk_except ||
 	  tckind == tk_native || tckind == tk_abstract_interface ||
+	  tckind == tk_local_interface ||
 	  tckind == tk_value || tckind == tk_value_box ||
 	  (tckind == TK_RECURSIVE && repoid.length() > 0)))
 	mico_throw (CORBA::TypeCode::BadKind());
@@ -1335,6 +952,7 @@ CORBA::TypeCode::name () const
 	  tckind == tk_union  || tckind == tk_enum ||
 	  tckind == tk_alias  || tckind == tk_except ||
 	  tckind == tk_native || tckind == tk_abstract_interface ||
+	  tckind == tk_local_interface ||
 	  tckind == tk_value || tckind == tk_value_box))
 	mico_throw (CORBA::TypeCode::BadKind());
     return tcname.c_str();
@@ -1549,7 +1167,7 @@ CORBA::TypeCode::member_name_inherited (ULong idx) const
     if (tckind != tk_value)
 	mico_throw (CORBA::TypeCode::BadKind());
 
-    Long n = member_count_inherited();
+    ULong n = member_count_inherited();
     if (idx >= n)
 	mico_throw (CORBA::TypeCode::Bounds());
     idx = n-1-idx;
@@ -1573,7 +1191,7 @@ CORBA::TypeCode::member_type_inherited (ULong idx) const
     if (tckind != tk_value)
 	mico_throw (CORBA::TypeCode::BadKind());
 
-    Long n = member_count_inherited();
+    ULong n = member_count_inherited();
     if (idx >= n)
 	mico_throw (CORBA::TypeCode::Bounds());
     idx = n-1-idx;
@@ -1597,7 +1215,7 @@ CORBA::TypeCode::member_visibility_inherited (ULong idx) const
     if (tckind != tk_value)
 	mico_throw (CORBA::TypeCode::BadKind());
 
-    Long n = member_count_inherited();
+    ULong n = member_count_inherited();
     if (idx >= n)
 	mico_throw (CORBA::TypeCode::Bounds());
     idx = n-1-idx;
@@ -1623,6 +1241,7 @@ CORBA::TypeCode::param_count () const
     case tk_wstring:
     case tk_objref:
     case tk_abstract_interface:
+    case tk_local_interface:
     case tk_native:
 	return 1;
     case tk_sequence:
@@ -1662,6 +1281,7 @@ CORBA::TypeCode::parameter (Long idx) const
     case tk_objref:
     case tk_native:
     case tk_abstract_interface:
+    case tk_local_interface:
 	*a <<= id();
 	break;
 
@@ -1857,21 +1477,22 @@ CORBA::TypeCode::decode (DataDecoder &dc, MapPosTC *_omap, ULong level)
     case tk_objref:
     case tk_native:
     case tk_abstract_interface:
+    case tk_local_interface:
         check (dc.encaps_begin (state, k));
-        check (dc.get_string (repoid));
-        check (dc.get_string (tcname));
+        check (dc.get_string_stl (repoid));
+        check (dc.get_string_stl (tcname));
         check (dc.encaps_end (state));
         break;
 
     case tk_except:
     case tk_struct: {
         check (dc.encaps_begin (state, k));
-        check (dc.get_string (repoid));
-        check (dc.get_string (tcname));
+        check (dc.get_string_stl (repoid));
+        check (dc.get_string_stl (tcname));
         check (dc.get_ulong (k));
         string s;
         for (CORBA::ULong i = 0; i < k; ++i) {
-            check (dc.get_string (s));
+            check (dc.get_string_stl (s));
             namevec.push_back (s);
             tcvec.push_back (new TypeCode ());
 	    check (tcvec.back()->decode (dc, omap, level+1));
@@ -1882,8 +1503,8 @@ CORBA::TypeCode::decode (DataDecoder &dc, MapPosTC *_omap, ULong level)
     }
     case tk_union: {
         check (dc.encaps_begin (state, k));
-        check (dc.get_string (repoid));
-        check (dc.get_string (tcname));
+        check (dc.get_string_stl (repoid));
+        check (dc.get_string_stl (tcname));
         discriminator = new TypeCode ();
 	check (discriminator->decode (dc, omap, level+1));
         check (dc.get_long (defidx));
@@ -1910,7 +1531,7 @@ CORBA::TypeCode::decode (DataDecoder &dc, MapPosTC *_omap, ULong level)
             }
 #endif
             labelvec.push_back (a);
-            check (dc.get_string (s));
+            check (dc.get_string_stl (s));
             namevec.push_back (s);
             tcvec.push_back (new TypeCode ());
 	    check (tcvec.back()->decode (dc, omap, level+1));
@@ -1921,12 +1542,12 @@ CORBA::TypeCode::decode (DataDecoder &dc, MapPosTC *_omap, ULong level)
     }
     case tk_enum: {
         check (dc.encaps_begin (state, k));
-        check (dc.get_string (repoid));
-        check (dc.get_string (tcname));
+        check (dc.get_string_stl (repoid));
+        check (dc.get_string_stl (tcname));
         check (dc.get_ulong (k));
         string s;
         for (CORBA::ULong i = 0; i < k; ++i) {
-            check (dc.get_string (s));
+            check (dc.get_string_stl (s));
             namevec.push_back (s);
         }
         check (dc.encaps_end (state));
@@ -1951,8 +1572,8 @@ CORBA::TypeCode::decode (DataDecoder &dc, MapPosTC *_omap, ULong level)
 
     case tk_alias:
         check (dc.encaps_begin (state, k));
-        check (dc.get_string (repoid));
-        check (dc.get_string (tcname));
+        check (dc.get_string_stl (repoid));
+        check (dc.get_string_stl (tcname));
         content = new TypeCode ();
 	check (content->decode (dc, omap, level+1));
         content->connect (this);
@@ -1967,8 +1588,8 @@ CORBA::TypeCode::decode (DataDecoder &dc, MapPosTC *_omap, ULong level)
     case tk_value: {
 	len = -1;
         check (dc.encaps_begin (state, k));
-        check (dc.get_string (repoid));
-        check (dc.get_string (tcname));
+        check (dc.get_string_stl (repoid));
+        check (dc.get_string_stl (tcname));
 	check (dc.get_short (value_mod));
 	CORBA::TypeCode_var tmptc = new TypeCode();
 	check (tmptc->decode (dc, omap, level+1));
@@ -1982,7 +1603,7 @@ CORBA::TypeCode::decode (DataDecoder &dc, MapPosTC *_omap, ULong level)
         string s;
 	CORBA::Visibility vis;
         for (CORBA::ULong i = 0; i < k; ++i) {
-            check (dc.get_string (s));
+            check (dc.get_string_stl (s));
             namevec.push_back (s);
             tcvec.push_back (new TypeCode ());
 	    check (tcvec.back()->decode (dc, omap, level+1));
@@ -1995,8 +1616,8 @@ CORBA::TypeCode::decode (DataDecoder &dc, MapPosTC *_omap, ULong level)
     }
     case tk_value_box:
         check (dc.encaps_begin (state, k));
-        check (dc.get_string (repoid));
-        check (dc.get_string (tcname));
+        check (dc.get_string_stl (repoid));
+        check (dc.get_string_stl (tcname));
         content = new TypeCode ();
 	check (content->decode (dc, omap, level+1));
         content->connect (this);
@@ -2004,14 +1625,30 @@ CORBA::TypeCode::decode (DataDecoder &dc, MapPosTC *_omap, ULong level)
         break;
 
     case TK_RECURSIVE: {
-        Long rpos = dc.buffer()->rpos();
 	Long recurse_pos;
 	check (dc.get_long (recurse_pos));
-	recurse_pos += rpos;
-	assert (recurse_pos >= 0);
+        Long rpos = dc.buffer()->rpos() - sizeof(CORBA::Long);
+        recurse_pos += rpos;
+
+#if 1
+        // fix for Visibroker (align to long boundary) ...
+        if (!strcmp (dc.type(), "cdr")) {
+            /*
+             * we dont know the alignment base (AB) for the typecode
+             * referred to, but we know that (CAB := current alignment base):
+             *   AB % 4 == CAB % 4
+             */
+            Long alignbase = dc.buffer()->ralign_base() & 3L;
+            recurse_pos = alignbase + ((recurse_pos - alignbase + 3) & ~3L);
+        }
+#endif
+
+	check (recurse_pos >= 0);
 
 	MapPosTC::iterator i = omap->find (recurse_pos);
-	assert (i != omap->end());
+
+	check (i != omap->end());
+
 	if (CORBA::is_nil ((*i).second.second)) {
 	    // recursive typecode
 	    assert ((*i).second.first < level);
@@ -2023,7 +1660,7 @@ CORBA::TypeCode::decode (DataDecoder &dc, MapPosTC *_omap, ULong level)
 	break;
     }
     default:
-        assert (0);
+      return FALSE;
     }
 
     (*omap)[buffer_pos].second = this;
@@ -2090,7 +1727,8 @@ CORBA::TypeCode::encode (DataEncoder &ec, MapTCPos *_omap) const
 
     case tk_objref:
     case tk_native:
-    case tk_abstract_interface: {
+    case tk_abstract_interface:
+    case tk_local_interface: {
 	CORBA::DataEncoder::EncapsState state;
 	ec.encaps_begin (state);
         ec.put_string (repoid);
@@ -2285,16 +1923,16 @@ CORBA::TypeCode::is_freestanding()
 CORBA::Boolean
 CORBA::TypeCode::is_freestanding( CORBA::ULong depth, CORBA::TypeCode_ptr tc )
 {
-  CORBA::TCKind kind = tc->kind();
-  assert( kind == CORBA::tk_union || kind == CORBA::tk_struct );
+  CORBA::TCKind kindval = tc->kind();
+  assert( kindval == CORBA::tk_union || kindval == CORBA::tk_struct );
   for( CORBA::ULong i = 0; i < tc->member_count(); i++ ) {
     CORBA::TypeCode_var mem_tc = tc->member_type( i );
     if( mem_tc->is_recursive_seq() ) {
       if( mem_tc->get_recurse_depth() > depth )
 	return false;
     }
-    kind = mem_tc->kind();
-    if( kind == CORBA::tk_union || kind == CORBA::tk_struct ) {
+    kindval = mem_tc->kind();
+    if( kindval == CORBA::tk_union || kindval == CORBA::tk_struct ) {
       CORBA::Boolean res = tc->is_freestanding( ++depth, mem_tc );
       if( !res )
 	return false;
@@ -2460,15 +2098,16 @@ CORBA::TypeCode::is_variable()
   if( tc->is_string() || tc->is_wstring() )
     return TRUE;
   
-  CORBA::TCKind kind = tc->kind();
+  CORBA::TCKind kindval = tc->kind();
 
-  if( kind == CORBA::tk_any || kind == CORBA::tk_sequence ||
-      kind == CORBA::tk_objref || kind == CORBA::tk_TypeCode ||
-      kind == CORBA::tk_native || kind == CORBA::tk_abstract_interface ||
-      kind == CORBA::tk_value  || kind == CORBA::tk_value_box )
+  if( kindval == CORBA::tk_any || kindval == CORBA::tk_sequence ||
+      kindval == CORBA::tk_objref || kindval == CORBA::tk_TypeCode ||
+      kindval == CORBA::tk_native || kindval == CORBA::tk_abstract_interface ||
+      kindval == CORBA::tk_local_interface ||
+      kindval == CORBA::tk_value  || kindval == CORBA::tk_value_box )
     return TRUE;
 
-  if( kind == CORBA::tk_struct || kind == CORBA::tk_union ) {
+  if( kindval == CORBA::tk_struct || kindval == CORBA::tk_union ) {
     for( CORBA::ULong i = 0; i < tc->member_count(); i++ ) {
       CORBA::TypeCode_var tc_member;
       tc_member = tc->member_type( i );
@@ -2477,7 +2116,7 @@ CORBA::TypeCode::is_variable()
     }
     return FALSE;
   }
-  if( kind == CORBA::tk_array ) {
+  if( kindval == CORBA::tk_array ) {
     CORBA::TypeCode_var tc_content = tc->content_type();
     if( tc_content->is_variable() )
       return TRUE;
@@ -2493,7 +2132,10 @@ CORBA::TypeCode::is_objref()
 {
   CORBA::TypeCode_ptr tc = this->unalias();
   
-  return tc->kind() == CORBA::tk_objref;
+  return
+    tc->kind() == CORBA::tk_objref ||
+    tc->kind() == CORBA::tk_abstract_interface ||
+    tc->kind() == CORBA::tk_local_interface;
 }
 
 /*
@@ -2505,8 +2147,8 @@ CORBA::TypeCode::is_object()
   CORBA::TypeCode_ptr tc = this->unalias();
 
   if( tc->kind() == CORBA::tk_objref ) {
-    CORBA::String_var name = tc->name();
-    if( strcmp( name, "Object" ) == 0 )
+    CORBA::String_var nameval = tc->name();
+    if( strcmp( nameval, "Object" ) == 0 )
       return true;
   }
   return false;
@@ -2952,9 +2594,9 @@ CORBA::TypeCodeChecker::union_selection (Long sel_idx)
 }
 
 CORBA::Boolean
-CORBA::TypeCodeChecker::union_end ()
+CORBA::TypeCodeChecker::union_end (Boolean sloppy)
 {
-    return leave (LevelRecord::LUnion);
+    return leave (LevelRecord::LUnion, sloppy);
 }
 
 CORBA::Boolean
@@ -2970,9 +2612,9 @@ CORBA::TypeCodeChecker::struct_begin ()
 }
 
 CORBA::Boolean
-CORBA::TypeCodeChecker::struct_end ()
+CORBA::TypeCodeChecker::struct_end (Boolean sloppy)
 {
-    return leave (LevelRecord::LStruct);
+    return leave (LevelRecord::LStruct, sloppy);
 }
 
 CORBA::Boolean
@@ -2988,9 +2630,9 @@ CORBA::TypeCodeChecker::except_begin ()
 }
 
 CORBA::Boolean
-CORBA::TypeCodeChecker::except_end ()
+CORBA::TypeCodeChecker::except_end (Boolean sloppy)
 {
-    return leave (LevelRecord::LExcept);
+    return leave (LevelRecord::LExcept, sloppy);
 }
 
 CORBA::Boolean
@@ -3007,9 +2649,9 @@ CORBA::TypeCodeChecker::seq_begin (ULong len)
 }
 
 CORBA::Boolean
-CORBA::TypeCodeChecker::seq_end ()
+CORBA::TypeCodeChecker::seq_end (Boolean sloppy)
 {
-    return leave (LevelRecord::LSequence);
+    return leave (LevelRecord::LSequence, sloppy);
 }
 
 CORBA::Boolean
@@ -3029,9 +2671,9 @@ CORBA::TypeCodeChecker::arr_begin ()
 }
 
 CORBA::Boolean
-CORBA::TypeCodeChecker::arr_end ()
+CORBA::TypeCodeChecker::arr_end (Boolean sloppy)
 {
-    return leave (LevelRecord::LArray);
+    return leave (LevelRecord::LArray, sloppy);
 }
 
 CORBA::Boolean
