@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  *
  * 	date     vers.  name            reason
  *	97/07/01 0.1    mueller         CREATED
@@ -665,7 +665,8 @@ uni_slong uni_fromUTF8(char *dest, const char *utf8, uni_uword *chars, uni_uword
   const cexcepttable  *except, *ibmexcept;
   uni_ulong unknown;
 
-  if( (string_type==C_UTF8) || (string_type==(uni_ulong)C_ASCII7) )
+  if( (string_type==C_UTF8) || (string_type==(uni_ulong)C_ASCII7) ||
+      (string_type==C_ISO646) )
   {
     for(ci=si=di=0; ci < (*chars); si++)
       switch(utf8[si])
@@ -686,7 +687,8 @@ uni_slong uni_fromUTF8(char *dest, const char *utf8, uni_uword *chars, uni_uword
           if( ((utf8[si] & 0x80) == 0) || ((utf8[si] & 0xC0) == 0x80) )
             ci++;
 
-          if(string_type==(uni_ulong)C_ASCII7)
+          if(string_type==(uni_ulong)C_ASCII7 ||
+	     string_type==C_ISO646)
             if((utf8[si] & 0x80) != 0)
               if((utf8[si] & 0x40) != 0)
                 di--;
@@ -757,7 +759,7 @@ uni_slong uni_fromUTF8(char *dest, const char *utf8, uni_uword *chars, uni_uword
     return(C_OK);
   }
 
-  if(string_type==C_UTF16)
+  if(string_type==C_UTF16 || string_type==C_UCS2)
   {
     utf16 = (uni_ushort*)dest;
     for(ci=si=di=0; si < (*chars); si++, ci++, di++)
@@ -780,11 +782,11 @@ uni_slong uni_fromUTF8(char *dest, const char *utf8, uni_uword *chars, uni_uword
         }
       else
       {
-        RC = uni_utf8toutf16(utf16, utf8, &pos);
+        RC = uni_utf8toutf16(&utf16[di], &utf8[si], &pos);
         if(RC != C_OK) return(RC);
         si += pos-1;
 
-        if(utf16[1] != 0l) di++;
+        if (utf16[di+1] != 0l) di++;
       }
     }
 
