@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  *
  * 	date     vers.  name            reason
  *	97/07/01 0.1    mueller         CREATED
@@ -917,7 +917,7 @@ uni_slong uni_toUTF8  (char *utf8, const char *src, uni_uword chars,
     return(C_OK);
   }
 
-  if(string_type == (uni_ulong)C_ASCII7)
+  if(string_type == (uni_ulong)C_ASCII7 || string_type == C_ISO646)
   {
     for(si=di=0; si<chars; si++, di++)
       if( (src[si] & 0x80) == 0 )
@@ -970,7 +970,7 @@ uni_slong uni_toUTF8  (char *utf8, const char *src, uni_uword chars,
     return(C_OK);
   }
 
-  if(string_type==C_UTF16)
+  if(string_type==C_UTF16 || string_type==C_UCS2)
   {
     utf16 = (uni_ushort*)src;
     for(si=di=0; si<chars; )
