@@ -26,20 +26,26 @@
 #ifndef _TYPES_H
 #define _TYPES_H
 
-#ifdef _WINDOWS
+#if defined (_WINDOWS)
 #include <mico/config-win32.h>
+#elif defined (_POCKET_PC)
+#include <mico/config-ppc.h>
 #else
 #include <mico/config.h>
 #endif
 #include <limits.h>
 #include <stdlib.h>   /* GNU-C defines here the type "quad" */
+#ifndef _POCKET_PC
 #include <sys/types.h>
+#endif
 
 typedef char  bits8;
 typedef short bits16;
 typedef long  bits32;
 #if defined(_WINDOWS) && !defined(__MINGW32__)
 typedef __int64 bits64;
+#elif defined (_POCKET_PC)
+typedef __int64 bits64;
 #else
 typedef long long bits64;
 #endif
