@@ -1,6 +1,6 @@
 /*
- *  MICO --- a free CORBA implementation
- *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
+ *  MICO --- an Open Source CORBA implementation
+ *  Copyright (c) 1997-2001 by The Mico Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,8 +16,8 @@
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Send comments and/or bug reports to:
- *                 mico@informatik.uni-frankfurt.de
+ *  For more information, visit the MICO Home Page at
+ *  http://www.mico.org/
  */
 
 #define MICO_CONF_IMR
@@ -27,7 +27,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <algorithm>
 #include <iostream.h>
 #include <fstream.h>
 #include <strstream.h>
@@ -40,9 +39,14 @@
 #include <mico/memtrace.h>
 #endif
 
+#ifdef _POCKET_PC
+extern char* getenv (const char*);
+#endif
+
 /******************************* assert *******************************/
 
 
+#ifndef _POCKET_PC
 void
 mico_assert (const char *file, int line)
 {
@@ -52,6 +56,7 @@ mico_assert (const char *file, int line)
 #endif
     abort ();
 }
+#endif
 
 
 /****************************** strerror ******************************/
@@ -72,105 +77,94 @@ extern "C" {
 }
 #endif
 
-const char *
+string
 xstrerror (int err)
 {
-    static string lasterr;
     char *cp = strerror (err);
 
     if (cp)
-        return cp;
+        return string(cp);
+
     // Cygnus CDK returnes (char *)0 for some errnos
+    string lasterr;
     lasterr = "error ";
     lasterr += xdec (errno);
-    return lasterr.c_str();
+    return lasterr;
 }
 
-const char *
+string
 xdec (int i)
 {
-    static string s;
-
     ostrstream ostr;
     ostr << i << ends;
 
-    s = ostr.str();
+    string s = ostr.str();
     ostr.rdbuf()->freeze (0);
-    return s.c_str();
+    return s;
 }
 
-const char *
+string
 xdec (long i)
 {
-    static string s;
-
     ostrstream ostr;
     ostr << i << ends;
 
-    s = ostr.str();
+    string s = ostr.str();
     ostr.rdbuf()->freeze (0);
-    return s.c_str();
+    return s;
 }
 
-const char *
+string
 xdec (OSMisc::longlong i)
 {
-    static string s;
-
     ostrstream ostr;
-#ifdef _WINDOWS
+#if defined(_WINDOWS) && !defined(__MINGW32__)
     ostr << long(i) << ends;   //XXX
 #else
     ostr << i << ends;
 #endif
 
-    s = ostr.str();
+    string s = ostr.str();
     ostr.rdbuf()->freeze (0);
-    return s.c_str();
+    return s;
 }
 
-const char *
+string
 xdec (unsigned int i)
 {
-    static string s;
-
     ostrstream ostr;
     ostr << i << ends;
 
-    s = ostr.str();
+    string s = ostr.str();
     ostr.rdbuf()->freeze (0);
-    return s.c_str();
+    return s;
 }
 
-const char *
+string
 xdec (unsigned long i)
 {
-    static string s;
-
     ostrstream ostr;
     ostr << i << ends;
 
-    s = ostr.str();
+    string s = ostr.str();
     ostr.rdbuf()->freeze (0);
-    return s.c_str();
+    return s;
 }
 
-const char *
+string
 xdec (OSMisc::ulonglong i)
 {
-    static string s;
-
     ostrstream ostr;
 
-#ifdef _WINDOWS
+#if defined(_WINDOWS) && !defined(__MINGW32__) && !defined(__BORLANDC__)
     ostr << unsigned long(i) << ends; //XXX
 #else
     ostr << i << ends;
 #endif
 
-    s = ostr.str();
+    string s = ostr.str();
     ostr.rdbuf()->freeze (0);
-    return s.c_str();
+    return s;
 }
 
 size_t
@@ -289,6 +283,14 @@ mico_url_decode (const char * ptr, CORBA::ULong & len)
     len++;
   }
 
+  /*
+   * Null-terminate the result so that it can be used as a string. The
+   * null is deliberately not added to the length, because it isn't part
+   * of the string.
+   */
+
+  *res = 0;
+
   return str;
 }
 
@@ -337,7 +339,11 @@ CORBA::MethodDispatcher::invoke (CORBA::ServerRequest_ptr _req,
 	if (_dispatcher[i]->dispatch (_req, _env))
 	    return;
     }
-    _req->exception (new BAD_OPERATION);
+#ifdef _POCKET_PC
+    NOT_IMPLEMENTED;
+#else
+	_req->exception (new BAD_OPERATION);
+#endif
 }
 
 
@@ -377,87 +383,121 @@ CORBA::StaticMethodDispatcher::invoke (CORBA::StaticServerRequest_ptr _req,
 /***************************** MICODebug ******************************/
 
 
-MICODebug *MICODebug::_instance = 0;
+MICO::Logger * MICO::Logger::_instance = 0;
+ostream ** MICO::Logger::_out = 0;
 
-MICODebug::MICODebug (const string &file, CORBA::ULong level)
+MICO::Logger::Logger ()
 {
-    assert (!_instance);
-    _instance = this;
+  assert (!_instance);
+  _instance = this;
 
-    _level = level;
-    _out = new ofstream (file.c_str());
-    _null = new ofstream ("/dev/null");
+  _out = new ostream * [All];
 
-    assert (_out);
-    assert (_null);
+  for (int i=0; i<All; i++) {
+    _out[i] = 0;
+  }
+
+  Log (Error);
 }
 
-MICODebug::MICODebug (CORBA::ULong level)
+MICO::Logger::~Logger ()
 {
-    assert (!_instance);
-    _instance = this;
+  _instance = 0;
 
-    _level = level;
-    _out = &cerr;
-    _null = new ofstream ("/dev/null");
-
-    assert (_null);
-}
-
-MICODebug::~MICODebug ()
-{
-    _instance = 0;
-
-    if (_out != &cerr)
-	delete _out;
-    delete _null;
+  for (int i=0; i<All; i++) {
+    if (_out[i] != &cout && _out[i] != &cerr) {
+      delete _out[i];
+    }
+  }
+  
+  delete [] _out;
 }
 
 void
-MICODebug::level (CORBA::ULong l)
-{
-    _level = l;
-}
-
-CORBA::ULong
-MICODebug::level () const
-{
-    return _level;
-}
-
-ostream &
-MICODebug::printer ()
-{
-    return *_out;
-}
-
-ostream &
-MICODebug::debugger ()
-{
-    return _level > 0 ? *_out : *_null;
-}
-
-ostream &
-MICODebug::tracer ()
-{
-    return _level > 1 ? *_out : *_null;
-}
-
-ostream &
-MICODebug::dumper ()
-{
-    return _level > 2 ? *_out : *_null;
-}
-
-MICODebug *
-MICODebug::instance ()
+MICO::Logger::Log (MessageType msg, CORBA::Boolean onoff,
+		   const char * file)
 {
+  if (!_instance) {
+    new Logger;
     assert (_instance);
-    return _instance;
+  }
+
+  if (msg == All) {
+    for (int i=0; i<All; i++) {
+      Log ((MessageType) i, onoff, file);
+    }
+    return;
+  }
+
+  if (_out[msg] != &cout && _out[msg] != &cerr) {
+    delete _out[msg];
+  }
+
+  if (onoff) {
+    if (file) {
+      _out[msg] = new ofstream (file, ios::app);
+    }
+    else {
+      _out[msg] = &cerr;
+    }
+  }
+  else {
+    _out[msg] = 0;
+  }
+}
+
+void
+MICO::Logger::Log (const char * msg, CORBA::Boolean onoff,
+		   const char * file)
+{
+  if (strcmp (msg, "Info") == 0) {
+    Log (Info, onoff, file);
+  }
+  else if (strcmp (msg, "Warning") == 0) {
+    Log (Warning, onoff, file);
+  }
+  else if (strcmp (msg, "Error") == 0) {
+    Log (Error, onoff, file);
+  }
+  else if (strcmp (msg, "GIOP") == 0) {
+    Log (GIOP, onoff, file);
+  }
+  else if (strcmp (msg, "IIOP") == 0) {
+    Log (IIOP, onoff, file);
+  }
+  else if (strcmp (msg, "Transport") == 0) {
+    Log (Transport, onoff, file);
+  }
+  else if (strcmp (msg, "All") == 0) {
+    Log (All, onoff, file);
+  }
+  else {
+    if (IsLogged (Warning)) {
+      Stream (Warning)
+	<< "Warning: No such debug level" << msg << " is available." << endl;
+    }
+  }
+}
+
+CORBA::Boolean
+MICO::Logger::IsLogged (MessageType msg)
+{
+  if (!_instance) {
+    return 0;
+  }
+  return (_out[msg] != 0);
+}
+
+ostream &
+MICO::Logger::Stream (MessageType msg)
+{
+  assert (_instance);
+  assert (_out[msg]);
+  return *_out[msg];
 }
 
 
-/***************************** MICODebug ******************************/
+/***************************** MICOGetOpt ******************************/
 
 
 MICOGetOpt::MICOGetOpt (const OptMap &opts)
