@@ -114,7 +114,7 @@ extract (CORBA::Any &a, char *types, int cond = 1, const char *val = "")
 	break;
     }
     case 't': {
-	char *o = 0;
+	const char *o = 0;
 	int ret = ((a >>= o) != cond || (cond && strcmp (o, val)));
 	if (ret)
 	    return "String";
@@ -124,7 +124,7 @@ extract (CORBA::Any &a, char *types, int cond = 1, const char *val = "")
 	assert (isdigit (types[1]));
 	CORBA::ULong len = strtol (types+1, &types, 10);
 	--types;
-	char *o = 0;
+	const char *o = 0;
 	int ret = ((a >>= CORBA::Any::to_string (o, len)) != cond ||
 		   (cond && strcmp (o, val)));
 	if (ret)
@@ -132,7 +132,7 @@ extract (CORBA::Any &a, char *types, int cond = 1, const char *val = "")
 	break;
     }
     case 'u': {
-	CORBA::WChar *o = 0;
+        const CORBA::WChar *o = 0;
 	int ret = ((a >>= o) != cond ||
 		   (cond && xwcscmp (o, (CORBA::WChar *)val)));
 	if (ret)
@@ -143,7 +143,7 @@ extract (CORBA::Any &a, char *types, int cond = 1, const char *val = "")
 	assert (isdigit (types[1]));
 	CORBA::ULong len = strtol (types+1, &types, 10);
 	--types;
-	CORBA::WChar *o = 0;
+	const CORBA::WChar *o = 0;
 	int ret = ((a >>= CORBA::Any::to_wstring (o, len)) != cond ||
 		   (cond && xwcscmp (o, (CORBA::WChar *)val)));
 	if (ret)
@@ -167,58 +167,58 @@ test_basic ()
 
     CORBA::Short s = 1234;
     a <<= s;
-    res += check (!extract (a, "slxfdD", 1, "1234") &&
-		  !extract (a, "slxfdD", 1, "1234") &&
-		  !extract (a, "SLXcwobtu", 0),
+    res += check (!extract (a, "s", 1, "1234") &&
+		  !extract (a, "s", 1, "1234") &&
+		  !extract (a, "SLXlxfdDcwobtu", 0),
 		  "Short");
 
     CORBA::UShort us = 4321;
     a <<= us;
-    res += check (!extract (a, "SlLxXfdD", 1, "4321") &&
-		  !extract (a, "SlLxXfdD", 1, "4321") &&
-		  !extract (a, "scwobtu", 0),
+    res += check (!extract (a, "S", 1, "4321") &&
+		  !extract (a, "S", 1, "4321") &&
+		  !extract (a, "scwobtulLxXfdD", 0),
 		  "UShort");
 
     CORBA::Long l = 12345678;
     a <<= l;
-    res += check (!extract (a, "lxdD", 1, "12345678") &&
-		  !extract (a, "lxdD", 1, "12345678") &&
-		  !extract (a, "sSLXfcwobtu", 0),
+    res += check (!extract (a, "l", 1, "12345678") &&
+		  !extract (a, "l", 1, "12345678") &&
+		  !extract (a, "sSLXfcwobtuxdD", 0),
 		  "Long");
 
     CORBA::LongLong ll = 12345678;
     a <<= ll;
-    res += check (!extract (a, "xD", 1, "12345678") &&
-		  !extract (a, "xD", 1, "12345678") &&
-		  !extract (a, "sSlLXfcwobtu", 0),
+    res += check (!extract (a, "x", 1, "12345678") &&
+		  !extract (a, "x", 1, "12345678") &&
+		  !extract (a, "sSlLXfcwobtuD", 0),
 		  "LongLong");
 
     CORBA::ULong ul = 87654321;
     a <<= ul;
-    res += check (!extract (a, "LxXdD", 1, "87654321") &&
-		  !extract (a, "LxXdD", 1, "87654321") &&
-		  !extract (a, "sSlfcwobtu", 0),
+    res += check (!extract (a, "L", 1, "87654321") &&
+		  !extract (a, "L", 1, "87654321") &&
+		  !extract (a, "sSlfcwobtuxXdD", 0),
 		  "ULong");
 
     CORBA::ULongLong ull = 87654321;
     a <<= ull;
-    res += check (!extract (a, "XD", 1, "87654321") &&
-		  !extract (a, "XD", 1, "87654321") &&
-		  !extract (a, "sSlLfdcwobtu", 0),
+    res += check (!extract (a, "X", 1, "87654321") &&
+		  !extract (a, "X", 1, "87654321") &&
+		  !extract (a, "sSlLfdcwobtuD", 0),
 		  "ULongLong");
 
     CORBA::Float f = 8.125;
     a <<= f;
-    res += check (!extract (a, "fdD", 1, "8.125") &&
-		  !extract (a, "fdD", 1, "8.125") &&
-		  !extract (a, "sSlLxXcwobtu", 0),
+    res += check (!extract (a, "f", 1, "8.125") &&
+		  !extract (a, "f", 1, "8.125") &&
+		  !extract (a, "sSlLxXcwobtudD", 0),
 		  "Float");
 
     CORBA::Double d = 16.0625;
     a <<= d;
-    res += check (!extract (a, "dD", 1, "16.0625") &&
-		  !extract (a, "dD", 1, "16.0625") &&
-		  !extract (a, "sSxXlLfcwobtu", 0),
+    res += check (!extract (a, "d", 1, "16.0625") &&
+		  !extract (a, "d", 1, "16.0625") &&
+		  !extract (a, "sSxXlLfcwobtuD", 0),
 		  "Double");
 
     CORBA::LongDouble dd = 16.0625;
@@ -355,7 +355,7 @@ CORBA::TypeCode_ptr _tc_LongSeq;
 void
 operator<<= (CORBA::Any &a, const LongSeq &seq)
 {
-    a.type (_tc_LongSeq);
+    a.set_type (_tc_LongSeq);
     a.seq_put_begin (seq.length());
     for (int i = 0; i < seq.length(); ++i)
         a <<= seq[i];
@@ -417,7 +417,7 @@ CORBA::TypeCode_ptr _tc_LongArrayArray;
 
 void operator<<= (CORBA::Any &a, const CORBA::Long arr[2][3])
 {
-    a.type (_tc_LongArrayArray);
+    a.set_type (_tc_LongArrayArray);
     a.array_put_begin();
     for (int i = 0; i < 2; ++i) {
         a.array_put_begin();
@@ -511,7 +511,7 @@ struct __mk_somestruct_typecode {
 void
 operator<<= (CORBA::Any &a, const SomeStruct &s)
 {
-    a.type (_tc_SomeStruct);
+    a.set_type (_tc_SomeStruct);
     a.struct_put_begin ();
     a <<= s.l;
     a <<= s.s;
@@ -556,9 +556,10 @@ test_struct ()
     CORBA::Any a2 (a.type(), (void *)a.value());
     int res = check (a == a2 && (a2 >>= s4) && s1 == s4, "struct 2");
 
-    CORBA::Any a3, a4;
+    CORBA::Any a3;
+    const CORBA::Any *a4;
     a3 <<= a;
-    res += check ((a3 >>= a4) && (a == a4), "struct 3");
+    res += check ((a3 >>= a4) && (a == *a4), "struct 3");
     return res;
 }
 
@@ -687,7 +688,7 @@ struct __mk_someunion_typecode {
 void
 operator<<= (CORBA::Any &a, const SomeUnion &u)
 {
-    a.type (_tc_SomeUnion);
+    a.set_type (_tc_SomeUnion);
     a.union_put_begin();
     a <<= u._d();
     switch (u._d()) {
@@ -914,7 +915,7 @@ operator<<= (CORBA::Any &a, const SomeRecStructSeq &s)
 	CORBA::TypeCode::create_sequence_tc (0,
 					     _tc_SomeRecStruct)->mk_constant();
 
-    a.type (_tc);
+    a.set_type (_tc);
     a.seq_put_begin (s.length());
     for (CORBA::ULong i = 0; i < s.length(); ++i) {
 	a <<= s[i];
@@ -939,7 +940,7 @@ operator>>= (const CORBA::Any &a, SomeRecStructSeq &s)
 void
 operator<<= (CORBA::Any &a, const SomeRecStruct &s)
 {
-    a.type (_tc_SomeRecStruct);
+    a.set_type (_tc_SomeRecStruct);
     a.struct_put_begin ();
     a <<= s.l;
     a <<= s.childs;
@@ -1093,7 +1094,7 @@ struct __mk_someenum_typecode {
 void
 operator<<= (CORBA::Any &a, const SomeEnum &en)
 {
-    a.type (_tc_SomeEnum);
+    a.set_type (_tc_SomeEnum);
     a.enum_put ((CORBA::ULong)en);
 }
 
@@ -1153,7 +1154,7 @@ CORBA::Boolean
 operator>>= (const CORBA::Any &a, SomeInterface_ptr &obj)
 {
     // XXX must *not* return a copy
-    CORBA::Object_ptr o;
+    CORBA::Object_var o;
     if (!(a >>= CORBA::Any::to_object (o)))
         return FALSE;
     obj = SomeInterface::narrow (o);
@@ -1262,7 +1263,7 @@ operator<<= (CORBA::Any &_a, Node *_v)
 {
     CORBA::Long _id;
 
-    _a.type (_tc_Node);
+    _a.set_type (_tc_Node);
     _a.value_put_begin (_id);
     _a <<= _v->l();
     _a.value_put_end (_id);
@@ -1351,7 +1352,7 @@ __any_insert (CORBA::Any &_a, LongList *_v, MapVBId &_m)
 {
     CORBA::Long _id;
 
-    _a.type (_tc_LongList);
+    _a.set_type (_tc_LongList);
 
     if (!_v || _m.count (_v)) {
 	_a.value_put_ref (_m[_v]);
