@@ -42,8 +42,8 @@ main (int argc, char *argv[])
     req->add_in_arg() <<= (CORBA::Long)1;
     req->add_in_arg() <<= (CORBA::Long)2;
     req->add_out_arg ();
-    req->arguments()->item(2)->value()->type (CORBA::_tc_string);
-    req->result()->value()->type (CORBA::_tc_long);
+    req->arguments()->item(2)->value()->set_type (CORBA::_tc_string);
+    req->result()->value()->set_type (CORBA::_tc_long);
 
     req->invoke ();
     if (req->env()->exception())
@@ -53,7 +53,7 @@ main (int argc, char *argv[])
     *req->result()->value() >>= z;
     cout << "result: " << z << endl;
 
-    char *str;
+    const char *str;
     *req->arguments()->item(2)->value() >>= str;
     cout << str << endl;
 
