@@ -7,31 +7,7 @@
 #include <CORBA-SMALL.h>
 #include <mico/template_impl.h>
 
-
-
-#define MICO_NO_TOPLEVEL_MODULES
-
-// Get module M
-#define MICO_MODULE_M
-MICO_NAMESPACE_DECL M {
-  #include "module.h"
-};
-#undef MICO_MODULE_M
-
-// Get module M2
-#define MICO_MODULE_M2
-MICO_NAMESPACE_DECL M2 {
-  #include "module.h"
-};
-#undef MICO_MODULE_M2
-
-// Get global definitions in module.h
-#define MICO_MODULE__GLOBAL
 #include "module.h"
-#undef MICO_MODULE__GLOBAL
-#undef MICO_NO_TOPLEVEL_MODULES
-
-
 
 class foo_impl : virtual public M::foo_skel
 {
